﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the AIReadInterval project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AIReadInterval.Utility;
using System.Text;

namespace AIReadInterval.ALLDAQ
{
    public class AnalogRange
    {
        private readonly double _offset;

        public uint RangeIndex
        {
            get; private set;
        }
        public string RangeText
        {
            get; private set;
        }
        public string UnitSymbol
        {
            get; private set;
        }
        public string UnitName
        {
            get; private set;
        }

        public uint DigitalMin
        {
            get;
        }
        public uint DigitalMax
        {
            get;
        }
        public double PhysicalMin
        {
            get;
        }
        public double PhysicalMax
        {
            get;
        }

        public AnalogRange(uint rangeIndex, uint physicalUnit, uint digitalMin, uint digitalMax, double physicalMin, double physicalMax)
        {
            RangeIndex = rangeIndex;
            DigitalMin = digitalMin;
            DigitalMax = digitalMax;
            PhysicalMin = physicalMin;
            PhysicalMax = physicalMax;
            PhysicalResolution = (physicalMax - physicalMin) / (digitalMax - digitalMin);
            _offset = physicalMin - digitalMin * PhysicalResolution;

            UnitSymbol = AdqCommon.GetPhysicalUnitSymbol(physicalUnit);
            UnitName = AdqCommon.GetPhysicalUnitName(physicalUnit);
            SetRangeUnitTexts(physicalMin, physicalMax);
        }

        public double PhysicalResolution
        {
            get;
        }

        public double DigitalToPhysical(uint digitalValue)
        {
            return digitalValue * PhysicalResolution + _offset;
        }

        private void SetRangeUnitTexts(double physicalMin, double physicalMax)
        {
            StringBuilder rangeTxt = new StringBuilder();

            rangeTxt.Append(RangeIndex);
            rangeTxt.Append(": ");
            rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMin));
            rangeTxt.Append(UnitSymbol);
            rangeTxt.Append("...");
            rangeTxt.Append(AdqCommon.FormatPhysicalValue(physicalMax));
            rangeTxt.Append(UnitSymbol);
            RangeText = rangeTxt.ToString();
        }
    }
}
