﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the AIReadInterval project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using AIReadInterval.ALLDAQ;
using AIReadInterval.Utility;
using ALLDAQ_DLL.Devices.Boards;
using ALLDAQ_DLL.Utilities;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace AIReadInterval
{
    public partial class FormMain : Form
    {
        private readonly Color _gridLineColor = Color.DarkSlateGray;
        private readonly Color _labelColor = Color.LightSkyBlue;
        private const int _rangeSubItemIndex = 1;
        private const int _idxChartTypeLine = 0;
        private const int _idxChartTypeLineAndMarkers = 1;

        private const int _idxXAxisLabelsTime = 0;

        private const int _idxGraphsToDisplayAllValues = 0;
        private const int _idxGraphsToDisplayDBU = 1;
        private const int _idxGraphsToDisplayDB = 2;
        private const int _idxGraphsToDisplayAllValuesAndDBUSeparated = 3;
        private const int _idxGraphsToDisplayAllValuesAndDBUTogether = 4;

        private const int _idxGraphicCoherentDB = 0;
        private const int _idxGraphicCoherentDBU = 1;

        private const int _idxColorSimilar = 0;
        private const int _idxColorRed = 1;

        private const string _txtTimeMs = "ms";
        private const string _txtTimeSec = "sec";
        private const string _txtTimeMin = "min";
        private const string _txtTimeHour = "h";

        private const string _txtFftWindowHann = "Hann";
        private const string _txtFftWindowHamming = "Hamming";
        private const string _txtFftWindowRectangular = "Rectangular";
        private const string _txtFftWindowBartlett = "Bartlett";
        private const string _txtFftWindowFlatTop = "Flat Top";
        private const string _txtFftWindowHarrisNutall = "Harris-Nutall";
        private const string _txtFftWindowBlackmanHarris7 = "Blackman-Harris 7-Term";

        private const string _txtAverageValues = "Average values";
        private const string _txtRmsValues = "RMS values";
        private const string _txtMinValues = "Minimum values";
        private const string _txtMaxValues = "Maximum values";
        private const string _txtPeek2PeekValues = "Peek-to-Peek values";
        private const string _txtMainFrequencies = "Main frequencies";
        private const string _txtFrequencyProportions = "Frequency proportions";
        private const string _txtThdValues = "THD values";
        private const string _txtThdrNValues = "THDr+N values";
        private const string _txtThdfNValues = "THDf+N values";
        private const string _txtRmsSpectrum = "RMS spectrum";
        private const string _txtMainFrequCoh = "Main frequencies Coherent";
        private const string _txtFrequPropCoh = "Frequency proportions Coherent";
        private const string _txtThdValuesCoh = "THD values Coherent";
        private const string _txtThdrNValuesCoh = "THDr+N values Coherent";
        private const string _txtThdfNValuesCoh = "THDf+N values Coherent";
        private const string _txtRmsSpectrumCoh = "RMS spectrum Coherent";

        private const string _fmtNames = "{0} [{1}]";
        private const string _lvNameAvg = "Average";
        private const string _lvNameRms = "RMS";
        private const string _lvNameMin = "Min value";
        private const string _lvNameMax = "Max value";
        private const string _lvNameP2P = "Peak-to-Peak";

        private const string _lvNameThd = "THD [%]";
        private const string _lvNameThdrPlusN = "THDr+N [%]";
        private const string _lvNameThdfPlusN = "THDf+N [%]";
        private const string _lvNameMainFrequency = "Frequency [Hz]";
        private const string _lvNameFrequencyProportion = "Freq. proportion [%]";
        private const string _lvNameRmsSpectrum = "RMS spectrum";

        private const string _lvNameThdCoherent = "THD Coherent [%]";
        private const string _lvNameThdrPlusNCoherent = "THDr+N Coherent [%]";
        private const string _lvNameThdfPlusNCoherent = "THDf+N Coherent [%]";
        private const string _lvNameMainFrequencyCoherent = "Frequency Coherent [Hz]";
        private const string _lvNameFrequencyProportionCoherent = "Freq. proportion Coherent [%]";
        private const string _lvNameRmsSpectrumCoherent = "RMS spectrum Coherent";

        private const string _titleAllValues = "All Values";
        private const string _titleAllValuesAndDBU = "All Values and FFT";
        private const string _titleDBU = "dBu";
        private const string _titleDB = "dB";
        private const string _titleFrequency = "Frequency [Hz]";
        private const string _titleFFTINDBU = "FFT [dBu]";
        private const string _titleFFTINDB = "FFT [dB]";
        private const string _titleFFTCoherentDBU = "FFT coherent [dBu]";
        private const string _titleFFTCoherentDB = "FFT coherent [dB]";

        private const string _xAxisLabelTime = "Time";
        private const string _xAxisLabelFrameIndex = "Frame index";

        private const string _SeriesConstAllValues = "AllValues";
        private const string _SeriesConstDBU = "DBU";
        private const string _SeriesConstFFT = "FFT";
        private const string _SeriesConstCalculated = "Calculated";

        private ModuleLocator _moduleLocator;
        private readonly List<AnalogInputModule> _aiModules = new List<AnalogInputModule>();
        private AnalogInputModule _selectedAiModule;
        private string _selectedUnitSymbol;
        private int _selectedModuleItemIndex = -1;
        private bool _moduleIsBusy = false;
        private ListViewHitTestInfo _hitinfoChannelList;
        private CsvFileWriter _csvFileWriter;
        private SignalUtility _signalUtility;

        private DateTime _startDateTime;
        private string _csvFilesFolder;
        private string _fmtStringDigitalPhysical;
        private string _fmtStringPhysValues;
        private string _fmtStringFftValues;
        private string _fmtStringDBValues;
        private readonly string _fmtStringDBIndex = "{0:0}";

        private int _intervalsRead;
        private int _maxIntervalsToDisplay;
        private int _stopAfterIntervals;
        private bool _isFormClosing;
        private bool _isIntervalReading;
        private bool _updateFftChartNeeded;

        private ListViewItem _lvItemAvgValue;
        private ListViewItem _lvItemRmsValue;
        private ListViewItem _lvItemMinValue;
        private ListViewItem _lvItemMaxValue;
        private ListViewItem _lvItemP2PValue;

        private ListViewItem _lvItemThdValue;
        private ListViewItem _lvItemThdrPlusNValue;
        private ListViewItem _lvItemThdfPlusNValue;
        private ListViewItem _lvItemMainFrequencyValue;
        private ListViewItem _lvItemFrequencyProportionValue;
        private ListViewItem _lvItemRmsValueSpectrumValue;

        private ListViewItem _lvItemThdCoherentValue;
        private ListViewItem _lvItemThdrPlusNCoherentValue;
        private ListViewItem _lvItemThdfPlusNCoherentValue;
        private ListViewItem _lvItemMainFrequencyCoherentValue;
        private ListViewItem _lvItemFrequencyProportionCoherentValue;
        private ListViewItem _lvItemRmsValueSpectrumCoherentValue;

        private readonly ComboBox _coBoxRanges = new ComboBox();

        public FormMain()
        {
            InitializeComponent();
            AdqCommon.SetDoubleBuffered(lvChannelList);
            AdqCommon.SetDoubleBuffered(lvValues);
            InitRangeComboBox();
            coBoxChartTypeUserConfigurable.SelectedIndex = _idxChartTypeLine;
            coBoxXAxisLabelsUserConfigurable.SelectedIndex = _idxXAxisLabelsTime;
            coBoxChartGraphicToDisplay.SelectedIndex = _idxGraphsToDisplayAllValuesAndDBUSeparated;

            coBoxChartTypeGraphicCoherent.SelectedIndex = _idxChartTypeLine;
            coBoxChartGraphicCoherent.SelectedIndex = _idxGraphicCoherentDBU;

            coBoxChartTypeCalculated.SelectedIndex = _idxChartTypeLine;
            coBoxXAxisLabelsCalcValues.SelectedIndex = _idxXAxisLabelsTime;
            coBoxCalculatedType.SelectedIndex = 0;

            coBoxColor.SelectedIndex = _idxColorRed;

            textBoxOverlap.Text = AdqCommon.DefaultWindowOverlapping.ToString();
        }

        private void FormMain_Load(object sender, EventArgs e)
        {
            EnableUI(false);
            try
            {
                InitTimeValueComboBox();
                InitWindowFunctionComboBox();
                InitCalculatedValuesComboBox();
                InitCsvFilesFolder();
                InitValuesListView();
                SetFormatStrings((int)numUDDecimalPlaces.Value);
                SetFormatStringFft((int)numUDDecimalPlacesFft.Value);
                _moduleLocator = new ModuleLocator(_aiModules, OnModuleInfoTextChanged);
                _moduleLocator.FindModules();
                FillModuleCombobox();
                SelectFirstComboboxItem(coBoxModules);
                textBoxInfo.AppendText(string.Format("Number of analog input modules found: {0}" + Environment.NewLine, _aiModules.Count));
            }
            catch (DllNotFoundException ex)
            {
                textBoxInfo.AppendText("The device driver is not properly installed." + Environment.NewLine);
                textBoxInfo.AppendText(ex.Message + Environment.NewLine);
            }
        }

        private void InitRangeComboBox()
        {
            _coBoxRanges.DropDownStyle = ComboBoxStyle.DropDownList;
            _coBoxRanges.Parent = lvChannelList;
            _coBoxRanges.Visible = false;
            _coBoxRanges.DropDownClosed += CoBoxRanges_DropDownClosed;
            _coBoxRanges.Leave += CoBoxRanges_Leave;
            _coBoxRanges.KeyPress += CoBoxRanges_KeyPress;
        }

        private void InitCsvFilesFolder()
        {
            _csvFilesFolder = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments) + @"\AIReadInterval";
            textBoxCsvFileFolder.Text = _csvFilesFolder;
        }

        private void FillModuleCombobox()
        {
            coBoxModules.Items.Clear();
            foreach (AnalogInputModule aiModule in _aiModules)
            {
                coBoxModules.Items.Add(aiModule.ModuleName);
            }
        }

        private void InitValuesListView()
        {
            _lvItemAvgValue = AddListViewItem(_lvNameAvg);
            _lvItemRmsValue = AddListViewItem(_lvNameRms);
            _lvItemMinValue = AddListViewItem(_lvNameMin);
            _lvItemMaxValue = AddListViewItem(_lvNameMax);
            _lvItemP2PValue = AddListViewItem(_lvNameP2P);
            _lvItemThdValue = AddListViewItem(_lvNameThd);
            _lvItemThdrPlusNValue = AddListViewItem(_lvNameThdrPlusN);
            _lvItemThdfPlusNValue = AddListViewItem(_lvNameThdfPlusN);
            _lvItemMainFrequencyValue = AddListViewItem(_lvNameMainFrequency);
            _lvItemFrequencyProportionValue = AddListViewItem(_lvNameFrequencyProportion);
            _lvItemRmsValueSpectrumValue = AddListViewItem(_lvNameRmsSpectrum);
            _lvItemThdCoherentValue = AddListViewItem(_lvNameThdCoherent);
            _lvItemThdrPlusNCoherentValue = AddListViewItem(_lvNameThdrPlusNCoherent);
            _lvItemThdfPlusNCoherentValue = AddListViewItem(_lvNameThdfPlusNCoherent);
            _lvItemMainFrequencyCoherentValue = AddListViewItem(_lvNameMainFrequencyCoherent);
            _lvItemFrequencyProportionCoherentValue = AddListViewItem(_lvNameFrequencyProportionCoherent);
            _lvItemRmsValueSpectrumCoherentValue = AddListViewItem(_lvNameRmsSpectrumCoherent);
        }

        private ListViewItem AddListViewItem(string text)
        {
            var lvItem = new ListViewItem(text);
            lvValues.Items.Add(lvItem);
            return lvItem;
        }

        private void OnModuleInfoTextChanged(object sender, InfoTextChangedEventArgs e)
        {
            textBoxInfo.AppendText(e.InfoText + Environment.NewLine);
        }

        private void OnPropertyChanged(object sender, InfoTextChangedEventArgs e)
        {
            if (e.InfoText == "CsvFileNameAllValues")
            {
                textBoxCsvFileNameAllValues.Text = Path.GetFileName(_csvFileWriter.CsvFileNameAllValues);
            }
            else if (e.InfoText == "CsvFileNameCalculatedValues")
            {
                textBoxCsvFileNameCalculated.Text = Path.GetFileName(_csvFileWriter.CsvFileNameCalculatedValues);
            }
            else if (e.InfoText == "CsvFileNameFftValues")
            {
                textBoxCsvFileNameFftValues.Text = Path.GetFileName(_csvFileWriter.CsvFileNameFftValues);
            }
            else if (e.InfoText == "CsvFileNameFftCoherentValues")
            {
                textBoxCsvFileNameFftCoherentValues.Text = Path.GetFileName(_csvFileWriter.CsvFileNameFftCoherentValues);
            }
        }

        private void DisplayErrorText(string functionName, uint adqError)
        {
            textBoxInfo.AppendText(
                string.Format("Function {0} returned Error number: 0x{1:X}" + Environment.NewLine,
                    functionName,
                    adqError));
        }

        private void SelectFirstComboboxItem(ComboBox coBox)
        {
            if (coBox.Items.Count > 0)
            {
                coBox.SelectedIndex = 0;
            }
        }

        private void CoBoxModules_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (coBoxModules.SelectedIndex == _selectedModuleItemIndex)
            {
                return;
            }

            _selectedModuleItemIndex = coBoxModules.SelectedIndex;
            ClearDisplayedValues();
            EnableUI(coBoxModules.SelectedIndex >= 0);
            btnStartStop.Enabled = coBoxModules.SelectedIndex >= 0;
            if (_selectedAiModule != null)
            {
                _selectedAiModule.ModuleInfoTextChanged -= OnModuleInfoTextChanged;
                _selectedAiModule.ClearResources();
            }
            if (coBoxModules.SelectedIndex < 0)
            {
                _selectedAiModule = null;
                textBoxSampleFrequency.Text = string.Empty;
                _selectedUnitSymbol = string.Empty;
            }
            else
            {
                _selectedAiModule = _aiModules[coBoxModules.SelectedIndex];
                _selectedAiModule.ModuleInfoTextChanged += OnModuleInfoTextChanged;
                coBoxNumOfFrames.Text = _selectedAiModule.NumberOfFrames.ToString();
                textBoxSampleFrequency.Text = string.Format("{0}", _selectedAiModule.DesiredSampleFrequency);
                _selectedUnitSymbol = _selectedAiModule.DefaultRange.UnitSymbol;
            }
            textBoxActualFrequency.Text = string.Empty;
            FillChannelsAndRanges(_selectedAiModule);
            UpdateToolstripMenuItems(_selectedAiModule);
            RemoveChannelColumns(lvValues);
        }

        private void FillChannelsAndRanges(AnalogInputModule aiModule)
        {
            FillChannels(aiModule);
            FillRanges(aiModule);
        }

        private void FillChannels(AnalogInputModule aiModule)
        {
            lvChannelList.Items.Clear();
            if (aiModule == null)
            {
                return;
            }

            for (uint i = 0; i < aiModule.NumberOfChannels; i++)
            {
                ListViewItem lvItem = lvChannelList.Items.Add("Channel " + i.ToString());
                lvItem.SubItems.Add(aiModule.DefaultRange.RangeText);
                lvItem.Tag = aiModule.DefaultRange;
                lvItem.BackColor = Color.Black;
                lvItem.ForeColor = AdqCommon.ChannelColors[i % AdqCommon.ChannelColors.Length];
            }
            CheckFirstChannels(1);
        }

        private void CheckFirstChannels(int count)
        {
            int idx = 0;
            while (idx < lvChannelList.Items.Count)
            {
                lvChannelList.Items[idx].Checked = idx < count;
                idx++;
            }
        }

        private void FillRanges(AnalogInputModule aiModule)
        {
            _coBoxRanges.Items.Clear();
            if (aiModule == null)
            {
                return;
            }

            foreach (AnalogRange range in aiModule.Ranges)
            {
                _coBoxRanges.Items.Add(new RangeComboBoxItem(range));
            }
            SelectFirstComboboxItem(_coBoxRanges);
        }

        private void UpdateToolstripMenuItems(AnalogInputModule aiModule)
        {
            tsMnuItemSetRange.DropDownItems.Clear();
            if (aiModule != null && aiModule.NumberOfRanges > 1)
            {
                foreach (AnalogRange range in aiModule.Ranges)
                {
                    var item = new ToolStripMenuItem(range.RangeText, null, TsMnuItemSetRange_Click)
                    {
                        Tag = range
                    };
                    tsMnuItemSetRange.DropDownItems.Add(item);
                }
            }
            else
            {
                var item = new ToolStripMenuItem("No selection available")
                {
                    Enabled = false
                };
                tsMnuItemSetRange.DropDownItems.Add(item);
            }
        }

        private void TsMnuItemSetRange_Click(object sender, EventArgs e)
        {
            if (sender is ToolStripMenuItem)
            {
                AnalogRange range = (AnalogRange)((ToolStripMenuItem)sender).Tag;
                SetChannelsToRange(range);
            }
        }

        private void SetChannelsToRange(AnalogRange range)
        {
            foreach (ListViewItem lvItem in lvChannelList.Items)
            {
                lvItem.Tag = range;
                lvItem.SubItems[_rangeSubItemIndex].Text = range.RangeText;
            }
        }

        private void LvChannelList_MouseDown(object sender, MouseEventArgs e)
        {
            _hitinfoChannelList = lvChannelList.HitTest(e.X, e.Y);
        }

        private void LvChannelList_MouseUp(object sender, MouseEventArgs e)
        {
            if (e.Button != MouseButtons.Left || _isIntervalReading || _selectedAiModule == null || _selectedAiModule.Ranges.Count <= 1 ||
                _hitinfoChannelList == null || _hitinfoChannelList.SubItem == null)
            {
                return;
            }

            ListViewHitTestInfo hitInfo = lvChannelList.HitTest(e.X, e.Y);
            if (_hitinfoChannelList.Item == hitInfo.Item && _hitinfoChannelList.Item.SubItems[_rangeSubItemIndex] == hitInfo.SubItem)
            {
                AnalogRange range = (AnalogRange)_hitinfoChannelList.Item.Tag;
                _coBoxRanges.Bounds = hitInfo.SubItem.Bounds;
                _coBoxRanges.SelectedIndex = (int)range.RangeIndex;
                _coBoxRanges.Visible = true;
                _coBoxRanges.BringToFront();
                _coBoxRanges.Focus();
            }
        }

        private void ContextMnuStripChanList_Opening(object sender, CancelEventArgs e)
        {
            if (_coBoxRanges.Visible)
            {
                _coBoxRanges.Visible = false;
            }
        }

        private void CoBoxRanges_KeyPress(object sender, KeyPressEventArgs e)
        {
            if (e.KeyChar == (char)Keys.Return)
            {
                UpdateListViewItemChannelList();
            }
            else if (e.KeyChar == (char)Keys.Escape)
            {
                AnalogRange range = (AnalogRange)_hitinfoChannelList.Item.Tag;
                _coBoxRanges.SelectedIndex = (int)range.RangeIndex;
                _coBoxRanges.Visible = false;
            }
        }

        private void CoBoxRanges_DropDownClosed(object sender, EventArgs e)
        {
            UpdateListViewItemChannelList();
        }

        private void CoBoxRanges_Leave(object sender, EventArgs e)
        {
            UpdateListViewItemChannelList();
        }

        private void UpdateListViewItemChannelList()
        {
            if (_hitinfoChannelList == null || _hitinfoChannelList.SubItem == null || _coBoxRanges.SelectedItem == null)
            {
                return;
            }

            RangeComboBoxItem rangeItem = (RangeComboBoxItem)_coBoxRanges.SelectedItem;
            _hitinfoChannelList.Item.Tag = rangeItem.Range;
            _hitinfoChannelList.Item.SubItems[_rangeSubItemIndex].Text = rangeItem.ToString();
            _coBoxRanges.Visible = false;
            lvChannelList.Focus();
        }

        private void EnableUI(bool enabled)
        {
            coBoxChartGraphicCoherent.Enabled = enabled;
            coBoxChartTypeGraphicCoherent.Enabled = enabled;
            chkBoxShowGraphicCoherent.Enabled = enabled;
            coBoxModules.Enabled = enabled;
            textBoxSampleFrequency.Enabled = enabled;
            coBoxNumOfFrames.Enabled = enabled;
            textBoxTime.Enabled = enabled;
            coBoxChartTypeCalculated.Enabled = enabled;
            coBoxTimeUnit.Enabled = enabled;
            coBoxXAxisLabelsCalcValues.Enabled = enabled;
            coBoxWindowFunction.Enabled = enabled;
            chkBoxShowGraphicUserConfigurable.Enabled = enabled;
            coBoxChartTypeUserConfigurable.Enabled = enabled;
            coBoxXAxisLabelsUserConfigurable.Enabled = enabled;
            coBoxChartGraphicToDisplay.Enabled = enabled;
            chkBoxWriteCsvFileCalculated.Enabled = enabled;
            chkBoxWriteCsvFileAllValues.Enabled = enabled;
            chkBoxWriteFftValues.Enabled = enabled;
            chkBoxWriteFftCoherentValues.Enabled = enabled;
            chkBoxShowGraphicCalculated.Enabled = enabled;
            chkBoxDisplayFftValues.Enabled = enabled;
            chkBoxDisplayFftCoherentValues.Enabled = enabled;
            coBoxCalculatedType.Enabled = enabled;
            numUDMaxIntervalsToDisplay.Enabled = enabled;
            numUDStopAfterIntervals.Enabled = enabled;
            numUDDecimalPlaces.Enabled = enabled;
            numUDDecimalPlacesFft.Enabled = enabled;
            textBoxOverlap.Enabled = enabled;
            textBoxSineSignalFrequency.Enabled = enabled;
            textBoxFrequIgnoreBelow.Enabled = enabled;
            textBoxFrequIgnoreAbove.Enabled = enabled;
            btnSetConfig.Enabled = enabled;
            btnReset.Enabled = enabled;
            tsMnuItemSelectAll.Enabled = enabled;
            tsMnuItemSelectNone.Enabled = enabled;
            tsMnuItemInvertSelection.Enabled = enabled;
            tsMnuItemSetRange.Enabled = enabled;
            btnClear.Enabled = enabled;
            btnSelectCsvFolder.Enabled = enabled;
            textBoxCsvFileFolder.Enabled = enabled;
            btnSaveInfo.Enabled = enabled;
            tsMenuItemSaveGraphic.Enabled = enabled;
            btnCalculateFft.Enabled = enabled;
            btnSaveLastInterval.Enabled = enabled;
            coBoxColor.Enabled = enabled;
        }

        private void CoBoxNumOfFrames_TextChanged(object sender, EventArgs e)
        {
            if (_selectedAiModule == null)
            {
                return;
            }

            _selectedAiModule.NumberOfFrames = ParseNumberOfFrames(coBoxNumOfFrames.Text);
            UpdateNumberOfValues();
            EnableFftUi(false);
        }

        private void UpdateNumberOfValues()
        {
            if (_selectedAiModule == null)
            {
                textBoxNumberOfValues.Text = string.Empty;
                return;
            }
            textBoxNumberOfValues.Text = (_selectedAiModule.NumberOfFrames * _selectedAiModule.FrameLength).ToString();
        }

        private void EnableFftUi(bool enabled)
        {
            btnCalculateFft.Enabled = enabled;
        }

        private void TxtBoxSampleFrequency_TextChanged(object sender, EventArgs e)
        {
            if (_selectedAiModule == null)
            {
                return;
            }

            _selectedAiModule.DesiredSampleFrequency = ParseFrequency(textBoxSampleFrequency.Text);
        }

        private double ParseFrequency(string sampleFrequency)
        {
            bool isValid = double.TryParse(sampleFrequency, out double actualFrequency);
            return (isValid && actualFrequency > 0) ? actualFrequency : 0.0;
        }

        private uint ParseNumberOfFrames(string numberOfFrames)
        {
            bool isValid = uint.TryParse(numberOfFrames, out uint numValues);
            return isValid ? numValues : 0;
        }

        private void BtnReset_Click(object sender, EventArgs e)
        {
            if (_selectedAiModule == null)
            {
                return;
            }

            _selectedAiModule.Reset();
        }

        private void BtnSetConfig_Click(object sender, EventArgs e)
        {
            try
            {
                SetAiModuleConfiguration();
            }
            catch (Exception ex)
            {
                DisplayExceptionMessage(ex);
            }
        }

        private void SetAiModuleConfiguration()
        {
            UpdateChannelSettings();
            _selectedAiModule.ConfigureModule();
            UpdateUI();
        }

        private void UpdateUI()
        {
            textBoxActualFrequency.Text = AdqCommon.FormatFrequency(_selectedAiModule.ActualSampleFrequency);
            double seconds = _selectedAiModule.GetSecondsToWait();
            lblDuration.Text = AdqCommon.FormatTimeToWait(seconds);
        }

        private void BtnCalculate_Click(object sender, EventArgs e)
        {
            if (_selectedAiModule == null)
            {
                return;
            }

            if (_selectedAiModule.AdqError != AdqErrors.ADQ_ERROR_NONE)
            {
                MessageBox.Show(string.Format("The API-function adqAIReadBuffer returned error 0x{0:X}.\n"
                    + "There are no values acquired.\nThe FFT cannot be calculated.", _selectedAiModule.AdqError),
                    "AIReadInterval", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            try
            {
                InitFftSettings();
                _signalUtility.CalculateSignalValues();
                if (_updateFftChartNeeded)
                {
                    _updateFftChartNeeded = false;

                    InitChartCoherentValues(chartFftCoherent, _selectedAiModule);
                    InitChartUserConfigurable(_selectedAiModule);
                }
                DisplayCoherentAsChart(chartFftCoherent, _signalUtility.FrameLength, _signalUtility.FrequencySpectrumsCoherent,
                    _signalUtility.FrequencySpectrumsCoherentDBU, _signalUtility.SpectrumSize, _signalUtility.FrequencyStepCoherent);
                DisplayUserConfigurableAsChart(_selectedAiModule, chartUserConfigurable, _signalUtility.FrameLength, _signalUtility.FrequencySpectrumsDBU,
                    _signalUtility.FrequencySpectrums, _signalUtility.SpectrumSize, _signalUtility.FrequencyStep);
                DisplayFftValuesAsText();
            }
            catch (Exception ex)
            {
                AddInfoTextLine(ex.Message);
            }
        }

        private void CalculateFft()
        {
            _signalUtility.CalculateSignalValues();
        }

        private void InitSignalUtility()
        {
            if (_signalUtility == null)
            {
                _signalUtility = new SignalUtility();
                _signalUtility.OnInfoTextChanged += OnModuleInfoTextChanged;
            }
        }

        private void InitFftSettings()
        {
            uint spectrumSize = uint.Parse(coBoxSpectrumSize.Text);
            double sineSignalFrequency = double.Parse(textBoxSineSignalFrequency.Text);
            double overlap = double.Parse(textBoxOverlap.Text);
            double frequencyIgnoreBelow = double.Parse(textBoxFrequIgnoreBelow.Text);
            double frequencyIgnoreAbove = double.Parse(textBoxFrequIgnoreAbove.Text);
            _signalUtility.InitFftSettings(_selectedAiModule, spectrumSize, sineSignalFrequency, ((StringValueItem<uint>)coBoxWindowFunction.SelectedItem),
                overlap, frequencyIgnoreBelow, frequencyIgnoreAbove);
        }

        private void BtnStartStop_Click(object sender, EventArgs e)
        {
            if (_selectedAiModule == null)
            {
                return;
            }

            if (_isIntervalReading)
            {
                StopReading();
                if (_moduleIsBusy)
                {
                    _selectedAiModule.Stop();
                }
                AddInfoTextLine(string.Format("Stopped by user. Read intervals: {0}", _intervalsRead));
                EnableChartsTooltipFunctions();
                return;
            }
            ClearDisplayedValues();
            EnableUI(false);
            btnStartStop.Text = "Stop";
            try
            {
                InitSignalUtility();
                SetCalculatedValueType();
                SetAiModuleConfiguration();
                InitFftSettings();
                SetCsvProperties();
                GenerateTimeAxisLabels(coBoxXAxisLabelsUserConfigurable, _selectedAiModule.NumberOfFrames, ref _xAxisLabelsUserConfigurable);
                BeginIntervalAiRead();
                AddTextToCombobox(coBoxNumOfFrames, coBoxNumOfFrames.Text);
            }
            catch (Exception ex)
            {
                _moduleIsBusy = false;
                _isIntervalReading = false;
                EnableUI(true);
                btnStartStop.Text = "Start";
                DisplayExceptionMessage(ex);
            }
        }

        private void DisplayExceptionMessage(Exception ex)
        {
            textBoxInfo.AppendText(ex.Message + Environment.NewLine + ex.StackTrace);
            tabControl1.SelectedTab = tpInfo;
        }

        private void UpdateChannelSettings()
        {
            _selectedAiModule.ChannelSettings.Clear();
            foreach (ListViewItem item in lvChannelList.CheckedItems)
            {
                AnalogRange range = (AnalogRange)item.Tag;
                _selectedAiModule.ChannelSettings.Add(new ChannelSetting((uint)item.Index, range));
            }
            _selectedAiModule.UpdateChannelList();
        }

        private void SetCalculatedValueType()
        {
            _signalUtility.SignalValuesType = (((StringValueItem<SignalValuesType>)(coBoxCalculatedType.SelectedItem)).Value);
        }

        private string GetInfoTextFileName(AnalogInputModule aiModule, string fileExt)
        {
            string s = string.Format("{0}, Board {1} Info {2}{3}",
                aiModule.ModelName,
                aiModule.BoardIndex,
                DateTime.Now.ToString("yyyy-MM-dd HH.mm.ss"),
                fileExt);
            return s;
        }

        private void SetCsvProperties()
        {
            if (CsvWritingEnabled())
            {
                if (_csvFileWriter == null)
                {
                    _csvFileWriter = new CsvFileWriter();
                    _csvFileWriter.PropertyChanged += OnPropertyChanged;
                }
                var csvProperties = new CsvProperties()
                {
                    AiModule = _selectedAiModule,
                    SignalUtility = _signalUtility,
                    FolderName = _csvFilesFolder,
                    SaveAllValues = chkBoxWriteCsvFileAllValues.Checked,
                    SaveCalculatedValues = chkBoxWriteCsvFileCalculated.Checked,
                    SaveFftValues = chkBoxWriteFftValues.Checked,
                    SaveFftCoherentValues = chkBoxWriteFftCoherentValues.Checked,
                    FormatStringDigitalAndPhysical = _fmtStringDigitalPhysical,
                    FormatStringPhysValues = _fmtStringPhysValues,
                    FormatStringFftValues = _fmtStringFftValues
                };
                _csvFileWriter.UpdateCsvFileWriter(csvProperties);
            }
            else
            {
                textBoxCsvFileNameAllValues.Text = string.Empty;
                textBoxCsvFileNameCalculated.Text = string.Empty;
                textBoxCsvFileNameFftValues.Text = string.Empty;
                textBoxCsvFileNameFftCoherentValues.Text = string.Empty;
            }
        }

        private bool CsvWritingEnabled()
        {
            return chkBoxWriteCsvFileAllValues.Checked
                || chkBoxWriteCsvFileCalculated.Checked
                || chkBoxWriteFftValues.Checked
                || chkBoxWriteFftCoherentValues.Checked;
        }

        private int GetTimeIntervalMilliseconds()
        {
            int interval = int.Parse(textBoxTime.Text);
            return interval * ((StringValueItem<int>)coBoxTimeUnit.SelectedItem).Value;
        }

        private void SetFormatStrings(int decimalPlaces)
        {
            string decStr = new string('0', decimalPlaces);
            _fmtStringDigitalPhysical = ";{0};{1:0." + decStr + "}";
            _fmtStringPhysValues = "{0:0." + decStr + "}";
            _fmtStringDBValues = "{0:0." + decStr + "}";
        }

        private void SetFormatStringFft(int decimalPlacesFft)
        {
            _fmtStringFftValues = "{0:0." + new string('0', decimalPlacesFft) + "}";
        }

        private void NumUDDecPlaces_ValueChanged(object sender, EventArgs e)
        {
            SetFormatStrings((int)numUDDecimalPlaces.Value);
        }

        private void NumUDDecimalPlacesFft_ValueChanged(object sender, EventArgs e)
        {
            SetFormatStringFft((int)numUDDecimalPlacesFft.Value);
        }

        private void BeginIntervalAiRead()
        {
            int timerInterval = GetTimeIntervalMilliseconds();
            if (timerInterval <= 0)
            {
                throw new ApplicationException("The timer interval must be greater than 0.");
            }
            timerPause.Interval = timerInterval;
            _intervalsRead = 0;
            _stopAfterIntervals = (int)numUDStopAfterIntervals.Value;
            _maxIntervalsToDisplay = (int)numUDMaxIntervalsToDisplay.Value;
            if (CsvWritingEnabled())
            {
                _csvFileWriter.WriteCsvFileHeader();
            }
            InitChartUserConfigurable(_selectedAiModule);
            InitChartCalculatedValues(_selectedAiModule);
            InitChartCoherentValues(chartFftCoherent, _selectedAiModule);
            UpdateLvValuesNameColumn();
            UpdateLvValuesColumns(_selectedAiModule);
            AddIntervalInfo(_selectedAiModule, chkBoxWriteCsvFileAllValues.Checked,
                chkBoxWriteCsvFileCalculated.Checked, chkBoxWriteFftValues.Checked,
                chkBoxDisplayFftCoherentValues.Checked, textBoxTime.Text + coBoxTimeUnit.Text);
            AddStartInfo();
            textBoxProgress.Text = " Please wait...";
            SetProgressBarStyle(true);
            _isIntervalReading = true;
            _startDateTime = DateTime.Now;
            StartAiRead();
        }

        private void SetProgressBarStyle(bool start)
        {
            pBarIntervals.Value = 0;
            pBarIntervals.Style = (_stopAfterIntervals == 0 && start) ? ProgressBarStyle.Marquee : ProgressBarStyle.Continuous;
        }

        private void AddStartInfo()
        {
            string startInfo = "Start reading";
            startInfo += _stopAfterIntervals == 0 ? "..." : string.Format(", stop after {0} {1}.", _stopAfterIntervals, GetIntervalsText());
            AddInfoTextLine(startInfo);
        }

        private string GetIntervalsText()
        {
            return _stopAfterIntervals == 1 ? "interval" : "intervals";
        }

        private void UpdateLvValuesNameColumn()
        {
            _lvItemAvgValue.Text = string.Format(_fmtNames, _lvNameAvg, _selectedUnitSymbol);
            _lvItemRmsValue.Text = string.Format(_fmtNames, _lvNameRms, _selectedUnitSymbol);
            _lvItemMinValue.Text = string.Format(_fmtNames, _lvNameMin, _selectedUnitSymbol);
            _lvItemMaxValue.Text = string.Format(_fmtNames, _lvNameMax, _selectedUnitSymbol);
            _lvItemP2PValue.Text = string.Format(_fmtNames, _lvNameP2P, _selectedUnitSymbol);
            _lvItemRmsValueSpectrumValue.Text = string.Format(_fmtNames, _lvNameRmsSpectrum, _selectedUnitSymbol);
            _lvItemRmsValueSpectrumCoherentValue.Text = string.Format(_fmtNames, _lvNameRmsSpectrumCoherent, _selectedUnitSymbol);
        }

        private void UpdateLvValuesColumns(AnalogInputModule aiModule)
        {
            RemoveChannelColumns(lvValues);
            AddCheckedChannelColumns(lvValues, aiModule);
            InitListViewSubItems(lvValues, aiModule);
        }

        private void RemoveChannelColumns(ListView listView)
        {
            for (int i = listView.Columns.Count - 1; i > 0; i--)
            {
                listView.Columns.RemoveAt(i);
            }
        }

        private void AddCheckedChannelColumns(ListView listView, AnalogInputModule aiModule)
        {
            int columnWidth = 85;
            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                listView.Columns.Add("Channel " + chanSetting.ChannelIndex.ToString(), columnWidth);
            }
        }

        private void InitListViewSubItems(ListView listView, AnalogInputModule aiModule)
        {
            for (int i = 0; i < listView.Items.Count; i++)
            {
                var subStrings = new string[aiModule.FrameLength];
                for (int j = 0; j < aiModule.FrameLength; j++)
                {
                    subStrings[j] = string.Empty;
                }
                listView.Items[i].SubItems.AddRange(subStrings);
            }
        }

        private void AddIntervalInfo(AnalogInputModule aiModule, bool saveCsvAllValues,
            bool saveCsvAverageValues, bool saveCsvFftValues, bool saveCsvFftCoherentValues, string intervalPauseTime)
        {
            var sb = new StringBuilder();
            sb.Append("Interval read: ");
            sb.AppendLine(aiModule.ModuleName);
            sb.AppendFormat("Serial number: 0x{0:X8}", aiModule.SerialNumber);
            sb.AppendLine();
            sb.Append("Number of selected channels: ");
            sb.Append(aiModule.FrameLength);
            sb.AppendLine();
            sb.Append(GetSelectedChannelsText(aiModule));
            sb.Append("Sample frequency: ");
            sb.AppendLine(AdqCommon.FormatFrequency(aiModule.ActualSampleFrequency));
            sb.Append("Number of frames: ");
            sb.Append(aiModule.NumberOfFrames);
            sb.AppendLine();
            sb.Append("Pause before next read operation: ");
            sb.Append(intervalPauseTime);
            sb.AppendLine();
            if (saveCsvFftValues)
            {
                sb.Append("FFT window function: ");
                sb.AppendLine(_signalUtility.WindowFunctionItem.ToString());
                sb.Append("FFT spectrum size: ");
                sb.Append(_signalUtility.SpectrumSize);
                sb.AppendLine();
            }
            if (_csvFileWriter != null)
            {
                if (saveCsvAllValues)
                {
                    sb.Append("File name all values: ");
                    sb.AppendLine(_csvFileWriter.CsvFileNameAllValues);
                    sb.Append("  - Number decimal separator: ");
                    sb.AppendLine(_csvFileWriter.NumberDecimalSeparator);
                }
                if (saveCsvAverageValues)
                {
                    sb.Append("File name calculated values: ");
                    sb.AppendLine(_csvFileWriter.CsvFileNameCalculatedValues);
                    sb.Append("  - Number decimal separator: ");
                    sb.AppendLine(_csvFileWriter.NumberDecimalSeparator);
                }
                if (saveCsvFftValues)
                {
                    sb.Append("File name FFT values: ");
                    sb.AppendLine(_csvFileWriter.CsvFileNameFftValues);
                    sb.Append("  - Number decimal separator: ");
                    sb.AppendLine(_csvFileWriter.NumberDecimalSeparator);
                }
                if (saveCsvFftCoherentValues)
                {
                    sb.Append("File name FFT coherent values: ");
                    sb.AppendLine(_csvFileWriter.CsvFileNameFftCoherentValues);
                    sb.Append("  - Number decimal separator: ");
                    sb.AppendLine(_csvFileWriter.NumberDecimalSeparator);
                }
            }
            textBoxInfo.AppendText(sb.ToString());
        }

        private string GetSelectedChannelsText(AnalogInputModule aiModule)
        {
            var sb = new StringBuilder();
            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                sb.Append("  - Channel ");
                sb.Append(chanSetting.ChannelIndex);
                sb.Append(", Range ");
                sb.Append(chanSetting.Range.RangeText);
                sb.AppendLine();
            }
            return sb.ToString();
        }

        private void StartAiRead()
        {
            _selectedAiModule.AdqError = AdqErrors.ADQ_ERROR_NONE;
            _moduleIsBusy = true;
            backgroundWorker1.RunWorkerAsync(_selectedAiModule);
        }

        private void TimerPause_Tick(object sender, EventArgs e)
        {
            timerPause.Stop();
            if (_isIntervalReading)
            {
                StartAiRead();
            }
        }

        private void AddInfoTextLine(string text)
        {
            textBoxInfo.AppendText(string.Format("[{0}] {1}" + Environment.NewLine,
                DateTime.Now.ToString("HH:mm:ss.fffffff"), text));
        }

        private void DisplayValues(AnalogInputModule aiModule, int mumIntervalsRead)
        {
            DisplayNumberOfIntervalsRead(mumIntervalsRead, aiModule.NumberOfFramesRead);
            UpdateProgressBar(mumIntervalsRead);
            DisplayCoherentAsChart(chartFftCoherent, _signalUtility.FrameLength, _signalUtility.FrequencySpectrumsCoherent, _signalUtility.FrequencySpectrumsCoherentDBU, _signalUtility.SpectrumSize, _signalUtility.FrequencyStepCoherent);
            DisplayFftValuesAsText();
            DisplayNumericSignalValues();
            DisplayCalculatedValuesAsChart(aiModule);
            DisplayUserConfigurableAsChart(aiModule, chartUserConfigurable, _signalUtility.FrameLength, _signalUtility.FrequencySpectrumsDBU, _signalUtility.FrequencySpectrums, _signalUtility.SpectrumSize, _signalUtility.FrequencyStep);
        }

        private static async Task<int> WriteCsvValuesAsync(CsvFileWriter csvFileWriter)
        {
            if (csvFileWriter == null)
            {
                return 0;
            }

            return await Task.Run(() => csvFileWriter.WriteCsvFileValues());
        }

        private async void BtnSaveLastInterval_Click(object sender, EventArgs e)
        {
            try
            {
                if (CsvWritingEnabled())
                {
                    if (_selectedAiModule.NumberOfFramesRead == 0)
                    {
                        MessageBox.Show("To save any values please acquire frames for the selected module.", "AIReadInterval", MessageBoxButtons.OK, MessageBoxIcon.Error);
                        return;
                    }
                    SetCsvProperties();
                    _csvFileWriter.WriteCsvFileHeader();
                    await WriteCsvValuesAsync(_csvFileWriter);
                }
                else
                {
                    MessageBox.Show("Please select at least one csv file to write.", "AIReadInterval", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
            catch (Exception ex)
            {
                DisplayExceptionMessage(ex);
            }
        }

        private void DisplayNumericSignalValues()
        {
            if (!chkBoxDisplayFftValues.Checked)
            {
                return;
            }

            int subItemIdx;
            for (int i = 0; i < _signalUtility.FrameLength; i++)
            {
                subItemIdx = i + 1;
                _lvItemAvgValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.AverageValues[i]);
                _lvItemRmsValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.RmsValues[i]);
                _lvItemMinValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.MinValues[i]);
                _lvItemMaxValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.MaxValues[i]);
                _lvItemP2PValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.P2PValues[i]);
                _lvItemThdValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.ThdValues[i] * 100.0);
                _lvItemThdrPlusNValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.ThdrPlusNValues[i] * 100.0);
                _lvItemThdfPlusNValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.ThdfPlusNValues[i] * 100.0);
                _lvItemMainFrequencyValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.MainFrequencies[i]);
                _lvItemFrequencyProportionValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.FrequProportions[i] * 100);
                _lvItemRmsValueSpectrumValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.RmsValuesSpectrum[i]);
                _lvItemThdCoherentValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.ThdValuesCoherent[i] * 100.0);
                _lvItemThdrPlusNCoherentValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.ThdrPlusNValuesCoherent[i] * 100.0);
                _lvItemThdfPlusNCoherentValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.ThdfPlusNValuesCoherent[i] * 100.0);
                _lvItemMainFrequencyCoherentValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.MainFrequenciesCoherent[i]);
                _lvItemFrequencyProportionCoherentValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.FrequProportionsCoherent[i] * 100);
                _lvItemRmsValueSpectrumCoherentValue.SubItems[subItemIdx].Text = FormatPhysicalValue(_signalUtility.RmsValuesSpectrumCoherent[i]);
            }
        }

        private string FormatPhysicalValue(double value)
        {
            return string.Format(_fmtStringPhysValues, value);
        }

        private string FormatDBValue(double value)
        {
            return string.Format(_fmtStringDBValues, value);
        }

        private void DisplayNumberOfIntervalsRead(int numIntervalsRead, uint numFramesRead)
        {
            textBoxProgress.Text = string.Format("Interval {0}: read {1} frames", numIntervalsRead, numFramesRead);
        }

        private void UpdateProgressBar(int numIntervalsRead)
        {
            if (_stopAfterIntervals == 0)
            {
                return;
            }

            pBarIntervals.Value = (int)((double)numIntervalsRead / _stopAfterIntervals * 100.0);
        }

        private ChartValueType GetChartXValueType(int selectedIndex)
        {
            return selectedIndex == _idxXAxisLabelsTime ? ChartValueType.String : ChartValueType.UInt32;
        }

        private void InitChartUserConfigurable(AnalogInputModule aiModule)
        {
            switch (coBoxChartGraphicToDisplay.SelectedIndex)
            {
                case _idxGraphsToDisplayAllValues:
                    InitChartUserConfigurableGraphAllValues(aiModule);
                    break;
                case _idxGraphsToDisplayDBU:
                    InitChartUserConfigurableGraphDBU(aiModule);
                    break;
                case _idxGraphsToDisplayAllValuesAndDBUSeparated:
                    InitChartUserConfigurableGraphAllValuesAndDBUSeparated(aiModule);
                    break;
                case _idxGraphsToDisplayAllValuesAndDBUTogether:
                    InitChartUserConfigurableGraphAllValuesAndDBUTogether(aiModule);
                    break;
                case _idxGraphsToDisplayDB:
                    InitChartUserConfigurableGraphDB(aiModule);
                    break;
            }
        }

        private void InitChartUserConfigurableGraphAllValues(AnalogInputModule aiModule)
        {
            chartUserConfigurable.GetToolTipText -= Chart_GetToolTipText;
            chartUserConfigurable.Series.Clear();
            chartUserConfigurable.ChartAreas.Clear();
            chartUserConfigurable.BackColor = Color.Black;
            var caAllValues = new ChartArea();
            chartUserConfigurable.ChartAreas.Add(caAllValues);
            InitChartAllValuesTitles(chartUserConfigurable, caAllValues, aiModule.ChannelSettings[0].Range.UnitName, coBoxXAxisLabelsUserConfigurable);
            InitChartArea(caAllValues, caAllValues.AxisX, caAllValues.AxisY, aiModule.MinResolution, _fmtStringPhysValues);

            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                ChartValueType cvt = GetChartXValueType(coBoxXAxisLabelsUserConfigurable.SelectedIndex);
                InitSeries(chanSetting.ChannelIndex, chartUserConfigurable, coBoxChartTypeUserConfigurable.SelectedIndex,
                    cvt, ChartValueType.Double, caAllValues, _SeriesConstAllValues);
            }
        }

        private void InitChartUserConfigurableGraphDBU(AnalogInputModule aiModule)
        {
            chartUserConfigurable.GetToolTipText -= Chart_GetToolTipText;
            chartUserConfigurable.Series.Clear();
            chartUserConfigurable.Titles.Clear();
            chartUserConfigurable.ChartAreas.Clear();
            chartUserConfigurable.BackColor = Color.Black;
            var caDBU = new ChartArea();
            chartUserConfigurable.ChartAreas.Add(caDBU);
            InitChartDBUTitles(caDBU);
            caDBU.AxisY.Interval = 10;

            InitChartArea(caDBU, caDBU.AxisX, caDBU.AxisY, 0.001, _fmtStringDBIndex);

            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                InitSeries(chanSetting.ChannelIndex, chartUserConfigurable, coBoxChartTypeUserConfigurable.SelectedIndex,
                    ChartValueType.Double, ChartValueType.String, caDBU, _SeriesConstDBU);
            }

        }

        private void InitChartUserConfigurableGraphAllValuesAndDBUSeparated(AnalogInputModule aiModule)
        {
            chartUserConfigurable.GetToolTipText -= Chart_GetToolTipText;
            chartUserConfigurable.Series.Clear();
            chartUserConfigurable.ChartAreas.Clear();
            chartUserConfigurable.BackColor = Color.Black;
            var caAllValues = new ChartArea();
            var caDBU = new ChartArea();
            chartUserConfigurable.ChartAreas.Add(caAllValues);
            chartUserConfigurable.ChartAreas.Add(caDBU);
            caDBU.AxisY.Interval = 10;
            InitChartAllValuesAndDBUSeparatedTitles(caAllValues, caDBU, aiModule.ChannelSettings[0].Range.UnitName);
            InitChartArea(caAllValues, caAllValues.AxisX, caAllValues.AxisY, aiModule.MinResolution, _fmtStringPhysValues);
            InitChartArea(caDBU, caDBU.AxisX, caDBU.AxisY, 0.001, _fmtStringDBIndex);
            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                ChartValueType cvt = GetChartXValueType(coBoxXAxisLabelsUserConfigurable.SelectedIndex);
                InitSeries(chanSetting.ChannelIndex, chartUserConfigurable, coBoxChartTypeUserConfigurable.SelectedIndex,
                    cvt, ChartValueType.Double, caAllValues, _SeriesConstAllValues);
            }
            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                InitSeries(chanSetting.ChannelIndex, chartUserConfigurable, coBoxChartTypeUserConfigurable.SelectedIndex,
                    ChartValueType.Double, ChartValueType.String, caDBU, _SeriesConstDBU);
            }
        }

        private void InitChartUserConfigurableGraphAllValuesAndDBUTogether(AnalogInputModule aiModule)
        {
            chartUserConfigurable.Titles.Clear();
            chartUserConfigurable.GetToolTipText -= Chart_GetToolTipText;
            chartUserConfigurable.Series.Clear();
            chartUserConfigurable.ChartAreas.Clear();
            chartUserConfigurable.BackColor = Color.Black;
            var caAllValuesDBU = new ChartArea();
            chartUserConfigurable.ChartAreas.Add(caAllValuesDBU);
            caAllValuesDBU.AxisY.Interval = 10;
            Title title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitChartTitle(title, caAllValuesDBU, _titleAllValuesAndDBU);

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitXAxisTitle(title, caAllValuesDBU, _titleFrequency);

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitYAxisTitle(title, caAllValuesDBU, _titleDBU);

            title = new Title();
            string titleText = GetXAxisTitle(coBoxXAxisLabelsUserConfigurable.SelectedIndex, _xAxisLabelTime, _xAxisLabelFrameIndex);
            chartUserConfigurable.Titles.Add(title);
            InitX2AxisTitle(title, caAllValuesDBU, titleText);

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitY2AxisTitle(title, caAllValuesDBU, aiModule.ChannelSettings[0].Range.UnitName);

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitChartInfo(title, caAllValuesDBU, "Use Mouse double click to change between Graph zooming, " + "dBu Selected");

            InitChartArea(caAllValuesDBU, caAllValuesDBU.AxisX, caAllValuesDBU.AxisY, 0.001, _fmtStringDBIndex);
            InitChartArea(caAllValuesDBU, caAllValuesDBU.AxisX2, caAllValuesDBU.AxisY2, aiModule.MinResolution, _fmtStringPhysValues);
            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                ChartValueType cvt = GetChartXValueType(coBoxXAxisLabelsUserConfigurable.SelectedIndex);
                InitSeries(chanSetting.ChannelIndex, chartUserConfigurable, coBoxChartTypeUserConfigurable.SelectedIndex,
                    cvt, ChartValueType.Double, caAllValuesDBU, _SeriesConstAllValues);
                chartUserConfigurable.Series[chartUserConfigurable.Series.Count - 1].XAxisType = AxisType.Secondary;
                chartUserConfigurable.Series[chartUserConfigurable.Series.Count - 1].YAxisType = AxisType.Secondary;
                InitSeries(chanSetting.ChannelIndex, chartUserConfigurable, coBoxChartTypeUserConfigurable.SelectedIndex,
                    ChartValueType.Double, ChartValueType.String, caAllValuesDBU, _SeriesConstDBU);
                switch (coBoxColor.SelectedIndex)
                {
                    case _idxColorSimilar:
                        chartUserConfigurable.Series[chartUserConfigurable.Series.Count - 1].Color =
                            AdqCommon.ChannelColorsDBUSimilar[chanSetting.ChannelIndex % AdqCommon.ChannelColorsDBUSimilar.Length];
                        continue;
                    case _idxColorRed:
                        chartUserConfigurable.Series[chartUserConfigurable.Series.Count - 1].Color = AdqCommon.ChannelColorsRed;
                        continue;
                }
            }
        }

        private void InitChartUserConfigurableGraphDB(AnalogInputModule aiModule)
        {
            chartUserConfigurable.Titles.Clear();
            chartUserConfigurable.GetToolTipText -= Chart_GetToolTipText;
            chartUserConfigurable.Series.Clear();
            chartUserConfigurable.ChartAreas.Clear();
            chartUserConfigurable.BackColor = Color.Black;
            var caDB = new ChartArea();
            chartUserConfigurable.ChartAreas.Add(caDB);
            Title title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitChartTitle(title, caDB, _titleFFTINDB + _selectedUnitSymbol);

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitXAxisTitle(title, caDB, _titleFrequency);

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitYAxisTitle(title, caDB, _titleDB + _selectedUnitSymbol);

            InitChartArea(caDB, caDB.AxisX, caDB.AxisY, 0.001, _fmtStringPhysValues);
            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                InitSeries(chanSetting.ChannelIndex, chartUserConfigurable, coBoxChartTypeUserConfigurable.SelectedIndex,
                    ChartValueType.Double, ChartValueType.String, caDB, _SeriesConstFFT);
            }
        }

        private void InitChartAllValuesTitles(Chart chart, ChartArea chartArea, string yAxisTitle, ComboBox coBoxXAxisLabels)
        {
            chart.Titles.SuspendUpdates();
            chart.Titles.Clear();

            var title = new Title();
            chart.Titles.Add(title);
            InitChartTitle(title, chartArea, _titleAllValues);

            title = new Title();
            chart.Titles.Add(title);
            InitYAxisTitle(title, chartArea, yAxisTitle);

            title = new Title();
            chart.Titles.Add(title);
            string titleText = GetXAxisTitle(coBoxXAxisLabels.SelectedIndex, _xAxisLabelTime, _xAxisLabelFrameIndex);
            InitXAxisTitle(title, chartArea, titleText);

            chart.Titles.ResumeUpdates();
        }

        private void InitChartAllValuesAndDBUSeparatedTitles(ChartArea chartAreaAllValues, ChartArea chartAreaDBU, string yAxisTitle)
        {
            chartUserConfigurable.Titles.SuspendUpdates();
            chartUserConfigurable.Titles.Clear();

            var title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitChartTitle(title, chartAreaAllValues, _titleAllValuesAndDBU);

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitTitle(title, chartAreaAllValues, yAxisTitle);
            title.Position.X = 0.5f;
            title.Position.Y = 0;
            title.Position.Width = 5;
            title.Position.Height = 50;
            title.Alignment = ContentAlignment.TopCenter;
            title.Docking = Docking.Left;
            title.TextOrientation = TextOrientation.Rotated270;

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            string titleText = GetXAxisTitle(coBoxXAxisLabelsUserConfigurable.SelectedIndex, _xAxisLabelTime, _xAxisLabelFrameIndex);
            InitTitle(title, chartAreaAllValues, titleText);
            title.Position.X = 3;
            title.Position.Y = 45;
            title.Position.Width = 94;
            title.Position.Height = 5;
            title.Alignment = ContentAlignment.BottomCenter;
            title.Docking = Docking.Bottom;

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitTitle(title, chartAreaDBU, _titleDBU);
            title.Position.X = 0.5f;
            title.Position.Y = 100;
            title.Position.Width = 5;
            title.Position.Height = 50;
            title.Alignment = ContentAlignment.TopCenter;
            title.Docking = Docking.Left;
            title.TextOrientation = TextOrientation.Rotated270;

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitTitle(title, chartAreaDBU, _titleFrequency);
            title.Position.X = 3;
            title.Position.Y = 95;
            title.Position.Width = 94;
            title.Position.Height = 5;
            title.Alignment = ContentAlignment.BottomCenter;
            title.Docking = Docking.Bottom;

            chartUserConfigurable.Titles.ResumeUpdates();
        }

        private void InitTitle(Title title, ChartArea chartArea, string titleText)
        {
            title.DockedToChartArea = chartArea.Name;
            title.Text = titleText;
            title.Position.Auto = false;
            title.IsDockedInsideChartArea = false;
            title.ForeColor = _labelColor;
        }

        private void InitChartDBUTitles(ChartArea chartArea)
        {
            chartUserConfigurable.Titles.SuspendUpdates();
            chartUserConfigurable.Titles.Clear();

            var title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitChartTitle(title, chartArea, _titleFFTINDBU);

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitYAxisTitle(title, chartArea, _titleDBU);

            title = new Title();
            chartUserConfigurable.Titles.Add(title);
            InitXAxisTitle(title, chartArea, _titleFrequency);

            chartUserConfigurable.Titles.ResumeUpdates();
        }

        private void InitChartCalculatedValues(AnalogInputModule aiModule)
        {
            chartCalculated.GetToolTipText -= Chart_GetToolTipText;
            chartCalculated.Series.Clear();
            chartCalculated.BackColor = Color.Black;
            chartCalculated.ChartAreas.Clear();
            var ca = new ChartArea();
            chartCalculated.ChartAreas.Add(ca);
            InitChartCalculatedTitles(ca, aiModule);
            InitChartArea(ca, ca.AxisX, ca.AxisY, aiModule.MinResolution, _fmtStringPhysValues);
            ca.AxisX.Minimum = 0;
            ca.AxisX.Maximum = _maxIntervalsToDisplay;
            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                ChartValueType cvt = GetChartXValueType(coBoxXAxisLabelsCalcValues.SelectedIndex);
                InitSeries(chanSetting.ChannelIndex, chartCalculated, coBoxChartTypeCalculated.SelectedIndex,
                    cvt, ChartValueType.Double, ca, _SeriesConstCalculated);
            }
        }

        private void InitChartCalculatedTitles(ChartArea chartArea, AnalogInputModule aiModule)
        {
            chartCalculated.Titles.SuspendUpdates();
            chartCalculated.Titles.Clear();

            var title = new Title();
            chartCalculated.Titles.Add(title);
            InitChartTitle(title, chartArea, coBoxCalculatedType.Text);

            title = new Title();
            chartCalculated.Titles.Add(title);
            InitYAxisTitle(title, chartArea, GetYAxisUnitNameCalculatedTitle(aiModule.ChannelSettings[0].Range.UnitName));

            title = new Title();
            chartCalculated.Titles.Add(title);
            string titleText = GetXAxisTitle(coBoxXAxisLabelsCalcValues.SelectedIndex, _xAxisLabelTime, _xAxisLabelFrameIndex);
            InitXAxisTitle(title, chartArea, titleText);

            chartCalculated.Titles.ResumeUpdates();
        }

        private string GetYAxisUnitNameCalculatedTitle(string physicalUnitName)
        {
            SignalValuesType sigValueType = (((StringValueItem<SignalValuesType>)(coBoxCalculatedType.SelectedItem)).Value);

            switch (sigValueType)
            {
                case SignalValuesType.AverageValues:
                case SignalValuesType.RMSValues:
                case SignalValuesType.MinimumValues:
                case SignalValuesType.MaximumValues:
                case SignalValuesType.P2PValues:
                case SignalValuesType.RMSValuesSpectrum:
                case SignalValuesType.RMSValuesSpectrumCoherent:
                    return physicalUnitName;
                case SignalValuesType.MainFrequencies:
                case SignalValuesType.FrequProportionsCoherent:
                    return "Hz";
                default:
                    return "Amplitude";
            }
        }

        private void InitChartCoherentValues(Chart chart, AnalogInputModule aiModule)
        {
            switch (coBoxChartGraphicCoherent.SelectedIndex)
            {
                case _idxGraphicCoherentDB:
                    InitGraphicCoherentDB(chart, aiModule);
                    break;
                case _idxGraphicCoherentDBU:
                    InitGraphicCoherentDBU(chart, aiModule);
                    break;
            }
        }

        private void InitGraphicCoherentDB(Chart chart, AnalogInputModule aiModule)
        {
            chart.GetToolTipText -= Chart_GetToolTipText;
            chart.Series.Clear();
            chart.Titles.Clear();
            chart.BackColor = Color.Black;
            chart.ChartAreas.Clear();
            var ca = new ChartArea();
            chart.ChartAreas.Add(ca);
            InitChartFftTitles(chart, ca, _titleDB + _selectedUnitSymbol, _titleFFTCoherentDB + _selectedUnitSymbol);
            InitChartArea(ca, ca.AxisX, ca.AxisY, 0.001, _fmtStringPhysValues);
            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                InitSeries(chanSetting.ChannelIndex, chart, coBoxChartTypeGraphicCoherent.SelectedIndex,
                    ChartValueType.Double, ChartValueType.String, ca, _SeriesConstFFT);
            }
        }

        private void InitGraphicCoherentDBU(Chart chart, AnalogInputModule aiModule)
        {
            chart.GetToolTipText -= Chart_GetToolTipText;
            chart.Series.Clear();
            chart.Titles.Clear();
            chart.BackColor = Color.Black;
            chart.ChartAreas.Clear();
            var ca = new ChartArea();
            chart.ChartAreas.Add(ca);
            ca.AxisY.Interval = 10;
            InitChartFftTitles(chart, ca, _titleDBU, _titleFFTCoherentDBU);
            InitChartArea(ca, ca.AxisX, ca.AxisY, 0.001, _fmtStringDBIndex);
            foreach (ChannelSetting chanSetting in aiModule.ChannelSettings)
            {
                InitSeries(chanSetting.ChannelIndex, chart, coBoxChartTypeGraphicCoherent.SelectedIndex,
                    ChartValueType.Double, ChartValueType.String, ca, _SeriesConstFFT);
            }
        }

        private void InitChartFftTitles(Chart chart, ChartArea chartArea, string yAxisTitle, string chartText)
        {
            chart.Titles.SuspendUpdates();
            chart.Titles.Clear();
            var title = new Title();
            chart.Titles.Add(title);
            InitChartTitle(title, chartArea, chartText);
            title = new Title();
            chart.Titles.Add(title);
            InitYAxisTitle(title, chartArea, yAxisTitle);
            title = new Title();
            chart.Titles.Add(title);
            InitXAxisTitle(title, chartArea, _titleFrequency);
            chart.Titles.ResumeUpdates();
        }

        private string GetXAxisTitle(int selectedIndex, string timeTitle, string valueTitle)
        {
            return (selectedIndex == _idxXAxisLabelsTime) ? timeTitle : valueTitle;
        }

        private void InitChartTitle(Title title, ChartArea chartArea, string titleText)
        {
            title.DockedToChartArea = chartArea.Name;
            title.Text = titleText;
            title.Alignment = ContentAlignment.TopCenter;
            title.Docking = Docking.Top;
            title.IsDockedInsideChartArea = false;
            title.ForeColor = _labelColor;
            title.Position.Auto = false;
            title.Position.X = 3;
            title.Position.Y = 0;
            title.Position.Width = 94;
            title.Position.Height = 5;
        }

        private void InitChartInfo(Title title, ChartArea chartArea, string titleText)
        {
            title.DockedToChartArea = chartArea.Name;
            title.Text = titleText;
            title.Alignment = ContentAlignment.TopCenter;
            title.Docking = Docking.Top;
            title.IsDockedInsideChartArea = false;
            title.ForeColor = _labelColor;
            title.Position.Auto = false;
            title.Position.X = 3;
            title.Position.Y = 0;
            title.Position.Width = 40;
            title.Position.Height = 15;
        }

        private void InitYAxisTitle(Title title, ChartArea chartArea, string titleText)
        {
            title.DockedToChartArea = chartArea.Name;
            title.Text = titleText;
            title.Position.Auto = false;
            title.Position.X = 0.5f;
            title.Position.Y = 0;
            title.Position.Width = 5;
            title.Position.Height = 95;
            title.Alignment = ContentAlignment.TopCenter;
            title.Docking = Docking.Left;
            title.IsDockedInsideChartArea = false;
            title.TextOrientation = TextOrientation.Rotated270;
            title.ForeColor = _labelColor;
        }

        private void InitY2AxisTitle(Title title, ChartArea chartArea, string titleText)
        {
            title.DockedToChartArea = chartArea.Name;
            title.Text = titleText;
            title.Position.Auto = false;
            title.Position.X = 100;
            title.Position.Y = 0;
            title.Position.Width = 3;
            title.Position.Height = 95;
            title.Alignment = ContentAlignment.TopCenter;
            title.Docking = Docking.Right;
            title.IsDockedInsideChartArea = false;
            title.TextOrientation = TextOrientation.Rotated270;
            title.ForeColor = _labelColor;
        }

        private void InitXAxisTitle(Title title, ChartArea chartArea, string titleText)
        {
            title.DockedToChartArea = chartArea.Name;
            title.Text = titleText;
            title.Position.Auto = false;
            title.Position.X = 3;
            title.Position.Y = 95;
            title.Position.Width = 94;
            title.Position.Height = 5;
            title.Alignment = ContentAlignment.BottomCenter;
            title.Docking = Docking.Bottom;
            title.IsDockedInsideChartArea = false;
            title.ForeColor = _labelColor;
        }

        private void InitX2AxisTitle(Title title, ChartArea chartArea, string titleText)
        {
            title.DockedToChartArea = chartArea.Name;
            title.Text = titleText;
            title.Position.Auto = false;
            title.Position.X = 3;
            title.Position.Y = 2;
            title.Position.Width = 94;
            title.Position.Height = 5;
            title.Alignment = ContentAlignment.TopCenter;
            title.Docking = Docking.Top;
            title.IsDockedInsideChartArea = false;
            title.ForeColor = _labelColor;
        }

        private void InitChartArea(ChartArea chartArea, Axis axisX, Axis axisY, double resolutionYAxis, string labelStyleFormat)
        {
            chartArea.BackColor = Color.Black;

            axisX.MajorGrid.LineColor = _gridLineColor;
            axisX.MajorTickMark.LineColor = _gridLineColor;
            axisX.MinorGrid.LineColor = _gridLineColor;
            axisX.MinorTickMark.LineColor = _gridLineColor;
            axisX.LineColor = _gridLineColor;
            axisX.LineWidth = 1;
            axisX.LabelStyle.ForeColor = _labelColor;
            axisX.ScrollBar.ButtonColor = Color.DarkGray;
            axisX.ScrollBar.BackColor = Color.Gray;
            axisX.ScaleView.Zoomable = true;
            axisX.ScrollBar.Enabled = true;

            axisY.MajorGrid.LineColor = _gridLineColor;
            axisY.MajorTickMark.LineColor = _gridLineColor;
            axisY.MinorGrid.LineColor = _gridLineColor;
            axisY.MinorTickMark.LineColor = _gridLineColor;
            axisY.LineColor = _gridLineColor;
            axisY.LineWidth = 1;
            axisY.IsLabelAutoFit = false;
            axisY.LabelStyle.ForeColor = _labelColor;
            axisY.LabelStyle.Format = labelStyleFormat;
            axisY.ScrollBar.ButtonColor = Color.DarkGray;
            axisY.ScrollBar.BackColor = Color.Gray;
            axisY.ScaleView.Zoomable = true;
            axisY.ScrollBar.Enabled = true;

            chartArea.CursorX.IsUserSelectionEnabled = true;
            chartArea.CursorY.IsUserSelectionEnabled = true;
            double scrollSize = resolutionYAxis / 4.0;
            axisY.ScaleView.SmallScrollMinSize = scrollSize;
            axisY.ScaleView.SmallScrollSize = scrollSize;
            chartArea.CursorY.Interval = scrollSize;
            chartArea.CursorX.Interval = scrollSize;
        }

        private void InitSeries(uint channelIndex, Chart chart, int chartTypeIndex, ChartValueType chartXValueType,
            ChartValueType chartYValueType, ChartArea chartArea, string type)
        {
            Series series = new Series
            {
                Name = "Channel " + channelIndex.ToString() + chartArea.Name.ToString() + type
            };
            series.ChartArea = chartArea.Name;
            if (chartTypeIndex == _idxChartTypeLine)
            {
                series.ChartType = SeriesChartType.Line;
            }
            else if (chartTypeIndex == _idxChartTypeLineAndMarkers)
            {
                series.ChartType = SeriesChartType.Line;
                series.MarkerStyle = AdqCommon.MarkerStyles[channelIndex % AdqCommon.MarkerStyles.Length];
            }
            else
            {
                series.ChartType = SeriesChartType.Point;
            }
            series.Color = AdqCommon.ChannelColors[channelIndex % AdqCommon.ChannelColors.Length];
            series.YValueType = chartYValueType;
            series.XValueType = chartXValueType;
            series.IsXValueIndexed = false;
            chart.Series.Add(series);
        }

        private string[] _xAxisLabelsUserConfigurable;

        private void GenerateTimeAxisLabels(ComboBox coBoxXAxisLabels, uint numberOfFrames, ref string[] _xAxisLabels)
        {
            if (_xAxisLabels == null || _xAxisLabels.Length != numberOfFrames)
            {
                _xAxisLabels = new string[numberOfFrames];
            }
            double xValue = 0.0;
            double xValueDelta = GetXValueDelta(coBoxXAxisLabels);
            for (int i = 0; i < numberOfFrames; i++)
            {
                _xAxisLabels[i] = GetXAsixLabel(coBoxXAxisLabels, xValue);
                xValue += xValueDelta;
            }
        }

        private void DisplayUserConfigurableAsChart(
            AnalogInputModule aiModule, Chart chart, uint frameLength, List<double[]> frequencySpectrumsDBU,
            List<double[]> frequencySpectrums, uint spectrumSize, double frequencyStep)
        {
            switch (coBoxChartGraphicToDisplay.SelectedIndex)
            {
                case _idxGraphsToDisplayAllValues:
                    DisplayUserConfigurableGraphAllValuesAsChart(aiModule, chart);
                    break;
                case _idxGraphsToDisplayDBU:
                    RemoveSeriesPoints(chart, 0);
                    DisplayUserConfigurableGraphDBUAsChart(chart, frequencySpectrumsDBU, spectrumSize, frequencyStep);
                    break;
                case _idxGraphsToDisplayAllValuesAndDBUSeparated:
                    DisplayUserConfigurableGraphAllValuesAndDBUSeperatedAsChart(aiModule, chart, frequencySpectrumsDBU, spectrumSize, frequencyStep);
                    break;
                case _idxGraphsToDisplayAllValuesAndDBUTogether:
                    RemoveSeriesPoints(chart, 0);
                    DisplayUserConfigurableGraphAllValuesAndDBUTogetherAsChart(aiModule, frequencySpectrumsDBU, spectrumSize, frequencyStep);
                    break;
                case _idxGraphsToDisplayDB:
                    RemoveSeriesPoints(chart, 0);
                    DisplayUserConfigurableGraphDBAsChart(chart, frameLength, frequencySpectrums, spectrumSize, frequencyStep);
                    break;
            }
        }

        private void DisplayUserConfigurableGraphAllValuesAsChart(AnalogInputModule aiModule, Chart chart)
        {
            RemoveSeriesPoints(chart, 0);
            if (!chkBoxShowGraphicUserConfigurable.Checked)
            {
                return;
            }

            if (chartUserConfigurable.Series.Count != aiModule.FrameLength)
            {
                return;
            }
            chartUserConfigurable.SuspendLayout();
            chartUserConfigurable.Series.SuspendUpdates();
            DisplaySeriesAllValues(chartUserConfigurable, aiModule, ref _xAxisLabelsUserConfigurable);
            if (chkBoxAutoScaleUserConfig.Checked)
            {
                chartUserConfigurable.ChartAreas[0].RecalculateAxesScale();
            }
            chartUserConfigurable.Series.ResumeUpdates();
            chartUserConfigurable.ResumeLayout();
        }

        private void DisplayUserConfigurableGraphDBUAsChart(Chart chart, List<double[]> frequencySpectrumsDBU, uint spectrumSize, double frequencyStep)
        {
            if (!chkBoxShowGraphicUserConfigurable.Checked)
            {
                return;
            }

            int idx = 0;
            foreach (Series series in chart.Series)
            {
                if (series.Name.Contains(_SeriesConstDBU))
                {
                    series.Points.SuspendUpdates();

                    double[] spectrum = frequencySpectrumsDBU[idx];
                    uint valIdx = 1;
                    double xValue = 0;
                    while (valIdx < spectrumSize)
                    {
                        double physicalValue = spectrum[valIdx];
                        var dp = new DataPoint(valIdx, physicalValue)
                        {
                            AxisLabel = string.Format("{0:0.0#####}", xValue),
                        };
                        series.Points.Add(dp);
                        xValue += frequencyStep;
                        valIdx++;
                    }
                    series.Points.ResumeUpdates();
                    idx++;
                }

            }
            if (chkBoxAutoScaleUserConfig.Checked)
            {
                chart.ChartAreas[0].RecalculateAxesScale();
            }
        }

        private void DisplayUserConfigurableGraphAllValuesAndDBUSeperatedAsChart(AnalogInputModule aiModule, Chart chart,
            List<double[]> frequencySpectrumsDBU, uint spectrumSize, double frequencyStep)
        {
            RemoveSeriesPoints(chart, 0);
            if (!chkBoxShowGraphicUserConfigurable.Checked)
            {
                return;
            }

            chartUserConfigurable.SuspendLayout();
            chartUserConfigurable.Series.SuspendUpdates();
            DisplaySeriesAllValues(chartUserConfigurable, aiModule, ref _xAxisLabelsUserConfigurable);
            RemoveSeriesPoints(chart, 1);
            DisplayUserConfigurableGraphDBUAsChart(chart, frequencySpectrumsDBU, spectrumSize, frequencyStep);
            if (chkBoxAutoScaleUserConfig.Checked)
            {
                chartUserConfigurable.ChartAreas[0].RecalculateAxesScale();
                chartUserConfigurable.ChartAreas[1].RecalculateAxesScale();
            }
            chartUserConfigurable.Series.ResumeUpdates();
            chartUserConfigurable.ResumeLayout();
        }

        private void DisplayUserConfigurableGraphAllValuesAndDBUTogetherAsChart(AnalogInputModule aiModule,
            List<double[]> frequencySpectrumsDBU, uint spectrumSize, double frequencyStep)
        {
            if (!chkBoxShowGraphicUserConfigurable.Checked)
            {
                return;
            }

            chartUserConfigurable.SuspendLayout();
            chartUserConfigurable.Series.SuspendUpdates();
            DisplaySeriesAllValues(chartUserConfigurable, aiModule, ref _xAxisLabelsUserConfigurable);
            DisplayUserConfigurableGraphDBUAsChart(chartUserConfigurable, frequencySpectrumsDBU, spectrumSize, frequencyStep);
            if (chkBoxAutoScaleUserConfig.Checked)
            {
                chartUserConfigurable.ChartAreas[0].RecalculateAxesScale();
            }
            chartUserConfigurable.Series.ResumeUpdates();
            chartUserConfigurable.ResumeLayout();
        }

        private void DisplayUserConfigurableGraphDBAsChart(Chart chart, uint frameLength,
            List<double[]> frequencySpectrums, uint spectrumSize, double frequencyStep)
        {
            if (!chkBoxShowGraphicUserConfigurable.Checked)
            {
                return;
            }

            chartUserConfigurable.SuspendLayout();
            chartUserConfigurable.Series.SuspendUpdates();
            for (int idx = 0; idx < frameLength; idx++)
            {
                Series series = chart.Series[idx];
                series.Points.SuspendUpdates();
                double[] spectrum = frequencySpectrums[idx];
                uint valIdx = 1;
                double xValue = 0;
                while (valIdx < spectrumSize)
                {
                    double physicalValue = spectrum[valIdx];
                    var dp = new DataPoint(valIdx, physicalValue)
                    {
                        AxisLabel = string.Format("{0:0.0#####}", xValue)
                    };
                    series.Points.Add(dp);
                    xValue += frequencyStep;
                    valIdx++;
                }
                series.Points.ResumeUpdates();
            }

            if (chkBoxAutoScaleUserConfig.Checked)
            {
                chart.ChartAreas[0].RecalculateAxesScale();
                chartUserConfigurable.ChartAreas[0].RecalculateAxesScale();
            }
            chartUserConfigurable.Series.ResumeUpdates();
            chartUserConfigurable.ResumeLayout();
        }

        private void DisplaySeriesAllValues(Chart chart, AnalogInputModule aiModule, ref string[] _xAxisLabels)
        {
            uint frameLength = aiModule.FrameLength;
            uint idx = 0;
            foreach (Series series in chart.Series)
            {
                if (series.Name.Contains(_SeriesConstAllValues))
                {
                    series.Points.SuspendUpdates();
                    uint valIdx = idx;
                    int pointIdx = 0;
                    while (valIdx < aiModule.TotalNumberOfValuesRead)
                    {
                        double physicalValue = aiModule.PhysicalFrames[valIdx];
                        var dp = new DataPoint(pointIdx, physicalValue)
                        {
                            AxisLabel = _xAxisLabels[pointIdx]
                        };
                        series.Points.Add(dp);
                        valIdx += frameLength;
                        pointIdx++;
                    }
                    idx++;
                    series.Points.ResumeUpdates();
                }
            }
        }

        private void DisplayCoherentAsChart(Chart chart, uint frameLength, List<double[]> frequencySpectrumsCoherent,
            List<double[]> frequencySpectrumsCoherentDBU, uint spectrumSize, double frequencyStep)
        {
            RemoveSeriesPoints(chart, 0);
            if (!chkBoxShowGraphicCoherent.Checked)
            {
                return;
            }

            switch (coBoxChartGraphicCoherent.SelectedIndex)
            {
                case _idxGraphicCoherentDB:
                    DisplayGraphicCoherent(chart, frameLength, frequencySpectrumsCoherent, spectrumSize, frequencyStep);
                    break;
                case _idxGraphicCoherentDBU:
                    DisplayGraphicCoherent(chart, frameLength, frequencySpectrumsCoherentDBU, spectrumSize, frequencyStep);
                    break;
            }
        }

        private void DisplayGraphicCoherent(Chart chart, uint frameLength,
            List<double[]> frequencySpectrums, uint spectrumSize, double frequencyStep)
        {
            for (int idx = 0; idx < frameLength; idx++)
            {
                Series series = chart.Series[idx];
                series.Points.SuspendUpdates();
                double[] spectrum = frequencySpectrums[idx];
                uint valIdx = 1;
                double xValue = 0;
                while (valIdx < spectrumSize)
                {
                    double physicalValue = spectrum[valIdx];
                    var dp = new DataPoint(valIdx, physicalValue)
                    {
                        AxisLabel = string.Format("{0:0.0#####}", xValue)
                    };
                    series.Points.Add(dp);
                    xValue += frequencyStep;
                    valIdx++;
                }
                series.Points.ResumeUpdates();
            }
            if (chkBoxAutoScaleCoherent.Checked)
            {
                chart.ChartAreas[0].RecalculateAxesScale();
            }
        }

        private void DisplayFftValuesAsText()
        {
            DisplayFftValuesAsText(textBoxFftValues, chkBoxDisplayFftValues.Checked,
                _signalUtility.FrequencySpectrums, _signalUtility.FrequencyStep);
            DisplayFftValuesAsText(textBoxFftValuesCoherent, chkBoxDisplayFftCoherentValues.Checked,
                _signalUtility.FrequencySpectrumsCoherent, _signalUtility.FrequencyStepCoherent);
        }

        private void DisplayFftValuesAsText(TextBox textBox, bool displayValues,
            List<double[]> frequencySpectrums, double frequencyStep)
        {
            textBox.Clear();
            if (!displayValues)
            {
                return;
            }

            var sb = new StringBuilder();
            sb.AppendLine("Channel; Frequency step; Frequency");
            for (int idx = 0; idx < _signalUtility.FrameLength; idx++)
            {
                double fStep = 0;
                double[] spectrum = frequencySpectrums[idx];
                uint channelIdx = _signalUtility.GetChannelIndex(idx);
                uint valIdx = 0;
                while (valIdx < _signalUtility.SpectrumSize)
                {
                    double physicalValue = spectrum[valIdx];
                    sb.AppendFormat("{0}; {1:0.000000000}; ", channelIdx, fStep);
                    sb.AppendFormat(_fmtStringFftValues, physicalValue);
                    sb.AppendLine();
                    fStep += frequencyStep;
                    valIdx++;
                }
            }
            textBox.AppendText(sb.ToString());
        }

        private string GetXAsixLabel(ComboBox coBoxXAxisLabels, double xValue)
        {
            if (coBoxXAxisLabels.SelectedIndex == _idxXAxisLabelsTime)
            {
                return AdqCommon.FormatSampleTime(xValue);
            }
            return ((int)xValue).ToString();
        }

        private double GetXValueDelta(ComboBox coBoxXAxisLabels)
        {
            return (coBoxXAxisLabels.SelectedIndex == _idxXAxisLabelsTime) ? _selectedAiModule.SampleTime : 1.0;
        }

        private void DisplayCalculatedValuesAsChart(AnalogInputModule aiModule)
        {
            if (!chkBoxShowGraphicCalculated.Checked)
            {
                return;
            }

            if (coBoxXAxisLabelsCalcValues.SelectedIndex == _idxXAxisLabelsTime)
            {
                TimeSpan tDiff = aiModule.DateTimeFirstFrame - _startDateTime;
                string frameTime = AdqCommon.FormatDuration(tDiff.TotalSeconds);
                for (int idx = 0; idx < aiModule.FrameLength; idx++)
                {
                    Series series = chartCalculated.Series[idx];
                    while (series.Points.Count >= _maxIntervalsToDisplay)
                    {
                        series.Points.RemoveAt(0);
                    }
                    var dp = new DataPoint(_intervalsRead - 1, _signalUtility.SelectedValuesType[idx])
                    {
                        AxisLabel = frameTime
                    };
                    series.Points.Add(dp);
                }
            }
            else
            {
                for (int idx = 0; idx < aiModule.FrameLength; idx++)
                {
                    Series series = chartCalculated.Series[idx];
                    while (series.Points.Count >= _maxIntervalsToDisplay)
                    {
                        series.Points.RemoveAt(0);
                    }
                    series.Points.AddXY(_intervalsRead - 1, _signalUtility.SelectedValuesType[idx]);
                }
            }
            if (_intervalsRead >= _maxIntervalsToDisplay)
            {
                ChartArea ca = chartCalculated.ChartAreas[0];
                if (ca.AxisX.ScaleView.IsZoomed)
                {
                    double axMin = _intervalsRead - _maxIntervalsToDisplay;
                    double axMax = _intervalsRead;
                    ca.AxisX.Minimum = axMin;
                    ca.AxisX.Maximum = axMax;
                    double newPos = ca.AxisX.ScaleView.Position + 1;
                    ca.AxisX.ScaleView.Position = Math.Max(Math.Min(newPos, axMax), axMin);
                }
                else
                {
                    ca.AxisX.Minimum = _intervalsRead - _maxIntervalsToDisplay;
                    ca.AxisX.Maximum = _intervalsRead;
                }
                if (chBoxAutoScaleCalculated.Checked)
                {
                    ca.RecalculateAxesScale();
                }
            }
        }

        private void RemoveSeriesPoints(Chart chart, int chartAreaIndex)
        {
            for (int i = 0; i < chart.Series.Count; i++)
            {
                if (!chart.ChartAreas[chartAreaIndex].Name.Equals(chart.Series[i].ChartArea))
                {
                    continue;
                }
                RemovePoints(chart.Series[i]);
            }
        }

        private void RemovePoints(Series series)
        {
            series.Points.SuspendUpdates();
            while (series.Points.Count > 0)
            {
                series.Points.RemoveAt(series.Points.Count - 1);
            }
            series.Points.ResumeUpdates();
            // mul 19.12.2019 removed the following line, for this forces the chart to autoscale again !
            //series.Points.Clear();
        }

        private void EnableChartsTooltipFunctions()
        {
            if (chkBoxShowGraphicCalculated.Checked && chartCalculated.Series.Count > 0)
            {
                chartCalculated.GetToolTipText += Chart_GetToolTipText;
            }
            if (chkBoxShowGraphicUserConfigurable.Checked && chartUserConfigurable.Series.Count > 0)
            {
                chartUserConfigurable.GetToolTipText += Chart_GetToolTipText;
            }
            if (chkBoxShowGraphicCoherent.Checked && chartFftCoherent.Series.Count > 0)
            {
                chartFftCoherent.GetToolTipText += Chart_GetToolTipText;
            }
        }

        private void Chart_GetToolTipText(object sender, ToolTipEventArgs e)
        {
            if (e.HitTestResult.ChartElementType == ChartElementType.DataPoint)
            {
                var series = e.HitTestResult.Series;
                if (series != null)
                {
                    var dp = series.Points[e.HitTestResult.PointIndex];
                    e.Text = FormatTooltipText(sender, dp);
                }
            }
            else if (e.HitTestResult.ChartElementType == ChartElementType.Axis
                || e.HitTestResult.ChartElementType == ChartElementType.TickMarks)
            {
                if (sender is Chart)
                {
                    HitTestResult result = ((Chart)sender).HitTest(e.X, e.Y, ChartElementType.DataPoint);
                    if (result.ChartElementType == ChartElementType.DataPoint)
                    {
                        var dp = result.Series.Points[result.PointIndex];
                        e.Text = FormatTooltipText(sender, dp);
                    }
                }
            }
        }

        private string FormatTooltipText(object sender, DataPoint dp)
        {
            try
            {
                if (sender == chartFftCoherent && coBoxChartGraphicCoherent.SelectedIndex == _idxGraphicCoherentDB)
                {
                    return string.Format("X={0}, Y={1}dB{2}", dp.AxisLabel, FormatDBValue(dp.YValues[0]), _selectedUnitSymbol);
                }
                if (sender == chartFftCoherent && coBoxChartGraphicCoherent.SelectedIndex == _idxGraphicCoherentDBU)
                {
                    return string.Format("X={0}, Y={1}dBu", dp.AxisLabel, FormatDBValue(dp.YValues[0]));
                }
                if (sender == chartUserConfigurable && coBoxChartGraphicToDisplay.SelectedIndex == _idxGraphsToDisplayAllValues)
                {
                    return string.Format("X={0}, Y={1}{2}", dp.XValue, AdqCommon.FormatPhysicalValue(_fmtStringPhysValues, dp.YValues[0]), _selectedUnitSymbol);
                }
                if (sender == chartUserConfigurable && coBoxChartGraphicToDisplay.SelectedIndex == _idxGraphsToDisplayDBU)
                {
                    return string.Format("X={0}, Y={1}dBu", dp.AxisLabel, FormatDBValue(dp.YValues[0]));
                }
                if (sender == chartUserConfigurable && coBoxChartGraphicToDisplay.SelectedIndex == _idxGraphsToDisplayDB)
                {
                    return string.Format("X={0}, Y={1}dB{2}", dp.AxisLabel, FormatDBValue(dp.YValues[0]), _selectedUnitSymbol);
                }
                if (sender == chartUserConfigurable && coBoxChartGraphicToDisplay.SelectedIndex == _idxGraphsToDisplayAllValuesAndDBUSeparated)
                {
                    foreach (Series series in chartUserConfigurable.Series)
                    {
                        if (series.Name.Contains(_SeriesConstDBU))
                        {
                            if (series.Points.Contains(dp))
                            {
                                return string.Format("X={0}, Y={1}dBu", dp.AxisLabel, FormatDBValue(dp.YValues[0]));
                            }
                        }
                        if (series.Name.Contains(_SeriesConstAllValues))
                        {
                            if (series.Points.Contains(dp))
                            {
                                return string.Format("X={0}, Y={1}{2}", dp.XValue,
                                    AdqCommon.FormatPhysicalValue(_fmtStringPhysValues, dp.YValues[0]), _selectedUnitSymbol);
                            }
                        }
                    }
                }
                if (sender == chartUserConfigurable && coBoxChartGraphicToDisplay.SelectedIndex == _idxGraphsToDisplayAllValuesAndDBUTogether)
                {
                    foreach (Series series in chartUserConfigurable.Series)
                    {
                        if (series.Name.Contains(_SeriesConstAllValues))
                        {
                            if (series.Points.Contains(dp))
                            {
                                return string.Format("X={0}, Y={1}{2}", dp.XValue,
                                    AdqCommon.FormatPhysicalValue(_fmtStringPhysValues, dp.YValues[0]), _selectedUnitSymbol);
                            }
                        }
                        if (series.Name.Contains(_SeriesConstDBU))
                        {
                            if (series.Points.Contains(dp))
                            {
                                return string.Format("X={0}, Y={1}dBu", dp.AxisLabel, FormatDBValue(dp.YValues[0]));
                            }
                        }
                    }
                }
                if (sender == chartCalculated)
                {
                    return string.Format("X={0}, Y={1}{2}", dp.XValue,
                        AdqCommon.FormatPhysicalValue(_fmtStringPhysValues, dp.YValues[0]), _selectedUnitSymbol);
                }
            }
            catch (Exception ex)
            {
                DisplayExceptionMessage(ex);
            }
            return "";
        }

        private void BackgroundWorker1_DoWork(object sender, DoWorkEventArgs e)
        {
            AnalogInputModule aiModule = e.Argument as AnalogInputModule;
            aiModule.ReadValuesFromModule();
            e.Result = aiModule;
        }

        private async void BackgroundWorker1_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            if (_isFormClosing)
            {
                return;
            }

            AnalogInputModule aiModule = e.Result as AnalogInputModule;
            _moduleIsBusy = false;
            if (aiModule.AdqError == AdqErrors.ADQ_ERROR_NONE)
            {
                try
                {
                    _intervalsRead++;
                    CalculateFft();
                    DisplayValues(aiModule, _intervalsRead);
                    await WriteCsvValuesAsync(_csvFileWriter);
                    if (_stopAfterIntervals > 0 && _stopAfterIntervals == _intervalsRead)
                    {
                        StopReading();
                        EnableChartsTooltipFunctions();
                        AddInfoTextLine(string.Format("Stopped after {0} {1}.", _stopAfterIntervals, GetIntervalsText()));
                        return;
                    }
                    if (_isIntervalReading)
                    {
                        timerPause.Start();
                    }
                }
                catch (Exception ex)
                {
                    StopReading();
                    DisplayExceptionMessage(ex);
                }
            }
            else
            {
                DisplayErrorText("adqReadBuffer", aiModule.AdqError);
                StopReading();
                tabControl1.SelectedTab = tpInfo;
            }
        }

        private void FormMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            StopReading();
            if (_moduleIsBusy)
            {
                _isFormClosing = true;
                _selectedAiModule.Stop();
            }
            e.Cancel = false;
        }

        private void BtnClose_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void FormMain_FormClosed(object sender, FormClosedEventArgs e)
        {
            CloseAdqDriver();
        }

        private void CloseAdqDriver()
        {
            if (_moduleLocator != null)
            {
                _moduleLocator.CloseAdqDriver();
            }
        }

        private void BtnClear_Click(object sender, EventArgs e)
        {
            ClearDisplayedValues();
        }

        private void ClearDisplayedValues()
        {
            chartUserConfigurable.Series.Clear();
            chartCalculated.Series.Clear();
            chartFftCoherent.Series.Clear();
            textBoxFftValues.Clear();
            textBoxFftValuesCoherent.Clear();
            textBoxProgress.Clear();
            SetProgressBarStyle(false);
            textBoxInfo.Clear();
        }

        private void TextBoxInfo_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Modifiers == Keys.Control && e.KeyCode == Keys.A)
            {
                textBoxInfo.SelectionStart = 0;
                textBoxInfo.SelectionLength = textBoxInfo.Text.Length;
            }
        }

        private void BtnInfo_Click(object sender, EventArgs e)
        {
            var aboutBox = new AboutBox();
            aboutBox.ShowDialog();
        }

        private void TsMnuItemSelectAll_Click(object sender, EventArgs e)
        {
            CheckListViewItems(true);
        }

        private void TsMnuItemInvertSelection_Click(object sender, EventArgs e)
        {
            for (int i = 0; i < lvChannelList.Items.Count; i++)
            {
                lvChannelList.Items[i].Checked = !lvChannelList.Items[i].Checked;
            }
        }

        private void TsMnuItemSelectNone_Click(object sender, EventArgs e)
        {
            CheckListViewItems(false);
        }

        private void CheckListViewItems(bool check)
        {
            for (int i = 0; i < lvChannelList.Items.Count; i++)
            {
                lvChannelList.Items[i].Checked = check;
            }
        }

        private void LvChannelList_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (_isIntervalReading)
            {
                e.NewValue = e.CurrentValue;
            }
        }

        private void LvChannelList_ItemChecked(object sender, ItemCheckedEventArgs e)
        {
            if (_selectedAiModule == null)
            {
                return;
            }

            _selectedAiModule.FrameLength = (uint)lvChannelList.CheckedItems.Count;
            textBoxNumberOfSelectedChannels.Text = _selectedAiModule.FrameLength.ToString();
            UpdateNumberOfValues();
            EnableFftUi(false);
        }

        private void AddTextToCombobox(ComboBox coBox, string text)
        {
            if (!coBox.Items.Contains(text))
            {
                coBox.Items.Insert(0, text);
            }
        }

        private void InitTimeValueComboBox()
        {
            coBoxTimeUnit.Items.Clear();
            coBoxTimeUnit.Items.Add(new StringValueItem<int>(_txtTimeMs, 1));
            coBoxTimeUnit.Items.Add(new StringValueItem<int>(_txtTimeSec, 1000));
            coBoxTimeUnit.Items.Add(new StringValueItem<int>(_txtTimeMin, 60 * 1000));
            coBoxTimeUnit.Items.Add(new StringValueItem<int>(_txtTimeHour, 60 * 60 * 1000));
            coBoxTimeUnit.SelectedIndex = 0;
        }

        private void InitWindowFunctionComboBox()
        {
            coBoxWindowFunction.Items.Clear();
            coBoxWindowFunction.Items.Add(new StringValueItem<uint>(_txtFftWindowHann, AdqUtilityConstants.FFT_WINDOW_FUNCTION_HANN));
            coBoxWindowFunction.Items.Add(new StringValueItem<uint>(_txtFftWindowHamming, AdqUtilityConstants.FFT_WINDOW_FUNCTION_HAMMING));
            coBoxWindowFunction.Items.Add(new StringValueItem<uint>(_txtFftWindowRectangular, AdqUtilityConstants.FFT_WINDOW_FUNCTION_RECTANGULAR));
            coBoxWindowFunction.Items.Add(new StringValueItem<uint>(_txtFftWindowBartlett, AdqUtilityConstants.FFT_WINDOW_FUNCTION_BARTLETT));
            coBoxWindowFunction.Items.Add(new StringValueItem<uint>(_txtFftWindowFlatTop, AdqUtilityConstants.FFT_WINDOW_FUNCTION_FLAT_TOP));
            coBoxWindowFunction.Items.Add(new StringValueItem<uint>(_txtFftWindowHarrisNutall, AdqUtilityConstants.FFT_WINDOW_FUNCTION_HARRIS_NUTALL));
            coBoxWindowFunction.Items.Add(new StringValueItem<uint>(_txtFftWindowBlackmanHarris7, AdqUtilityConstants.FFT_WINDOW_FUNCTION_BLACKMAN_HARRIS_7_TERM));
            coBoxWindowFunction.SelectedIndex = 0;
        }

        private void InitCalculatedValuesComboBox()
        {
            coBoxCalculatedType.Items.Clear();
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtAverageValues, SignalValuesType.AverageValues));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtRmsValues, SignalValuesType.RMSValues));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtMinValues, SignalValuesType.MinimumValues));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtMaxValues, SignalValuesType.MaximumValues));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtPeek2PeekValues, SignalValuesType.P2PValues));

            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtMainFrequencies, SignalValuesType.MainFrequencies));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtFrequencyProportions, SignalValuesType.FrequProportions));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtThdValues, SignalValuesType.ThdValues));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtThdrNValues, SignalValuesType.ThdrPlusNValues));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtThdfNValues, SignalValuesType.ThdfPlusNValues));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtRmsSpectrum, SignalValuesType.RMSValuesSpectrum));

            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtMainFrequCoh, SignalValuesType.MainFrequenciesCoherent));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtFrequPropCoh, SignalValuesType.FrequProportionsCoherent));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtThdValuesCoh, SignalValuesType.ThdValuesCoherent));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtThdrNValuesCoh, SignalValuesType.ThdrPlusNValuesCoherent));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtThdfNValuesCoh, SignalValuesType.ThdfPlusNValuesCoherent));
            coBoxCalculatedType.Items.Add(new StringValueItem<SignalValuesType>(_txtRmsSpectrumCoh, SignalValuesType.RMSValuesSpectrumCoherent));
            coBoxCalculatedType.SelectedIndex = 0;
        }

        private void StopReading()
        {
            _isIntervalReading = false;
            timerPause.Stop();
            SetProgressBarStyle(false);
            EnableUI(true);
            btnStartStop.Text = "Start";
        }

        private void BtnSelectCsvFolder_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(folderBrowserDialog1.SelectedPath))
            {
                folderBrowserDialog1.SelectedPath = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments);
            }
            var result = folderBrowserDialog1.ShowDialog();
            if (result == DialogResult.OK)
            {
                _csvFilesFolder = folderBrowserDialog1.SelectedPath;
                textBoxCsvFileFolder.Text = _csvFilesFolder;
            }
        }

        private void BtnSaveInfo_Click(object sender, EventArgs e)
        {
            saveFileDialog1.FileName = GetInfoTextFileName(_selectedAiModule, ".txt");
            saveFileDialog1.InitialDirectory = _csvFilesFolder;
            saveFileDialog1.Filter = "Text files (*.txt)|*.txt|All files (*.*)|*.*";
            DialogResult result = saveFileDialog1.ShowDialog();
            if (result == DialogResult.OK)
            {
                string name = saveFileDialog1.FileName;
                File.WriteAllText(name, textBoxInfo.Text);
            }
        }

        private string GetImageFileName(AnalogInputModule aiModule, string fileExt)
        {
            string s = string.Empty;
            if (aiModule != null)
            {
                s = string.Format("{0}, Board {1}, Module {2}, ",
                    aiModule.ModelName,
                    aiModule.BoardIndex,
                    aiModule.ModuleIndex);
            }
            return s + DateTime.Now.ToString("yyyy-MM-dd HH.mm.ss") + fileExt;
        }

        private ChartImageFormat GetImageFormat(int filterIndex)
        {
            ChartImageFormat imgFormat;
            switch (filterIndex)
            {
                case 1:
                    imgFormat = ChartImageFormat.Png;
                    break;
                case 2:
                    imgFormat = ChartImageFormat.Gif;
                    break;
                case 3:
                    imgFormat = ChartImageFormat.Bmp;
                    break;
                case 4:
                    imgFormat = ChartImageFormat.Tiff;
                    break;
                case 5:
                    imgFormat = ChartImageFormat.Jpeg;
                    break;
                default:
                    imgFormat = ChartImageFormat.Png;
                    break;
            }
            return imgFormat;
        }

        private string _imgInitialDirectory = string.Empty;

        private void TsMnuItemSaveGraphic_Click(object sender, EventArgs e)
        {
            saveFileDialog1.FileName = GetImageFileName(_selectedAiModule, ".png");
            saveFileDialog1.InitialDirectory = _imgInitialDirectory;
            saveFileDialog1.Filter = "PNG files (*.png)|*.png|Gif files (*.gif)|*.gif|BMP files (*.bmp)|*.bmp|Tiff files (*.tif)|*.tif|Jpg files (*.jpg)|*.jpg";
            DialogResult result = saveFileDialog1.ShowDialog();
            if (result == DialogResult.OK)
            {
                _imgInitialDirectory = Path.GetDirectoryName(saveFileDialog1.FileName);
                ChartImageFormat imgFormat = GetImageFormat(saveFileDialog1.FilterIndex);
                if (tabControl1.SelectedTab == tpChartCalculatedValues)
                {
                    chartCalculated.SaveImage(saveFileDialog1.FileName, imgFormat);
                }
                else if (tabControl1.SelectedTab == tpChartFftCoherent)
                {
                    chartFftCoherent.SaveImage(saveFileDialog1.FileName, imgFormat);
                }
                else if (tabControl1.SelectedTab == tpChartFftAndAllValues)
                {
                    chartUserConfigurable.SaveImage(saveFileDialog1.FileName, imgFormat);
                }
            }
        }

        private void TsMnuItemCopyToClipboard_Click(object sender, EventArgs e)
        {
            using (MemoryStream ms = new MemoryStream())
            {

                if (tabControl1.SelectedTab == tpChartCalculatedValues)
                {
                    chartCalculated.SaveImage(ms, ChartImageFormat.Png);
                }
                else if (tabControl1.SelectedTab == tpChartFftCoherent)
                {
                    chartFftCoherent.SaveImage(ms, ChartImageFormat.Png);
                }
                else if (tabControl1.SelectedTab == tpChartFftAndAllValues)
                {
                    chartUserConfigurable.SaveImage(ms, ChartImageFormat.Png);
                }
                Clipboard.SetImage(new Bitmap(ms));
            }
        }

        private void ChartUserConfigurable_MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if (coBoxChartGraphicToDisplay.SelectedIndex == _idxGraphsToDisplayAllValuesAndDBUTogether &&
                chartUserConfigurable.ChartAreas.Count > 0 &&
                chartUserConfigurable.Series.Count > 1)
            {
                if (chartUserConfigurable.Titles.Count > 0)
                {
                    chartUserConfigurable.Titles.RemoveAt(chartUserConfigurable.Titles.Count - 1);
                }

                var title = new Title();
                chartUserConfigurable.Titles.Add(title);
                if (chartUserConfigurable.ChartAreas[0].CursorY.AxisType == AxisType.Primary)
                {
                    chartUserConfigurable.ChartAreas[0].CursorX.AxisType = AxisType.Secondary;
                    chartUserConfigurable.ChartAreas[0].CursorY.AxisType = AxisType.Secondary;
                    InitChartInfo(title, chartUserConfigurable.ChartAreas[0], "Double click to change between Graph zooming, " + "All Values Selected");
                }
                else
                {
                    chartUserConfigurable.ChartAreas[0].CursorX.AxisType = AxisType.Primary;
                    chartUserConfigurable.ChartAreas[0].CursorY.AxisType = AxisType.Primary;
                    InitChartInfo(title, chartUserConfigurable.ChartAreas[0], "Double click to change between Graph zooming, " + "dBu Selected");
                }
            }
        }

        private void CoBoxChartGraphsToDisplay_SelectedIndexChanged(object sender, EventArgs e)
        {
            ClearDisplayedValues();
        }

        private void CoBoxChartTypeUserConfigurable_SelectedIndexChanged(object sender, EventArgs e)
        {
            _updateFftChartNeeded = true;
        }

        private void CoBoxChartGraphicCoherent_SelectedIndexChanged(object sender, EventArgs e)
        {
            ClearDisplayedValues();
        }

        private void CoBoxChartTypeGraphicCoherent_SelectedIndexChanged(object sender, EventArgs e)
        {
            _updateFftChartNeeded = true;
        }
    }
}