﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the AISingle project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System;
using System.Collections.Generic;

namespace AISingle.Utility
{
    public static class AdqCommon
    {
        public static uint NotOpen = AdqErrors.ADQ_ERROR_TYPE_ERROR | AdqErrors.ADQ_ERROR_NOT_OPEN;

        public static string GetPhysicalUnitSymbol(uint physicalUnit)
        {
            return GetDictionaryValue(_physicalUnitSymbols, physicalUnit);
        }

        private static string GetDictionaryValue(IDictionary<uint, string> dictionary, uint key)
        {
            bool found = dictionary.TryGetValue(key, out string stringValue);
            if (!found)
            {
                stringValue = string.Format("0x{0:X8}", key);
            }
            return stringValue;
        }

        private static readonly Dictionary<uint, string> _physicalUnitSymbols = new Dictionary<uint, string>
        {
            {AdqConstants.ADQ_PHYSICAL_UNIT_VOLT, "V"},
            {AdqConstants.ADQ_PHYSICAL_UNIT_AMPERE, "A"},
            {AdqConstants.ADQ_PHYSICAL_UNIT_OHM, "Ω"}
        };

        public static string GetModelName(string boardName)
        {
            int posADQ = boardName.IndexOf("ADQ");
            if (posADQ < 0) posADQ = 0;

            int posEnd = boardName.IndexOf(" ", posADQ);
            if (posEnd < 0)
            {
                posEnd = boardName.Length;
            }

            return boardName.Substring(posADQ, posEnd - posADQ);
        }

        public static string FormatPhysicalValue(double physicalValue)
        {
            string physicalTxt;
            string fmt = "{0:0.######}{1}";
            double tmpVal = Math.Abs(physicalValue);
            if (tmpVal == 0)
            {
                physicalTxt = "0";
            }
            else if (tmpVal < 0.001)
            {
                physicalTxt = string.Format(fmt, physicalValue * 1000000, "µ");
            }
            else if (tmpVal < 1)
            {
                physicalTxt = string.Format(fmt, physicalValue * 1000, "m");
            }
            else
            {
                physicalTxt = string.Format(fmt, Math.Round(physicalValue, 6), "");
            }
            return physicalTxt;
        }
    }
}
