﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the CommonSettings project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using CommonSettings.Utility;
using System;

namespace CommonSettings.ALLDAQ
{
    public class AdqModule
    {
        public AdqModule(uint boardIndex, uint moduleIndex)
        {
            BoardIndex = boardIndex;
            ModuleIndex = moduleIndex;
        }

        public event EventHandler<InfoTextChangedEventArgs> OnInfoTextChanged;

        public uint BoardIndex { get; private set; }
        public uint ModuleIndex { get; private set; }

        public bool Reset()
        {
            uint adqError = AdqDriver.adqResetModule(BoardIndex, ModuleIndex);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                RaiseInfoTextChanged(AdqCommon.FormatFunctionErrorNumber("adqResetModule", adqError));
            }
            return adqError == AdqErrors.ADQ_ERROR_NONE;
        }

        protected void SetFunctionErrorInfo(string functionName, uint adqError)
        {
            RaiseInfoTextChanged(AdqCommon.FormatFunctionErrorNumber(functionName, adqError));
        }

        protected virtual void RaiseInfoTextChanged(string text)
        {
            OnInfoTextChanged?.Invoke(this, new InfoTextChangedEventArgs(text));
        }
    }
}
