﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the CommonSettings project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using CommonSettings.Utility;
using ALLDAQ.Boards;

namespace CommonSettings.ALLDAQ
{
    public class AnalogModule : AdqModule
    {
        public AnalogModule(string boardName, uint boardIndex, uint moduleIndex, uint moduleType)
            : base(boardIndex, moduleIndex)
        {  
            BuildModuleName(boardName, moduleType);
        }

        public string ModuleName { get; private set; }

        private void BuildModuleName(string boardName, uint moduleType)
        {
            ModuleName = string.Format("{0}, Board {1}, Module {2}, Type {3}",
                AdqCommon.GetModelName(boardName),
                BoardIndex,
                ModuleIndex,
                AdqCommon.GetModuleTypeText(moduleType));
        }

        public bool GetSelectedCalibration(out uint calibrationID)
        {
            uint adqError = AdqDriver.adqAnalogGetSelectedCalibration(BoardIndex, ModuleIndex, out calibrationID);
            if(adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqAnalogGetSelectedCalibration", adqError);
            }
            return adqError == AdqErrors.ADQ_ERROR_NONE;
        }

        public bool SetSelectedCalibration(uint calibrationID)
        {
            uint adqError = AdqDriver.adqAnalogSelectCalibration(BoardIndex, ModuleIndex, calibrationID);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqAnalogSelectedCalibration", adqError);
            }
            return adqError == AdqErrors.ADQ_ERROR_NONE;
        }

    }
}
