﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the CommonSettings project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using CommonSettings.Utility;
using System.Collections.Generic;

namespace CommonSettings.ALLDAQ
{
    public class DigitalInputModule : AdqModule
    {
        public DigitalInputModule(string boardName, uint boardIndex, uint moduleIndex,
            ref AdqDriver.ADQDIModuleInfo adqDiModuleInfo)
            : base(boardIndex, moduleIndex)
        {
            BuildModuleName(boardName);
            FillFilterTimes(ref adqDiModuleInfo);
        }

        public string ModuleName { get; private set; }
        public List<ulong> FilterTimes { get; } = new List<ulong>();

        private void FillFilterTimes(ref AdqDriver.ADQDIModuleInfo adqDiModuleInfo)
        {
            for (uint idx = 0; idx < adqDiModuleInfo.NumberOfFilters; idx++)
            {
                FilterTimes.Add(adqDiModuleInfo.FilterTimePicoseconds[idx]);
            }
        }

        private void BuildModuleName(string boardName)
        {
            ModuleName = string.Format("{0}, Board {1}, Module {2}",
                AdqCommon.GetModelName(boardName),
                BoardIndex,
                ModuleIndex);
        }

        public bool GetDigitalFilter(out ulong filterPicoseconds)
        {
            uint adqError = AdqDriver.adqDIOGetFilterPicoseconds(BoardIndex, ModuleIndex, out filterPicoseconds);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqDIOGetFilterPicoseconds", adqError);
            }
            return adqError == AdqErrors.ADQ_ERROR_NONE;
        }

        public bool SetDigitalFilter(ulong filterPicoseconds)
        {
            uint adqError = AdqDriver.adqDIOSetFilterPicoseconds(BoardIndex, ModuleIndex, filterPicoseconds);
            if (adqError != AdqErrors.ADQ_ERROR_NONE)
            {
                SetFunctionErrorInfo("adqDIOSetFilterPicoseconds", adqError);
            }
            return adqError == AdqErrors.ADQ_ERROR_NONE;
        }
    }
}
