﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//      This file contains a programming example for Intel 82C54 ICs
//      used on ALLDAQ Data Acquisition Boards
//-----------------------------------------------------------------------------

using Counter8254.Utility;
using System.Collections.Generic;

namespace Counter8254.ALLDAQ
{
    public class Counter8254Modules
    {
        private readonly string _modelName;
        private readonly List<CounterModule> _counters = new List<CounterModule>();

        public Counter8254Modules(uint boardIndex, string boardName)
        {
            BoardIndex = boardIndex;
            _modelName = AdqCommon.GetModelName(boardName);
        }

        public uint BoardIndex { get; }

        public CounterModule GetCounter(int index)
        {
            if (index < _counters.Count)
            {
                return _counters[index];
            }
            return null;
        }

        public void AddCounter(CounterModule counter)
        {
            _counters.Add(counter);
        }

        private string _countersName = string.Empty;
        public override string ToString()
        {
            if (string.IsNullOrEmpty(_countersName))
            {
                _countersName = string.Format("{0}, Board {1}, Counter module", _modelName, BoardIndex);
                for (int i = 0; i < _counters.Count; i++)
                {
                    if (i > 0)
                    {
                        _countersName += string.Format(", {0}", _counters[i].ModuleIndex);
                    }
                    else
                    {
                        _countersName += string.Format(" {0}", _counters[i].ModuleIndex);
                    }
                }
            }
            return _countersName;
        }
    }
}
