﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//      This file contains a programming example for Intel 82C54 ICs
//      used on ALLDAQ Data Acquisition Boards
//-----------------------------------------------------------------------------

using ALLDAQ.Boards;
using System.Collections.Generic;

namespace Counter8254.ALLDAQ
{
    public class CounterModule
    {
        private readonly uint _boardIndex;

        public CounterModule(uint boardIndex, uint moduleIndex, uint widthBits, uint clockSources)
        {
            _boardIndex = boardIndex;
            ModuleIndex = moduleIndex;
            CounterMaxValue = (uint)((1UL << (int)widthBits) - 1);
            SetClockSourceValues(clockSources);
        }

        public uint ModuleIndex { get; }
        public uint CounterMaxValue { get; }
        public List<uint> ClockSourceValues { get; } = new List<uint>();

        public uint Config(uint clockSource, uint counterMode, uint counterStartValue)
        {
            return AdqDriver.adqCNTRConfig(_boardIndex, ModuleIndex, clockSource, counterMode, counterStartValue);
        }

        public uint ReadSingle(out uint value)
        {
            return AdqDriver.adqReadSingle(_boardIndex, ModuleIndex, out value);
        }

        public uint Reset()
        {
            return AdqDriver.adqResetModule(_boardIndex, ModuleIndex);
        }

        private void SetClockSourceValues(uint clockSources)
        {
            AddClockSourceValue(clockSources, AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL);
            AddClockSourceValue(clockSources, AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ);
        }

        private void AddClockSourceValue(uint clockSources, uint clockSourceConstant)
        {
            if ((clockSources & clockSourceConstant) == clockSourceConstant)
            {
                ClockSourceValues.Add(clockSourceConstant);
            }
        }
    }
}
