﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;

namespace WriteBufferAO.Models
{
    public class ChargeDischargeWaveGenerator : ISignalGenerator
    {
        private const int _cycleTimesToCharge = 12;

        public string GeneratorIcon
        {
            get
            {
                return "/WriteBufferAO;component/Images/signalExponential.png";
            }
        }

        public string GeneratorName
        {
            get
            {
                return "Charge, discharge";
            }
        }

        public double GetValue(double cycleTime, double signalMinimum, double signalMaximum)
        {
            double signalRange = signalMaximum - signalMinimum;
            if (cycleTime <= 0.5)
            {
                return signalMinimum + signalRange * (1 - Math.Pow(Math.E, -cycleTime * _cycleTimesToCharge));
            }
            return signalMinimum + signalRange * Math.Pow(Math.E, -(cycleTime - 0.5) * _cycleTimesToCharge);
        }
    }
}
