﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;

namespace WriteBufferAO.Models
{
    public class GaussianNoiseSignalGenerator : ISignalGenerator
    {
        private readonly Random _random = new Random();

        public string GeneratorIcon
        {
            get
            {
                return "/WriteBufferAO;component/Images/signalGaussNoise.png";
            }
        }

        public string GeneratorName
        {
            get
            {
                return "Gaussian noise";
            }
        }

        public double GetValue(double cycleTime, double signalMinimum, double signalMaximum)
        {
            //https://en.wikipedia.org/wiki/Marsaglia_polar_method
            double stdDev = (signalMaximum - signalMinimum) / 8.0;
            double mean = (signalMaximum + signalMinimum) / 2.0;

            double u1, u2, r;
            while (true)
            {
                do
                {
                    u1 = 2 * _random.NextDouble() - 1;
                    u2 = 2 * _random.NextDouble() - 1;
                    r = u1 * u1 + u2 * u2;
                } while (r >= 1 || r == 0);

                double polar = Math.Sqrt(-2.0 * Math.Log(r) / r);
                double value = polar * u1 * stdDev + mean;
                if (value >= signalMinimum && value <= signalMaximum)
                {
                    return value;
                }
            }
        }
    }
}
