﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;
using WriteBufferAO.Utility;

namespace WriteBufferAO.Models
{
    public class MorletWaveletGenerator : ISignalGenerator
    {
        private const double _constant = 0.95;
        public string GeneratorIcon
        {
            get
            {
                return "/WriteBufferAO;component/Images/signalMorletWavelet.png";
            }
        }

        public string GeneratorName
        {
            get
            {
                return "Morlet wavelet";
            }
        }

        public double GetValue(double cycleTime, double signalMinimum, double signalMaximum)
        {
            // https://en.wikipedia.org/wiki/Morlet_wavelet
            // Morlet wavelet real part
            double x = cycleTime * 6.0 - 3.0;
            double xSquared = x * x;
            return (signalMaximum + signalMinimum) / 2.0 + (signalMaximum - signalMinimum) / 2.0 * Math.Pow(Math.E, -xSquared / 2.0) * Math.Cos(AdqCommon.TwoPI * _constant * x);

            // Morlet wavelet imaginary part
            //return (signalMaximum + signalMinimum) / 2.0 + (signalMaximum - signalMinimum) / 2.0 * Math.Pow(Math.E, -xSquared / 2.0) * Math.Sin(AdqCommon.TwoPI * _constant * x);
        }
    }
}
