﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;

namespace WriteBufferAO.Models
{
    public class NonReturnToZeroWaveGenerator : ISignalGenerator
    {
        private Random _random = new Random();
        private int _value = 0;

        public string GeneratorIcon
        {
            get
            {
                return "/WriteBufferAO;component/Images/signalNonZero.png";
            }
        }

        public string GeneratorName
        {
            get
            {
                return "Non-return-to-zero";
            }
        }

        public double GetValue(double cycleTime, double signalMinimum, double signalMaximum)
        {
            // http://en.wikipedia.org/wiki/Non-return-to-zero
            if (cycleTime == 0)
            {
                _value = 1 - _random.Next(2);
            }

            if (_value > 0)
            {
                return signalMaximum;
            }
            return signalMinimum;
        }
    }
}
