﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;

namespace WriteBufferAO.Models
{
    public class PoissonWaveletGenerator : ISignalGenerator
    {
        public string GeneratorIcon
        {
            get
            {
                return "/WriteBufferAO;component/Images/signalPoissonWavelet.png";
            }
        }

        public string GeneratorName
        {
            get
            {
                return "Poisson wavelet";
            }
        }

        public double GetValue(double cycleTime, double signalMinimum, double signalMaximum)
        {
            // https://en.wikipedia.org/wiki/Poisson_wavelet
            double x = cycleTime * 18.0 - 9.0;
            double xSquared = x * x;
            return (signalMaximum + signalMinimum) / 2.0 + (signalMaximum - signalMinimum) / 2.0 * ((1 - xSquared) / Math.Pow(1 + xSquared, 2));
        }
    }
}
