﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;

namespace WriteBufferAO.Models
{
    public class ShannonWaveletGenerator : ISignalGenerator
    {
        private const double _piHalf = Math.PI / 2.0;

        public string GeneratorIcon
        {
            get
            {
                return "/WriteBufferAO;component/Images/signalShannonWavelet.png";
            }
        }

        public string GeneratorName
        {
            get
            {
                return "Shannon wavelet";
            }
        }

        public double GetValue(double cycleTime, double signalMinimum, double signalMaximum)
        {
            // https://en.wikipedia.org/wiki/Shannon_wavelet
            double x = cycleTime * 8 - 4.0;
            double k = _piHalf * x;
            double val;
            if (k == 0)
            {
                val = (signalMaximum + signalMinimum) / 2.0 + (signalMaximum - signalMinimum) / 2.0;
            }
            else
            {
                val = (signalMaximum + signalMinimum) / 2.0 + (signalMaximum - signalMinimum) / 2.0 * (Math.Sin(k) / k) * Math.Cos(3 * k);
            }
            return val;
        }
    }
}
