﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// http://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferAO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;
using WriteBufferAO.Utility;

namespace WriteBufferAO.Models
{
    public class SincWaveGenerator : ISignalGenerator
    {
        public string GeneratorIcon
        {
            get
            {
                return "/WriteBufferAO;component/Images/signalSinc.png";
            }
        }

        public string GeneratorName
        {
            get
            {
                return "Sinc function";
            }
        }

        public double GetValue(double cycleTime, double signalMinimum, double signalMaximum)
        {
            // https://en.wikipedia.org/wiki/Sinc_function
            double offset = (signalMaximum + signalMinimum) / 2.0;
            double factor = (signalMaximum - signalMinimum) / 2.0;

            double x = AdqCommon.TwoPI * (cycleTime - 0.5) * 10;
            if (x == 0) return offset + factor;

            return offset + factor * Math.Sin(x) / x;
        }
    }
}
