﻿//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the WriteBufferDO project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

using System;

namespace WriteBufferDO.Models
{
    public class RandomValueGenerator : IValueGenerator
    {
        private Random _random;
        public string GeneratorName => "Random";

        public uint[] GenerateValues(uint portWidthBits, uint numberOfValues, bool lastValueZero)
        {
            _random = new Random();
            uint[] values;
            if (portWidthBits < 32)
            {
                values = GenerateRandomValuesInt(portWidthBits, numberOfValues);
            }
            else
            {
                values = GenerateRandomValuesUint(numberOfValues);
            }

            if (lastValueZero)
            {
                values[numberOfValues - 1] = 0;
            }
            return values;
        }

        private uint[] GenerateRandomValuesInt(uint portWidthBits, uint numberOfValues)
        {
            int maxValue = (1 << (int)portWidthBits);
            uint[] values = new uint[numberOfValues];
            for (int i = 0; i < numberOfValues; i++)
            {
                values[i] = (uint)_random.Next(maxValue);
            }
            return values;
        }

        private uint[] GenerateRandomValuesUint(uint numberOfValues)
        {
            uint[] values = new uint[numberOfValues];

            for (int i = 0; i < numberOfValues; i++)
            {
                int number = _random.Next(int.MinValue, int.MaxValue);
                values[i] = (uint)(number + (uint)int.MaxValue);
            }
            return values;
        }
    }
}
