﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the AIReadContinuous project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports AIReadContinuous.ALLDAQ.Boards

Public Class HardwareTrigger
    Private _hardwareTriggerEnable As AdqDriver.ADQHWTriggerEnable = New AdqDriver.ADQHWTriggerEnable(AdqDriver.ADQHWTriggerEnable_ID)
    Private ReadOnly _boardIndex As UInteger
    Private ReadOnly _moduleIndex As UInteger

    Public Sub New(boardIndex As UInteger, moduleIndex As UInteger, ByRef hwTriggerInfo As AdqDriver.ADQHWTriggerInfo)
        _boardIndex = boardIndex
        _moduleIndex = moduleIndex
        TriggerStartSource = hwTriggerInfo.HWTriggerStartSource
        TriggerStopSource = hwTriggerInfo.HWTriggerStopSource
        SupportsChannelListPerTrigger = hwTriggerInfo.SupportsChannelListPerTrigger
        TriggerTimeoutMilliseconds = AdqCommon.DefaultTriggerTimeout
        TriggerDescription = AdqCommon.GetTriggerDescription(hwTriggerInfo)
    End Sub

    Public ReadOnly Property TriggerDescription As String
    ReadOnly Property TriggerStartSource As UInteger
    ReadOnly Property TriggerStopSource As UInteger
    ReadOnly Property SupportsChannelListPerTrigger As Boolean

    Public Property TriggerStartRisingEdges As UInteger
        Get
            Return _hardwareTriggerEnable.HWTriggerStartRisingEdge
        End Get
        Set(value As UInteger)
            _hardwareTriggerEnable.HWTriggerStartRisingEdge = value
        End Set
    End Property

    Public Property TriggerStartFallingEdges As UInteger
        Get
            Return _hardwareTriggerEnable.HWTriggerStartFallingEdge
        End Get
        Set(value As UInteger)
            _hardwareTriggerEnable.HWTriggerStartFallingEdge = value
        End Set
    End Property

    Public Property TriggerStopRisingEdges As UInteger
        Get
            Return _hardwareTriggerEnable.HWTriggerStopRisingEdge
        End Get
        Set(value As UInteger)
            _hardwareTriggerEnable.HWTriggerStopRisingEdge = value
        End Set
    End Property

    Public Property TriggerStopFallingEdges As UInteger
        Get
            Return _hardwareTriggerEnable.HWTriggerStopFallingEdge
        End Get
        Set(value As UInteger)
            _hardwareTriggerEnable.HWTriggerStopFallingEdge = value
        End Set
    End Property

    Public Property ChannelListPerTrigger As Boolean
        Get
            Return _hardwareTriggerEnable.ChannelListPerTrigger
        End Get
        Set(value As Boolean)
            _hardwareTriggerEnable.ChannelListPerTrigger = value
        End Set
    End Property

    Public Property TriggerAutomaticRearm As Boolean
        Get
            Return _hardwareTriggerEnable.AutomaticRearm
        End Get
        Set(value As Boolean)
            _hardwareTriggerEnable.AutomaticRearm = value
        End Set
    End Property

    Public Property UseTriggerTimeout As Boolean
    Public Property TriggerTimeoutMilliseconds As UInteger

    ReadOnly Property StartTriggerEnabled As Boolean
        Get
            Return _hardwareTriggerEnable.HWTriggerStartRisingEdge <> 0 OrElse _hardwareTriggerEnable.HWTriggerStartFallingEdge <> 0
        End Get
    End Property
    Public Function EnableHardwareTrigger() As Boolean
        Dim timeout As UInteger = GetTimeoutValue()

        Dim adqError As UInteger = AdqDriver.adqEnableHWTrigger(_boardIndex, _moduleIndex, _hardwareTriggerEnable, True, timeout)
        If adqError <> AdqErrors.ADQ_ERROR_NONE Then
            Throw New ApplicationException(String.Format("Function adqEnableHWTrigger returned Error number: 0x{0:X}", adqError))
        End If

        Return adqError = AdqErrors.ADQ_ERROR_NONE
    End Function

    Private Function GetTimeoutValue() As UInteger
        Dim timeout As UInteger = 0
        If _hardwareTriggerEnable.HWTriggerStartRisingEdge <> 0 OrElse _hardwareTriggerEnable.HWTriggerStartFallingEdge <> 0 Then
            timeout = If(UseTriggerTimeout, TriggerTimeoutMilliseconds, AdqConstants.ADQ_HW_START_TRIGGER_TIMEOUT_INFINITE)
        End If
        Return timeout
    End Function
End Class
