﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'      This file contains a programming example for Intel counter 82C54 ICs
'      on ALLDAQ Data Acquisition Boards
'-----------------------------------------------------------------------------

Public Class Counter8254Modules
    Private ReadOnly _boardIndex As UInteger
    Private ReadOnly _modelName As String
    Private ReadOnly _counters As List(Of CounterModule) = New List(Of CounterModule)
    Private _countersName As String = String.Empty

    Public Sub New(boardIndex As UInteger, boardName As String)
        _boardIndex = boardIndex
        _modelName = AdqCommon.GetModelName(boardName)
    End Sub

    Public Function GetCounter(index As Integer) As CounterModule
        If index < _counters.Count Then
            Return _counters(index)
        End If
        Return Nothing
    End Function

    Public Sub AddCounter(counter As CounterModule)
        _counters.Add(counter)
    End Sub

    Public Overrides Function ToString() As String
        If String.IsNullOrEmpty(_countersName) Then
            _countersName = String.Format("{0}, Board {1}, Counter module", _modelName, _boardIndex)
            For i As Integer = 0 To _counters.Count - 1
                If i > 0 Then
                    _countersName &= String.Format(", {0}", _counters(i).ModuleIndex)
                Else
                    _countersName &= String.Format(" {0}", _counters(i).ModuleIndex)
                End If
            Next
        End If
        Return _countersName
    End Function
End Class

