﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'      This file contains a programming example for Intel counter 82C54 ICs
'      on ALLDAQ Data Acquisition Boards
'-----------------------------------------------------------------------------

Imports Counter8254.ALLDAQ.Boards

Public Class AdqCommon
    Public Shared ReadOnly NotOpen As UInteger = AdqErrors.ADQ_ERROR_TYPE_FATAL Or AdqErrors.ADQ_ERROR_NOT_OPEN
    Public Shared ReadOnly Counter8254Modes As UInteger() = {AdqConstants.ADQ_CNTR_8254_MODE_0,
                                                             AdqConstants.ADQ_CNTR_8254_MODE_1,
                                                             AdqConstants.ADQ_CNTR_8254_MODE_2,
                                                             AdqConstants.ADQ_CNTR_8254_MODE_3,
                                                             AdqConstants.ADQ_CNTR_8254_MODE_4,
                                                             AdqConstants.ADQ_CNTR_8254_MODE_5}

    Public Shared Function GetCounterClockSourceName(counterClockSource As UInteger) As String
        Return GetDictionaryValue(_counterClockSourceNames, counterClockSource)
    End Function

    Private Shared Function GetDictionaryValue(dictionary As IDictionary(Of UInteger, String), key As UInteger) As String
        Dim stringValue As String = String.Empty
        Dim found As Boolean = dictionary.TryGetValue(key, stringValue)
        If Not found Then
            stringValue = String.Format("&H{0:X8}", key)
        End If
        Return stringValue
    End Function

    Private Shared ReadOnly _counterClockSourceNames As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_DEFAULT, "Default"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_EXTERNAL, "External"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_10_MHZ, "Internal 10 MHz"},
            {AdqConstants.ADQ_CNTR_CLOCK_SOURCE_INTERNAL_66_MHZ, "Internal 66 MHz"}
        }

    Public Shared Function GetModelName(boardName As String) As String
        Dim posADQ As Integer = boardName.IndexOf("ADQ")
        If posADQ < 0 Then
            posADQ = 0
        End If

        Dim posEnd As Integer = boardName.IndexOf(" ", posADQ)
        If posEnd < 0 Then
            posEnd = boardName.Length
        End If

        Return boardName.Substring(posADQ, posEnd - posADQ)
    End Function
End Class
