﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the DISingle project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports DISingle.ALLDAQ.Boards

Public Class AdqCommon
    Public Shared ReadOnly NotOpen As UInteger = AdqErrors.ADQ_ERROR_TYPE_FATAL Or AdqErrors.ADQ_ERROR_NOT_OPEN
    Private Shared ReadOnly _binStrings As String() = {"0000", "0001", "0010", "0011",
                                                     "0100", "0101", "0110", "0111",
                                                     "1000", "1001", "1010", "1011",
                                                     "1100", "1101", "1110", "1111"}
    Private Shared ReadOnly _bitCounts As UInteger() = {1, 8, 16, 24, 32}

    Public Shared Function IntToBin(value As UInteger, bitCount As UInteger) As String
        Dim tmpStr As String = ""

        If _bitCounts.Contains(bitCount) Then
            If bitCount = 1 Then
                tmpStr = (value And 1).ToString()
            Else
                For i As Integer = CInt(bitCount / 4) To 1 Step -1
                    Dim binIdx As Integer = CInt((value >> ((i - 1) * 4)) And &HF)
                    tmpStr &= _binStrings(binIdx)
                Next
            End If
        End If
        Return tmpStr
    End Function

    Public Shared Function GetModuleTypeName(moduleType As UInteger) As String
        Return GetDictionaryValue(_moduleTypeNames, moduleType)
    End Function

    Private Shared Function GetDictionaryValue(dictionary As IDictionary(Of UInteger, String), key As UInteger) As String
        Dim stringValue As String = String.Empty
        Dim found As Boolean = dictionary.TryGetValue(key, stringValue)
        If Not found Then
            stringValue = String.Format("&H{0:X8}", key)
        End If
        Return stringValue
    End Function

    Private Shared ReadOnly _moduleTypeNames As Dictionary(Of UInteger, String) =
        New Dictionary(Of UInteger, String) From
        {
            {AdqConstants.ADQ_MODULE_TYPE_DIO, "DIO"},
            {AdqConstants.ADQ_MODULE_TYPE_DO, "DO"},
            {AdqConstants.ADQ_MODULE_TYPE_DI, "DI"},
            {AdqConstants.ADQ_MODULE_TYPE_CNTR, "Counter"},
            {AdqConstants.ADQ_MODULE_TYPE_EXTINT, "External Interrupt"},
            {AdqConstants.ADQ_MODULE_TYPE_AI, "AI"},
            {AdqConstants.ADQ_MODULE_TYPE_AO, "AO"},
            {AdqConstants.ADQ_MODULE_TYPE_I2C, "I2C"},
            {AdqConstants.ADQ_MODULE_TYPE_PWM, "PWM"},
            {AdqConstants.ADQ_MODULE_TYPE_FIN, "FIN"},
            {AdqConstants.ADQ_MODULE_TYPE_I2SO, "I2SO"}
        }

    Public Shared Function GetModelName(boardName As String) As String
        Dim posADQ As Integer = boardName.IndexOf("ADQ")
        If posADQ < 0 Then
            posADQ = 0
        End If

        Dim posEnd As Integer = boardName.IndexOf(" ", posADQ)
        If posEnd < 0 Then
            posEnd = boardName.Length
        End If

        Return boardName.Substring(posADQ, posEnd - posADQ)
    End Function
End Class
