﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the ReadBuffer project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports ReadBuffer.ALLDAQ.Boards

Public Class AnalogInputModule
    Inherits InputModule

    Private ReadOnly _ranges As List(Of AnalogRange) = New List(Of AnalogRange)

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
                   ByRef generalInfo As AdqDriver.ADQGeneralModuleInfo, ByRef aiModuleInfo As AdqDriver.ADQAIModuleInfo)
        MyBase.New(boardName, boardIndex, moduleIndex, generalInfo)
        NumberOfChannels = aiModuleInfo.NumberOfChannels
        NumberOfRanges = aiModuleInfo.NumberOfRanges
        AddRanges(aiModuleInfo)
        _selectedRange = _ranges(0)
        _moduleName = GetModuleName()
    End Sub

    Private Sub AddRanges(ByRef aiModuleInfo As AdqDriver.ADQAIModuleInfo)
        Dim rangeIdx As Integer = 0
        While rangeIdx < aiModuleInfo.NumberOfRanges
            Dim aiRange As AdqDriver.ADQAIRange = aiModuleInfo.Ranges(rangeIdx)
            Dim physicalRange = New AnalogRange(
                            aiRange.PhysicalUnit,
                            aiRange.DigitalMin,
                            aiRange.DigitalMax,
                            aiRange.PhysicalMin,
                            aiRange.PhysicalMax)
            _ranges.Add(physicalRange)

            rangeIdx += 1
        End While
    End Sub

    Private Function GetModuleName() As String
        Return String.Format("{0}, Board {1}, Module {2}, Type {3}, {4} {5}, {6} clock, max. {7}",
            ModelName,
            BoardIndex,
            ModuleIndex,
            AdqCommon.GetModuleTypeName(ModuleType),
            NumberOfChannels,
            If(NumberOfChannels = 1, "Channel", "Channels"),
            AdqCommon.GetClockTypeName(ClockType),
            AdqCommon.FormatFrequency(MaxfrequencyHz))
    End Function

    ReadOnly Property NumberOfChannels As UInteger

    Public Function GetRange(index As Integer) As AnalogRange
        Return _ranges(index)
    End Function

    Private _selectedRange As AnalogRange
    ReadOnly Property SelectedRange As AnalogRange
        Get
            Return _selectedRange
        End Get
    End Property

    Private _selectedChannelIndex As UInteger
    ReadOnly Property SelectedChannelIndex As UInteger
        Get
            Return _selectedChannelIndex
        End Get
    End Property

    Private _selectedRangeIndex As UInteger
    ReadOnly Property SelectedRangeIndex As UInteger
        Get
            Return _selectedRangeIndex
        End Get
    End Property

    ReadOnly Property NumberOfRanges() As UInteger

    Function ConfigureModule(channelIndex As UInteger, rangeIndex As UInteger) As UInteger
        Dim adqError As UInteger = AdqDriver.adqAISetChannelAndRange(BoardIndex, ModuleIndex, channelIndex, rangeIndex)
        If adqError = AdqErrors.ADQ_ERROR_NONE Then
            _selectedChannelIndex = channelIndex
            _selectedRangeIndex = rangeIndex
            _selectedRange = _ranges(CInt(rangeIndex))
        End If
        Return adqError
    End Function
End Class
