﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the ReadBuffer project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Imports System.Runtime.InteropServices
Imports ReadBuffer.ALLDAQ.Boards

Public Class DigitalInputModule
    Inherits InputModule

    Public Sub New(boardName As String, boardIndex As UInteger, moduleIndex As UInteger,
                   ByRef generalInfo As AdqDriver.ADQGeneralModuleInfo, portWidthBits As UInteger)
        MyBase.New(boardName, boardIndex, moduleIndex, generalInfo)
        Me.PortWidthBits = portWidthBits
        _moduleName = GetModuleName()
    End Sub

    Public ReadOnly Property PortWidthBits As UInteger

    Public Function GetModuleName() As String
        Return String.Format("{0}, Board {1}, Module {2}, Type {3}, {4} Bit, {5} clock, max. {6}",
            ModelName,
            BoardIndex,
            ModuleIndex,
            AdqCommon.GetModuleTypeName(ModuleType),
            PortWidthBits,
            AdqCommon.GetClockTypeName(ClockType),
            AdqCommon.FormatFrequency(MaxfrequencyHz))
    End Function

    Function ConfigureModule() As UInteger
        If ModuleType = AdqConstants.ADQ_MODULE_TYPE_DIO Then
            Dim adqError As UInteger = AdqDriver.adqDIOSetDirection(
                    BoardIndex,
                    ModuleIndex,
                    AdqConstants.ADQ_DIO_INPUT)
            If adqError <> AdqErrors.ADQ_ERROR_NONE Then
                ErrorTextChanged("adqDIOSetDirection", adqError)
                Return adqError
            End If
        End If
        Return AdqErrors.ADQ_ERROR_NONE
    End Function
End Class
