﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Public Class RandomRectangleWaveGenerator
    Implements ISignalGenerator

    Private ReadOnly _random As Random = New Random()
    Private _firstSignalPartGenerated As Boolean
    Private _secondSignalPartGenerated As Boolean
    Private _randomValue As Double

    Public ReadOnly Property GeneratorIcon As String Implements ISignalGenerator.GeneratorIcon
        Get
            Return "/WriteBufferAO;component/Images/signalRandomRectangle.png"
        End Get
    End Property

    Public ReadOnly Property GeneratorName As String Implements ISignalGenerator.GeneratorName
        Get
            Return "Random rectangle"
        End Get
    End Property

    Public Function GetValue(cycleTime As Double, signalMinimum As Double, signalMaximum As Double) As Double Implements ISignalGenerator.GetValue
        If cycleTime <= 0.5 Then
            If (Not _firstSignalPartGenerated) Then
                _firstSignalPartGenerated = True
                _secondSignalPartGenerated = False
                _randomValue = _random.NextDouble() * (signalMaximum - signalMinimum) + signalMinimum
            End If
        Else
            If Not _secondSignalPartGenerated Then
                _firstSignalPartGenerated = False
                _secondSignalPartGenerated = True
                _randomValue = _random.NextDouble() * (signalMaximum - signalMinimum) + signalMinimum
            End If
        End If
        Return _randomValue
    End Function
End Class
