﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferAO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Public Class WaveForm
    Private _outputFrequency As Double
    Private _signalLengthSeconds As Double

    Public Sub InitWaveForm(outputFrequency As Double, signalLengthSeconds As Double, numberOfValues As UInteger)
        _outputFrequency = outputFrequency
        _signalLengthSeconds = signalLengthSeconds
        If _values Is Nothing OrElse _values.Length <> numberOfValues Then
            _values = New Double(CInt(numberOfValues - 1UI)) {}
        End If
        Array.Clear(_values, 0, _values.Length)
    End Sub

    Private _values As Double()
    Public ReadOnly Property Values As Double()
        Get
            Return _values
        End Get
    End Property

    Public Sub ApplySignalGenerator(signalGenerator As ISignalGenerator, signalSettings As SignalSettings)
        Dim frequencyShift As Double = (signalSettings.SignalEndFrequency - signalSettings.SignalBeginFrequency) / _signalLengthSeconds
        Dim physicalMinShift As Double = (signalSettings.SignalEndPhysicalMin - signalSettings.SignalBeginPhysicalMin) / _signalLengthSeconds
        Dim physicalMaxShift As Double = (signalSettings.SignalEndPhysicalMax - signalSettings.SignalBeginPhysicalMax) / _signalLengthSeconds

        For i = 0 To _values.Length - 1
            Dim time As Double = i / _outputFrequency
            Dim signalFrequency As Double = signalSettings.SignalBeginFrequency + time * frequencyShift
            Dim signalPhysicalMin As Double = signalSettings.SignalBeginPhysicalMin + time * physicalMinShift
            Dim signalPhysicalMax As Double = signalSettings.SignalBeginPhysicalMax + time * physicalMaxShift
            Dim timeOffset As Double = time * signalFrequency + signalSettings.PhaseShift
            Dim cycleTime As Double = timeOffset - Math.Floor(timeOffset)

            Dim currValue As Double = _values(i) + signalGenerator.GetValue(cycleTime, signalPhysicalMin, signalPhysicalMax)

            If currValue < signalSettings.AoRangePhysicalMin Then
                currValue = signalSettings.AoRangePhysicalMin
            ElseIf currValue > signalSettings.AoRangePhysicalMax Then
                currValue = signalSettings.AoRangePhysicalMax
            End If

            _values(i) = currValue
        Next
    End Sub
End Class
