﻿'-----------------------------------------------------------------------------
' ALLDAQ, a division of ALLNET GmbH
' https://www.alldaq.com
'
' Purpose:
'       This file Is part of the WriteBufferDO project of the ALLDAQ-SDK.  
'       It shows the usage of the API functions to access
'       ALLDAQ Data Acquisition Boards
' 
'-----------------------------------------------------------------------------

Public Class RandomValueGenerator
    Implements IValueGenerator

    Private _random As Random
    Public ReadOnly Property GeneratorName As String Implements IValueGenerator.GeneratorName
        Get
            Return "Random"
        End Get
    End Property

    Public Function GenerateValues(portWidthBits As UInteger, numberOfValues As UInteger, lastValueZero As Boolean) As UInteger() _
        Implements IValueGenerator.GenerateValues

        _random = New Random()
        Dim values As UInteger()
        If portWidthBits < 32 Then
            values = GenerateRandomValuesInt(portWidthBits, numberOfValues)
        Else
            values = GenerateRandomValuesUint(numberOfValues)
        End If

        If lastValueZero Then
            values(CInt(numberOfValues - 1UI)) = 0
        End If

        Return values
    End Function

    Private Function GenerateRandomValuesInt(portWidthBits As UInteger, numberOfValues As UInteger) As UInteger()
        Dim maxValue = (1 << CInt(portWidthBits))
        Dim values As UInteger() = New UInteger(CInt(numberOfValues - 1UI)) {}
        Dim idx As Integer = 0
        While idx < numberOfValues
            values(idx) = CUInt(_random.Next(maxValue))
            idx += 1
        End While
        Return values
    End Function

    Private Function GenerateRandomValuesUint(numberOfValues As UInteger) As UInteger()
        Dim values As UInteger() = New UInteger(CInt(numberOfValues - 1UI)) {}
        Dim number As Integer
        Dim idx As Integer = 0
        While idx < numberOfValues
            number = _random.Next(Integer.MinValue, Integer.MaxValue)
            values(idx) = CUInt(number + CUInt(Integer.MaxValue))
            idx += 1
        End While
        Return values
    End Function
End Class
