#include <windows.h>

#include <stdio.h>
#include <stdlib.h>

#include <tchar.h>

#include <conio.h>

#include "..\\adqDevice.h"

#ifdef WIN64

#pragma comment(lib, "..\\adqDevice_64.lib")

#else

#pragma comment(lib, "..\\adqDevice.lib")

#endif

struct SAIModule
{
    unsigned int    m_uiBoardIndex;

    unsigned int    m_uiModuleIndex;

    unsigned int    m_uiNumberOfChannels;

    unsigned int    m_uiNumberOfRanges;

    SADQAIRange     m_arrRanges[ADQ_MAX_AI_RANGES];
};

struct SAIChannelCalibration
{
    unsigned int    m_uiChannelIndex;

    double          m_dFactor;

    double          m_dConstant;

    wchar_t*        m_wszPhysicalUnit;
};

int _tmain(int argc, _TCHAR* argv[])
{
   // Increase the size of the console buffer
    
    HANDLE  h_console = GetStdHandle(STD_OUTPUT_HANDLE);

    COORD coord;

    coord.X = 160;
    
    coord.Y = 9999;
    
    SetConsoleScreenBufferSize(h_console, coord);

    bool b_terminate = false;

    bool b_remote = false;

    wchar_t wsz_remote_name_or_IP_address[128];

    while(true)
    {
        printf( "1. Open local\n"
                "2. Open remote\n"
                "\n"
                "Your choice: 1 or 2 (RETURN to exit): ");

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            b_terminate = true;
            
            break;
        }

        unsigned int ui_choice = 0;

        if( sscanf_s(&sz_response[0], "%u", &ui_choice) != 1)
        {
            continue;
        }

        if(ui_choice == 1)
        {
            break;
        }

        if(ui_choice == 2)
        {
            b_remote = true;

            printf("Name or IP-Address (RETURN to exit): ");

            _getws_s(&wsz_remote_name_or_IP_address[0], 128);

            printf("\n\n");

            if( wsz_remote_name_or_IP_address[0] == (wchar_t)0 )
            {
                b_terminate = true;
            }

            break;
        }
    }

    if(b_terminate)
    {
        printf("Press any key to exit.\n");

        _getch();

        exit(-1);
    }

    unsigned int ui_adq_error;
    
    if(b_remote)
    {
        wchar_t* pwc_user_name_or_address[2] = {&wsz_remote_name_or_IP_address[0],
                                                NULL                                };

        ui_adq_error = adqOpenEx(pwc_user_name_or_address);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpenEx returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }
    else
    {
        ui_adq_error = adqOpen();

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpen returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }

    unsigned int ui_number_of_boards;
    
    ui_adq_error = adqGetNumberOfBoards(ui_number_of_boards);

    if(ui_adq_error != ADQ_ERROR_NONE)
    {
        printf("adqGetNumberOfBoards returned 0x%08X\n\n", ui_adq_error);

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    if(ui_number_of_boards == 0)
    {
        printf("No ADQ boards found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    printf("%3u ADQ Boards found\n", ui_number_of_boards);

    // Count the number of AI modules

    unsigned int ui_number_of_ai_modules = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_index_board, ui_index_module, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if(ui_module_type == ADQ_MODULE_TYPE_AI)
            {
                ++ui_number_of_ai_modules;
            }
        }
    }

    if(ui_number_of_ai_modules == 0)
    {
        printf("No AI modules found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    unsigned int ui_max_number_of_ai_modules = ui_number_of_ai_modules;

    SAIModule* arr_ai_modules = new SAIModule[ui_max_number_of_ai_modules];

    ui_number_of_ai_modules = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_index_board, ui_index_module, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            SADQGeneralModuleInfo s_adq_general_module_info;

            s_adq_general_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_adq_general_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            if(ui_module_type == ADQ_MODULE_TYPE_AI)
            {
                SADQAIModuleInfo s_adq_ai_module_info;

                s_adq_ai_module_info.m_uiSize = sizeof(SADQAIModuleInfo);

                s_adq_ai_module_info.m_uiID = SADQAIModuleInfo_ID;

                unsigned int ui_info_size;

                ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQAIModuleInfo), (unsigned char*)&s_adq_ai_module_info, ui_info_size);

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    continue;
                }

                if(s_adq_ai_module_info.m_bRangeInfoPerChannel)
                {
                    printf("        Board index: %3u Module index: %3u Board has variable ranges. Test program must be extended\n\n", ui_index_board, ui_index_module);

                    printf("Press any key to exit.\n");

                    adqClose();

                    _getch();

                    exit(-1);
                }

                if(s_adq_ai_module_info.m_uiNumberOfRanges > 1)
                {
                    printf("        Board index: %3u Module index: %3u Board has multiple ranges. Test program must be extended\n\n", ui_index_board, ui_index_module);

                    printf("Press any key to exit.\n");

                    adqClose();

                    _getch();

                    delete arr_ai_modules;

                    exit(-1);
                }

                arr_ai_modules[ui_number_of_ai_modules].m_uiBoardIndex = ui_index_board;

                arr_ai_modules[ui_number_of_ai_modules].m_uiModuleIndex = ui_index_module;

                arr_ai_modules[ui_number_of_ai_modules].m_uiNumberOfChannels = s_adq_ai_module_info.m_uiNumberOfChannels;

                arr_ai_modules[ui_number_of_ai_modules].m_uiNumberOfRanges = s_adq_ai_module_info.m_uiNumberOfRanges;

                memcpy( &arr_ai_modules[ui_number_of_ai_modules].m_arrRanges[0],  &s_adq_ai_module_info.m_arrRanges[0], arr_ai_modules[ui_number_of_ai_modules].m_uiNumberOfRanges * sizeof(SADQAIRange) );

                ++ui_number_of_ai_modules;
            }
        }
    }

    while(true)
    {
        printf("Please choose an AI module:\n\n");

        unsigned int ui_index_ai_module;

        for(ui_index_ai_module = 0; ui_index_ai_module < ui_number_of_ai_modules; ui_index_ai_module++)
        {
            printf("%2u.  Board index: %d  Module index: %-2d  Number of channels: %-2d  Number of ranges: %-2d\n",
                   ui_index_ai_module + 1,
                   arr_ai_modules[ui_index_ai_module].m_uiBoardIndex,
                   arr_ai_modules[ui_index_ai_module].m_uiModuleIndex,
                   arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels,
                   arr_ai_modules[ui_index_ai_module].m_uiNumberOfRanges                                                            );
        }

        printf("\n");

        printf("Your choice 1 - %d (RETURN to exit): ",
               ui_number_of_ai_modules                  );

        char sz_response[256];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            break;
        }

        if( sscanf_s(&sz_response[0], "%u", &ui_index_ai_module) != 1)
        {
            continue;
        }

        if( (ui_index_ai_module < 1)||(ui_index_ai_module > ui_number_of_ai_modules) )
        {
            continue;
        }

        --ui_index_ai_module;

        SADQAIChannel* p_arr_s_channels = new SADQAIChannel[ arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels ];

        memset( &p_arr_s_channels[0], 0, arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels * sizeof(SADQAIChannel) );

        bool b_continue = false;

        while(true)
        {
            printf( "The module has %u channels, please enter a comma separated channel list - e.g 0,2, %u (RETURN to exit): ",
                    arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels,
                    arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels - 1                                                 );
                 
            gets_s(&sz_response[0], 256);

            printf("\n\n");

            if(sz_response[0] == 0)
            {
                b_continue = true;
                    
                break;
            }

            char* p_c_ch_context;
            
            char* p_ch_channel = strtok_s(&sz_response[0], ",.", &p_c_ch_context);
            
            while(p_ch_channel != NULL)
            {
                unsigned int ui_index_channel;

                if( sscanf_s(p_ch_channel, "%u", &ui_index_channel) != 1)
                {
                    continue;
                }

                if(ui_index_channel >= arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels)
                {
                    printf("Invalid channel: %u - Press any key to continue\n\n", ui_index_channel);

                    b_continue = true;

                    break;
                }

                p_arr_s_channels[ui_index_channel].m_bAcquireChannel = true;

                p_arr_s_channels[ui_index_channel].m_uiRangeIndex = 0;

                p_ch_channel = strtok_s(NULL, ",.", &p_c_ch_context);
            }

            break;
        }

        if(b_continue)
        {
            delete p_arr_s_channels;

            continue;
        }

        double d_actual_frequency = 0.0;

        ui_adq_error = adqAISetChannelListAndFrequency(arr_ai_modules[ui_index_ai_module].m_uiBoardIndex, arr_ai_modules[ui_index_ai_module].m_uiModuleIndex, p_arr_s_channels, arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels, 0, d_actual_frequency);
        
        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf( "        Board index: %3u Module index: %3u - adqAISetChannelListAndFrequency returned 0x%08X\n\n", 
                    arr_ai_modules[ui_index_ai_module].m_uiBoardIndex,
                    arr_ai_modules[ui_index_ai_module].m_uiModuleIndex,
                    ui_adq_error                                                                                        );

           delete p_arr_s_channels;

           printf("Press any key to continue.\n");

           _getch();

            continue;
        }

        unsigned int ui_frame_length = 0;

        for(unsigned int ui_index_channel = 0; ui_index_channel < arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels; ui_index_channel++)
        {
            if(p_arr_s_channels[ui_index_channel].m_bAcquireChannel)
            {
                ++ui_frame_length;
            }
        }

        SAIChannelCalibration* p_arr_s_calibrations = new SAIChannelCalibration[ui_frame_length];

        unsigned int ui_frame_index = 0;

        for(unsigned int ui_index_channel = 0; ui_index_channel < arr_ai_modules[ui_index_ai_module].m_uiNumberOfChannels; ui_index_channel++)
        {
            if(p_arr_s_channels[ui_index_channel].m_bAcquireChannel)
            {
                p_arr_s_calibrations[ui_frame_index].m_uiChannelIndex = ui_index_channel;

                unsigned int ui_index_range = p_arr_s_channels[ui_index_channel].m_uiRangeIndex;

                p_arr_s_calibrations[ui_frame_index].m_dFactor = (arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_dPhysicalMax - arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_dPhysicalMin) / 
                                            (double)(arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiDigitalMax - arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiDigitalMin);
        
                p_arr_s_calibrations[ui_frame_index].m_dConstant = arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_dPhysicalMin - (double)arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiDigitalMin * p_arr_s_calibrations[ui_frame_index].m_dFactor; 

                if(arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_VOLT)
                {
                    p_arr_s_calibrations[ui_frame_index].m_wszPhysicalUnit = L"Volt";
                }
                else if(arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_AMPERE)
                {
                    p_arr_s_calibrations[ui_frame_index].m_wszPhysicalUnit = L"Ampere";
                }
                else // if(arr_ai_modules[ui_index_ai_module].m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_OHM)
                {
                    p_arr_s_calibrations[ui_frame_index].m_wszPhysicalUnit = L"Ohm";
                }

                ++ui_frame_index;
            }
        }

        printf( "Press ESC to continue or any other key to read from board index: %u, module index: %u, channels ",
                arr_ai_modules[ui_index_ai_module].m_uiBoardIndex, 
                arr_ai_modules[ui_index_ai_module].m_uiModuleIndex                                                      );

        for(unsigned int ui_frame_index = 0; ui_frame_index < ui_frame_length; ui_frame_index++)
        {
            printf("%d ", p_arr_s_calibrations[ui_frame_index].m_uiChannelIndex);
        }

        printf("\n\n");

        unsigned int* p_ui_frame = new unsigned int [ui_frame_length];

        while(true)
        {
            if(_getch() == 27)
            {
                break;
            }

            ui_adq_error = adqAIReadSingle(arr_ai_modules[ui_index_ai_module].m_uiBoardIndex, arr_ai_modules[ui_index_ai_module].m_uiModuleIndex, ui_frame_length, p_ui_frame);

            for(unsigned int ui_index_channel = 0; ui_index_channel < ui_frame_length; ui_index_channel++)
            {
                printf("Channel %3d         ", p_arr_s_calibrations[ui_index_channel].m_uiChannelIndex);
            }

            printf("\n");

            for(unsigned int ui_index_channel = 0; ui_index_channel < ui_frame_length; ui_index_channel++)
            {
                printf("-----------         ");
            }

            printf("\n\n");

            for(unsigned int ui_index_channel = 0; ui_index_channel < ui_frame_length; ui_index_channel++)
            {
                printf("Digital   %7S   ", p_arr_s_calibrations[ui_index_channel].m_wszPhysicalUnit);
            }

            printf("\n");

            for(unsigned int ui_index_channel = 0; ui_index_channel < ui_frame_length; ui_index_channel++)
            {
                printf("-------   -------   ");
            }

            printf("\n\n");

            for(unsigned int ui_index_channel = 0; ui_index_channel < ui_frame_length; ui_index_channel++)
            {
                unsigned int ui_data = p_ui_frame[ui_index_channel];

                double d_data = (double)ui_data * p_arr_s_calibrations[ui_index_channel].m_dFactor + p_arr_s_calibrations[ui_index_channel].m_dConstant;

                printf("%7u   %7.3f   ",
                        ui_data,
                        d_data              );
            }

            printf("\n\n\n");
        }

        delete p_ui_frame;

        delete p_arr_s_calibrations;

        delete p_arr_s_channels;

        printf("Press any key to continue\n\n");

        _getch();
    }

    printf("\n\n");

    delete []arr_ai_modules;

    adqClose();

    printf("Press any key to terminate.\n");

    _getch();

    return 0;
}

