#include <windows.h>

#include <stdio.h>
#include <stdlib.h>

#include <tchar.h>

#include <conio.h>

#include "..\\adqDevice.h"

#ifdef WIN64

#pragma comment(lib, "..\\adqDevice_64.lib")

#else

#pragma comment(lib, "..\\adqDevice.lib")

#endif

struct SDigitalModule
{
    unsigned int    m_uiBoardIndex;

    unsigned int    m_uiModuleIndex;

    unsigned int    m_uiModuleType;

    unsigned int    m_uiPortWidthBits;

    bool            m_bIsOptoIsolated;

    bool            m_bIsTTL;
};

void ReadFromDigitalModule(SDigitalModule* pDigitalModule);

void WriteToDigitalModule(SDigitalModule* pDigitalModule);

int _tmain(int argc, _TCHAR* argv[])
{
   // Increase the size of the console buffer
    
    HANDLE  h_console = GetStdHandle(STD_OUTPUT_HANDLE);

    COORD coord;

    coord.X = 160;
    
    coord.Y = 9999;
    
    SetConsoleScreenBufferSize(h_console, coord);

    bool b_terminate = false;

    bool b_remote = false;

    wchar_t wsz_remote_name_or_IP_address[128];

    while(true)
    {
        printf( "1. Open local\n"
                "2. Open remote\n"
                "\n"
                "Your choice: 1 or 2 (RETURN to exit): ");

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            b_terminate = true;
            
            break;
        }

        unsigned int ui_choice = 0;

        if( sscanf_s(&sz_response[0], "%u", &ui_choice) != 1)
        {
            continue;
        }

        if(ui_choice == 1)
        {
            break;
        }

        if(ui_choice == 2)
        {
            b_remote = true;

            printf("Name or IP-Address (RETURN to exit): ");

            _getws_s(&wsz_remote_name_or_IP_address[0], 128);

            printf("\n\n");

            if( wsz_remote_name_or_IP_address[0] == (wchar_t)0 )
            {
                b_terminate = true;
            }

            break;
        }
    }

    if(b_terminate)
    {
        printf("Press any key to exit.\n");

        _getch();

        exit(-1);
    }

    unsigned int ui_adq_error;
    
    if(b_remote)
    {
        wchar_t* pwc_user_name_or_address[2] = {&wsz_remote_name_or_IP_address[0],
                                                NULL                                };

        ui_adq_error = adqOpenEx(pwc_user_name_or_address);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpenEx returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }
    else
    {
        ui_adq_error = adqOpen();

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpen returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }

    unsigned int ui_number_of_boards;
    
    ui_adq_error = adqGetNumberOfBoards(ui_number_of_boards);

    if(ui_adq_error != ADQ_ERROR_NONE)
    {
        printf("adqGetNumberOfBoards returned 0x%08X\n\n", ui_adq_error);

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    if(ui_number_of_boards == 0)
    {
        printf("No ADQ boards found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    printf("%3u ADQ Boards found\n", ui_number_of_boards);

    // Count the number of DI, DO or DIO modules

    unsigned int ui_number_of_digital_modules = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_index_board, ui_index_module, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if( (ui_module_type == ADQ_MODULE_TYPE_DI)||(ui_module_type == ADQ_MODULE_TYPE_DO)||(ui_module_type == ADQ_MODULE_TYPE_DIO) )
            {
                ++ui_number_of_digital_modules;
            }
        }
    }

    if(ui_number_of_digital_modules == 0)
    {
        printf("No DI, DO or DIO modules found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    unsigned int ui_max_number_of_digital_modules = ui_number_of_digital_modules;

    SDigitalModule* arr_digital_modules = new SDigitalModule[ui_max_number_of_digital_modules];

    ui_number_of_digital_modules = 0;

    for(unsigned int ui_index_board = 0; ui_index_board < ui_number_of_boards; ui_index_board++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_index_board, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned 0x%08X\n\n", ui_index_board, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_index_module = 0; ui_index_module < ui_number_of_modules; ui_index_module++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_index_board, ui_index_module, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned 0x%08X\n\n", ui_index_board, ui_index_module, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            SADQGeneralModuleInfo s_adq_general_module_info;

            s_adq_general_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_adq_general_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            if( (ui_module_type == ADQ_MODULE_TYPE_DI)||(ui_module_type == ADQ_MODULE_TYPE_DO)||(ui_module_type == ADQ_MODULE_TYPE_DIO) )
            {
                unsigned int ui_info_size;

                arr_digital_modules[ui_number_of_digital_modules].m_uiBoardIndex = ui_index_board;

                arr_digital_modules[ui_number_of_digital_modules].m_uiModuleIndex = ui_index_module;

                arr_digital_modules[ui_number_of_digital_modules].m_uiModuleType = ui_module_type;

                if(ui_module_type == ADQ_MODULE_TYPE_DI)
                {
                    SADQDIModuleInfo s_adq_di_module_info;

                    s_adq_di_module_info.m_uiSize = sizeof(SADQDIModuleInfo);

                    s_adq_di_module_info.m_uiID = SADQDIModuleInfo_ID;

                    ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQDIModuleInfo), (unsigned char*)&s_adq_di_module_info, ui_info_size);

                    if(ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index (DI module): %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board + 1, ui_index_module + 1, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    arr_digital_modules[ui_number_of_digital_modules].m_uiPortWidthBits = s_adq_di_module_info.m_uiPortWidthBits;

                    arr_digital_modules[ui_number_of_digital_modules].m_bIsOptoIsolated = s_adq_di_module_info.m_bIsOptoIsolated;

                    arr_digital_modules[ui_number_of_digital_modules].m_bIsTTL = s_adq_di_module_info.m_bIsTTL;
                }
                else if(ui_module_type == ADQ_MODULE_TYPE_DO)
                {
                    SADQDOModuleInfo s_adq_do_module_info;

                    s_adq_do_module_info.m_uiSize = sizeof(SADQDOModuleInfo);

                    s_adq_do_module_info.m_uiID = SADQDOModuleInfo_ID;

                    ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQDOModuleInfo), (unsigned char*)&s_adq_do_module_info, ui_info_size);

                    if(ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index (DO module): %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board + 1, ui_index_module + 1, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    arr_digital_modules[ui_number_of_digital_modules].m_uiPortWidthBits = s_adq_do_module_info.m_uiPortWidthBits;

                    arr_digital_modules[ui_number_of_digital_modules].m_bIsOptoIsolated = s_adq_do_module_info.m_bIsOptoIsolated;

                    arr_digital_modules[ui_number_of_digital_modules].m_bIsTTL = s_adq_do_module_info.m_bIsTTL;
                }
                else // if(ui_module_type == ADQ_MODULE_TYPE_DIO)
                {
                    SADQDIOModuleInfo s_adq_dio_module_info;

                    s_adq_dio_module_info.m_uiSize = sizeof(SADQDIOModuleInfo);

                    s_adq_dio_module_info.m_uiID = SADQDIOModuleInfo_ID;

                    ui_adq_error = adqGetModuleInfo(ui_index_board, ui_index_module, s_adq_general_module_info, sizeof(SADQDIOModuleInfo), (unsigned char*)&s_adq_dio_module_info, ui_info_size);

                    if(ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_index_board + 1, ui_index_module + 1, ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        continue;
                    }

                    arr_digital_modules[ui_number_of_digital_modules].m_uiPortWidthBits = s_adq_dio_module_info.m_uiPortWidthBits;

                    arr_digital_modules[ui_number_of_digital_modules].m_bIsOptoIsolated = s_adq_dio_module_info.m_bIsOptoIsolated;

                    arr_digital_modules[ui_number_of_digital_modules].m_bIsTTL = s_adq_dio_module_info.m_bIsTTL;
                }

                ++ui_number_of_digital_modules;
            }
        }
    }

    while(true)
    {
        printf("Please choose a module:\n\n");

        unsigned int ui_index_digital_module;

        for(ui_index_digital_module = 0; ui_index_digital_module < ui_number_of_digital_modules; ui_index_digital_module++)
        {
            char* sz_module_type;

            if(arr_digital_modules[ui_index_digital_module].m_uiModuleType == ADQ_MODULE_TYPE_DI)
            {
                sz_module_type = " DI";
            }
            else if(arr_digital_modules[ui_index_digital_module].m_uiModuleType == ADQ_MODULE_TYPE_DO)
            {
                sz_module_type = " DO";
            }
            else // if(arr_digital_modules[ui_index_digital_module].m_uiModuleType == ADQ_MODULE_TYPE_DIO)
            {
                sz_module_type = "DIO";
            }

            printf("%2u.  Board index: %d  Module index: %-2d  Module type: %s  Port width: %-2d bits  %s  %s\n",
                   ui_index_digital_module + 1,
                   arr_digital_modules[ui_index_digital_module].m_uiBoardIndex,
                   arr_digital_modules[ui_index_digital_module].m_uiModuleIndex,
                   sz_module_type,
                   arr_digital_modules[ui_index_digital_module].m_uiPortWidthBits,
                   arr_digital_modules[ui_index_digital_module].m_bIsOptoIsolated ? "Opto isolated" : "             ",
                   arr_digital_modules[ui_index_digital_module].m_bIsTTL ? "TTL" : "   "                              );
        }

        printf("\n");

        printf("Your choice 1 - %d (RETURN to exit): ",
               ui_number_of_digital_modules             );

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            break;
        }

        if( sscanf_s(&sz_response[0], "%u", &ui_index_digital_module) != 1)
        {
            continue;
        }

        if( (ui_index_digital_module < 1)||(ui_index_digital_module > ui_number_of_digital_modules) )
        {
            continue;
        }

        --ui_index_digital_module;

        if(arr_digital_modules[ui_index_digital_module].m_uiModuleType == ADQ_MODULE_TYPE_DI)
        {
            printf("Press any key to read from Board index: %u, Module index %u\n\n",
                   arr_digital_modules[ui_index_digital_module].m_uiBoardIndex,
                   arr_digital_modules[ui_index_digital_module].m_uiModuleIndex              );

            _getch();
           
           ReadFromDigitalModule(&arr_digital_modules[ui_index_digital_module]);
        }
        else if(arr_digital_modules[ui_index_digital_module].m_uiModuleType == ADQ_MODULE_TYPE_DO)
        {
            WriteToDigitalModule(&arr_digital_modules[ui_index_digital_module]);
        }
        else // if(arr_digital_modules[ui_index_digital_module].m_uiModuleType == ADQ_MODULE_TYPE_DIO)
        {
            unsigned int ui_direction;
            
            ui_adq_error = adqDIOGetDirection(  arr_digital_modules[ui_index_digital_module].m_uiBoardIndex,
                                                arr_digital_modules[ui_index_digital_module].m_uiModuleIndex,
                                                ui_direction                                                );

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("adqDIOGetDirection returned 0x%08X\n\n", ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            char ch_user;

            while(true)
            {
                printf("Board index: %u, Module index %u - Port direction is set to %s\n\n",
                       arr_digital_modules[ui_index_digital_module].m_uiBoardIndex,
                       arr_digital_modules[ui_index_digital_module].m_uiModuleIndex,
                       (ui_direction == ADQ_DIO_INPUT) ? "Input" : "Output"                 );

                printf("(C)hange direction to %s\n",
                       (ui_direction == ADQ_DIO_INPUT) ? "Output" : "Input" );

                if(ui_direction == ADQ_DIO_INPUT)
                {
                    printf("(R)ead from port\n\n");

                    printf("Your choice C or R (ESC to cancel):");
                }
                else // if(ui_direction == ADQ_DIO_OutPUT)
                {
                    printf("(W)rite to port\n\n");

                    printf("Your choice C or W (ESC to cancel):");
                }

                ch_user = _getch();

                printf("\n\n");

                if( (ch_user == 'c') ||(ch_user == 'C') )
                {
                    ui_adq_error = adqDIOSetDirection(  arr_digital_modules[ui_index_digital_module].m_uiBoardIndex,
                                                        arr_digital_modules[ui_index_digital_module].m_uiModuleIndex,
                                                        (ui_direction == ADQ_DIO_INPUT) ? ADQ_DIO_OUTPUT : ADQ_DIO_INPUT    );

                    if(ui_adq_error != ADQ_ERROR_NONE)
                    {
                        printf("adqDIOSetDirection returned 0x%08X\n\n", ui_adq_error);

                        printf("Press any key to continue.\n");

                        _getch();

                        break;
                    }

                    printf("Direction set to %s\n\n",
                           (ui_direction == ADQ_DIO_INPUT) ? "Output" : "Input" );

                    break;
                }
                else if( (ui_direction == ADQ_DIO_INPUT)&&( (ch_user == 'r')||(ch_user == 'R') ) )
                {
                    ReadFromDigitalModule(&arr_digital_modules[ui_index_digital_module]);

                    break;
                }
                else if( (ui_direction == ADQ_DIO_OUTPUT)&&( (ch_user == 'w')||(ch_user == 'W') ) )
                {
                    WriteToDigitalModule(&arr_digital_modules[ui_index_digital_module]);

                    break;
                }
                else if(ch_user = 27)
                {
                    break;
                }
            }
        }

        printf("Press any key to continue\n\n");

        _getch();
    }

    delete []arr_digital_modules;

    adqClose();

    printf("\n\n");

    printf("Press any key to terminate.\n");

    _getch();

    return 0;
}

void ReadFromDigitalModule(SDigitalModule* pDigitalModule)
{
    unsigned int ui_value;

    unsigned int ui_adq_error = adqReadSingle(  pDigitalModule->m_uiBoardIndex,
                                                pDigitalModule->m_uiModuleIndex,
                                                ui_value                        );

    if(ui_adq_error != ADQ_ERROR_NONE)
    {
        printf("adqReadSingle returned 0x%08X\n\n", ui_adq_error);
    }
    else if(pDigitalModule->m_uiPortWidthBits == 8)
    {
        printf("Read 0x%02X from Board index %u Module index %u\n\n", 
                (unsigned char)(ui_value & 0x000000FF), 
                pDigitalModule->m_uiBoardIndex,
                pDigitalModule->m_uiModuleIndex                     );
    }
    else if(pDigitalModule->m_uiPortWidthBits == 16)
    {
        printf("Read 0x%04X from Board index %u Module index %u\n\n", 
                (unsigned short)(ui_value & 0x0000FFFF), 
                pDigitalModule->m_uiBoardIndex,
                pDigitalModule->m_uiModuleIndex                     );
    }
    else // if(pDigitalModule->m_uiPortWidthBits == 32)
    {
        printf("Read 0x%08X from Board index %u Module index %u\n\n", 
                ui_value, 
                pDigitalModule->m_uiBoardIndex,
                pDigitalModule->m_uiModuleIndex                     );
    }
}

void WriteToDigitalModule(SDigitalModule* pDigitalModule)
{
    unsigned int ui_value;

    while(true)
    {
        printf("Write value to Board index %u Module index %u(%d bytes - RETURN to cancel): 0x",
                pDigitalModule->m_uiBoardIndex,
                pDigitalModule->m_uiModuleIndex,
                pDigitalModule->m_uiPortWidthBits / 8                                            );

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            return;
        }

        if( sscanf_s(&sz_response[0], "%x", &ui_value) != 1 )
        {
            continue;
        }

        if( ( (pDigitalModule->m_uiPortWidthBits == 8)&&(ui_value > 0x000000FF) )||
                            ( (pDigitalModule->m_uiPortWidthBits == 16)&&( ui_value > 0x0000FFFF ) ) )
        {
            continue;
        }

        break;
    }

    unsigned int ui_adq_error = adqWriteSingle( pDigitalModule->m_uiBoardIndex,
                                                pDigitalModule->m_uiModuleIndex,
                                                ui_value                        );

    if(ui_adq_error != ADQ_ERROR_NONE)
    {
        printf("adqWriteSingle returned 0x%08X\n\n", ui_adq_error);
    }
    else if(pDigitalModule->m_uiPortWidthBits == 8)
    {
        printf("Wrote 0x%02X to Board index %u Module index %u\n\n", 
                (unsigned char)(ui_value & 0x000000FF), 
                pDigitalModule->m_uiBoardIndex,
                pDigitalModule->m_uiModuleIndex                     );
    }
    else if(pDigitalModule->m_uiPortWidthBits == 16)
    {
        printf("Wrote 0x%04X to Board index %u Module index %u\n\n", 
                (unsigned short)(ui_value & 0x0000FFFF), 
                pDigitalModule->m_uiBoardIndex,
                pDigitalModule->m_uiModuleIndex                     );
    }
    else // if(pDigitalModule->m_uiPortWidthBits == 32)
    {
        printf("Wrote 0x%08X to Board index %u Module index %u\n\n", 
                ui_value, 
                pDigitalModule->m_uiBoardIndex,
                pDigitalModule->m_uiModuleIndex                     );
    }
}

