#include <windows.h>

#include <stdio.h>
#include <stdlib.h>

#include <tchar.h>

#include <conio.h>

#include "..\\adqDevice.h"

#ifdef WIN64

#pragma comment(lib, "..\\adqDevice_64.lib")

#else

#pragma comment(lib, "..\\adqDevice.lib")

#endif

struct SReadBufferModule
{
    unsigned int    m_uiBoardIndex;

    unsigned int    m_uiModuleIndex;

    unsigned int    m_uiModuleType;

    unsigned int    m_uiClockType;

    double          m_dMaxfrequencyHz;
};

int _tmain(int argc, _TCHAR* argv[])
{
   // Increase the size of the console buffer
    
    HANDLE  h_console = GetStdHandle(STD_OUTPUT_HANDLE);

    COORD coord;

    coord.X = 160;
    
    coord.Y = 9999;
    
    SetConsoleScreenBufferSize(h_console, coord);

    bool b_terminate = false;

    bool b_remote = false;

    wchar_t wsz_remote_name_or_IP_address[128];

    while(true)
    {
        printf( "1. Open local\n"
                "2. Open remote\n"
                "\n"
                "Your choice: 1 or 2 (RETURN to exit): ");

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            b_terminate = true;
            
            break;
        }

        unsigned int ui_choice = 0;

        if( sscanf_s(&sz_response[0], "%u", &ui_choice) != 1)
        {
            continue;
        }

        if(ui_choice == 1)
        {
            break;
        }

        if(ui_choice == 2)
        {
            b_remote = true;

            printf("Name or IP-Address (RETURN to exit): ");

            _getws_s(&wsz_remote_name_or_IP_address[0], 128);

            printf("\n\n");

            if( wsz_remote_name_or_IP_address[0] == (wchar_t)0 )
            {
                b_terminate = true;
            }

            break;
        }
    }

    if(b_terminate)
    {
        printf("Press any key to exit.\n");

        _getch();

        exit(-1);
    }

    unsigned int ui_adq_error;
    
    if(b_remote)
    {
        wchar_t* pwc_user_name_or_address[2] = {&wsz_remote_name_or_IP_address[0],
                                                NULL                                };

        ui_adq_error = adqOpenEx(pwc_user_name_or_address);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpenEx returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }
    else
    {
        ui_adq_error = adqOpen();

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpen returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }

    unsigned int ui_number_of_boards;
    
    ui_adq_error = adqGetNumberOfBoards(ui_number_of_boards);

    if(ui_adq_error != ADQ_ERROR_NONE)
    {
        printf("adqGetNumberOfBoards returned 0x%08X\n\n", ui_adq_error);

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    if(ui_number_of_boards == 0)
    {
        printf("No ADQ boards found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    printf("%3u ADQ Boards found\n", ui_number_of_boards);

    // Count the number of input modules

    unsigned int ui_number_of_read_buffer_modules = 0;

    for(unsigned int ui_board_index = 0; ui_board_index < ui_number_of_boards; ui_board_index++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_board_index, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned 0x%08X\n\n", ui_board_index, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_module_index = 0; ui_module_index < ui_number_of_modules; ui_module_index++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_board_index, ui_module_index, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if( (ui_module_type == ADQ_MODULE_TYPE_DI)||(ui_module_type == ADQ_MODULE_TYPE_DIO)||(ui_module_type == ADQ_MODULE_TYPE_CNTR)||(ui_module_type == ADQ_MODULE_TYPE_AI) )
            {
                ++ui_number_of_read_buffer_modules;
            }
        }
    }

    if(ui_number_of_read_buffer_modules == 0)
    {
        printf("No Modules found that support adqReadBuffer!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    unsigned int ui_max_number_of_read_buffer_modules = ui_number_of_read_buffer_modules;

    SReadBufferModule* arr_read_buffer_modules = new SReadBufferModule[ui_max_number_of_read_buffer_modules];

    ui_number_of_read_buffer_modules = 0;

    for(unsigned int ui_board_index = 0; ui_board_index < ui_number_of_boards; ui_board_index++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_board_index, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned 0x%08X\n\n", ui_board_index, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_module_index = 0; ui_module_index < ui_number_of_modules; ui_module_index++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_board_index, ui_module_index, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            SADQGeneralModuleInfo s_adq_general_module_info;

            s_adq_general_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_adq_general_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            if( (ui_module_type == ADQ_MODULE_TYPE_DI)||(ui_module_type == ADQ_MODULE_TYPE_DIO)||(ui_module_type == ADQ_MODULE_TYPE_CNTR)||(ui_module_type == ADQ_MODULE_TYPE_AI) )
            {
                unsigned int ui_info_size;

                ui_adq_error = adqGetModuleInfo(ui_board_index, ui_module_index, s_adq_general_module_info, 0, NULL, ui_info_size);

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    continue;
                }

                arr_read_buffer_modules[ui_number_of_read_buffer_modules].m_uiBoardIndex = ui_board_index;

                arr_read_buffer_modules[ui_number_of_read_buffer_modules].m_uiModuleIndex = ui_module_index;

                arr_read_buffer_modules[ui_number_of_read_buffer_modules].m_uiModuleType = ui_module_type;

                arr_read_buffer_modules[ui_number_of_read_buffer_modules].m_uiClockType = s_adq_general_module_info.m_uiClockType;

                if(s_adq_general_module_info.m_uiClockMinimumDivider != 0)
                {
                    arr_read_buffer_modules[ui_number_of_read_buffer_modules].m_dMaxfrequencyHz = s_adq_general_module_info.m_dClockBaseFrequencyHz / (double)s_adq_general_module_info.m_uiClockMinimumDivider;
                }
                else
                {
                    arr_read_buffer_modules[ui_number_of_read_buffer_modules].m_dMaxfrequencyHz = 0.0;
                }

                ++ui_number_of_read_buffer_modules;
            }
        }
    }

    while(true)
    {
        printf("Please choose a module for adqReadBuffer:\n\n");

        unsigned int ui_index_read_buffer_module;

        for(ui_index_read_buffer_module = 0; ui_index_read_buffer_module < ui_number_of_read_buffer_modules; ui_index_read_buffer_module++)
        {
            wchar_t* wsz_module_type = L"???????";

            switch(arr_read_buffer_modules[ui_index_read_buffer_module].m_uiModuleType)
            {
            case ADQ_MODULE_TYPE_DI:

                wsz_module_type = L"     DI";

                break;

            case ADQ_MODULE_TYPE_DIO:
                
                wsz_module_type = L"    DIO";

                break;

            case ADQ_MODULE_TYPE_CNTR:
                
                wsz_module_type = L"Counter";

                break;

            case ADQ_MODULE_TYPE_AI:
                
                wsz_module_type = L"     AI";

                break;
            }

            wchar_t* wsz_clock_type = L"????????";

            switch(arr_read_buffer_modules[ui_index_read_buffer_module].m_uiClockType)
            {
            case ADQ_CLOCK_TYPE_NONE:

                wsz_clock_type = L"    None";

                break;

            case ADQ_CLOCK_TYPE_SOFTWARE:

                wsz_clock_type = L"Software";

                break;

            case ADQ_CLOCK_TYPE_HARDWARE:

                wsz_clock_type = L"Hardware";

                break;
            }

            printf("%2u.  Board index: %d  Module index: %-2d  Module type: %S  Clock type: %S  Max. frequency: %8.2f Hz\n",
                   ui_index_read_buffer_module + 1,
                   arr_read_buffer_modules[ui_index_read_buffer_module].m_uiBoardIndex,
                   arr_read_buffer_modules[ui_index_read_buffer_module].m_uiModuleIndex,
                   wsz_module_type,
                   wsz_clock_type,
                   arr_read_buffer_modules[ui_index_read_buffer_module].m_dMaxfrequencyHz                          );
        }

        printf("\n");

        printf("Your choice 1 - %d (RETURN to exit): ",
               ui_number_of_read_buffer_modules         );

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            break;
        }

        if( sscanf_s(&sz_response[0], "%u", &ui_index_read_buffer_module) != 1)
        {
            continue;
        }

        if( (ui_index_read_buffer_module < 1)||(ui_index_read_buffer_module > ui_number_of_read_buffer_modules) )
        {
            continue;
        }

        --ui_index_read_buffer_module;

        unsigned int ui_board_index = arr_read_buffer_modules[ui_index_read_buffer_module].m_uiBoardIndex;

        unsigned int ui_module_index = arr_read_buffer_modules[ui_index_read_buffer_module].m_uiModuleIndex;

        bool b_terminate = false;

        double d_factor = 1.0;
        
        double d_constant = 0.0; 

        wchar_t* wsz_physical_unit = L"????";;

        switch(arr_read_buffer_modules[ui_index_read_buffer_module].m_uiModuleType)
        {
        case ADQ_MODULE_TYPE_DIO:

            // Switch to input

            ui_adq_error = adqDIOSetDirection(ui_board_index, ui_module_index, ADQ_DIO_INPUT);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u direction: Input (0x%08X) - adqDIOSetDirection returned 0x%08X\n\n", ui_board_index, ui_module_index, ADQ_DIO_INPUT, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            break;

        case ADQ_MODULE_TYPE_CNTR:

            // Ask the user for mode and start value ????

            unsigned int ui_cntr_mode;
            ui_cntr_mode = ADQ_CNTR_8254_MODE_0;

            bool b_continue;
            b_continue = false;

            while(true)
            {
                printf("Please choose a counter mode:\n\n");

                printf("0. Mode 0: Change state at zero\n");
                printf("1. Mode 1: Retriggerable \"One Shot\"\n");
                printf("2. Mode 2: Asymmetric divider\n");
                printf("3. Mode 3: Symmetric divider\n");
                printf("4. Mode 4: Counter start by software trigger\n");
                printf("5. Mode 5: Counter start by hardware trigger\n\n");
                 
                printf("Your choice 0 - 5 (RETURN to exit): ");

                char sz_response[8];

                gets_s(&sz_response[0], 8);

                printf("\n\n");

                if(sz_response[0] == 0)
                {
                    b_continue = true;
                    
                    break;
                }

                unsigned int ui_mode_index;

                if( sscanf_s(&sz_response[0], "%u", &ui_mode_index) != 1)
                {
                    continue;
                }

                if(ui_mode_index > 5)
                {
                    continue;
                }

                switch(ui_mode_index)
                {
                case 0:

                    ui_cntr_mode = ADQ_CNTR_8254_MODE_0;

                    break;

                case 1:

                    ui_cntr_mode = ADQ_CNTR_8254_MODE_1;

                    break;

                case 2:

                    ui_cntr_mode = ADQ_CNTR_8254_MODE_2;

                    break;

                case 3:

                    ui_cntr_mode = ADQ_CNTR_8254_MODE_3;

                    break;

                case 4:

                    ui_cntr_mode = ADQ_CNTR_8254_MODE_4;

                    break;

                case 5:

                    ui_cntr_mode = ADQ_CNTR_8254_MODE_5;

                    break;
                }

                break;
            }

            if(b_continue)
            {
                continue;
            }

            unsigned int ui_start_value;
            ui_start_value = 0xFFFF;

            while(true)
            {
                printf("Start value, 0 - 65535 (RETURN to cancel): ");

                char sz_response[8];

                gets_s(&sz_response[0], 8);

                printf("\n\n");

                if(sz_response[0] == 0)
                {
                    b_continue = true;
                    
                    break;
                }

                if( sscanf_s(&sz_response[0], "%u", &ui_start_value) != 1 )
                {
                    continue;
                }

                if(ui_start_value > 65535)
                {
                    continue;
                }

                break;
            }

            if(b_continue)
            {
                continue;
            }

            ui_adq_error = adqCNTRConfig(ui_board_index, ui_module_index, ADQ_CNTR_CLOCK_SOURCE_DEFAULT, ui_cntr_mode, ui_start_value);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u clock source: 0x%08X counter mode: %u start value: %u - adqCNTRConfig returned 0x%08X\n\n", ui_board_index, ui_module_index, ADQ_CNTR_CLOCK_SOURCE_DEFAULT, ui_cntr_mode, ui_start_value, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            break;

        case ADQ_MODULE_TYPE_AI:

            // Ask the user for channel and range

            SADQGeneralModuleInfo s_adq_general_module_info;

            s_adq_general_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_adq_general_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            SADQAIModuleInfo s_adq_ai_module_info;

            s_adq_ai_module_info.m_uiSize = sizeof(SADQAIModuleInfo);

            s_adq_ai_module_info.m_uiID = SADQAIModuleInfo_ID;

            unsigned int ui_info_size;

            ui_adq_error = adqGetModuleInfo(ui_board_index, ui_module_index, s_adq_general_module_info, sizeof(SADQAIModuleInfo), (unsigned char*)&s_adq_ai_module_info, ui_info_size);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if( s_adq_ai_module_info.m_bRangeInfoPerChannel )
            { 
                printf("        Board index: %3u Module index: %3u - Not all the channels have the same ranges - Program must be extended\n\n", ui_board_index, ui_module_index, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            unsigned int ui_channel_index;
            ui_channel_index = 0;

            b_continue = false;

            while(true)
            {
                printf( "The module has %u channels, please choose one 0 - %u (RETURN to exit): ",
                        s_adq_ai_module_info.m_uiNumberOfChannels,
                        s_adq_ai_module_info.m_uiNumberOfChannels - 1                               );
                 
                char sz_response[8];

                gets_s(&sz_response[0], 8);

                printf("\n\n");

                if(sz_response[0] == 0)
                {
                    b_continue = true;
                    
                    break;
                }

                if( sscanf_s(&sz_response[0], "%u", &ui_channel_index) != 1)
                {
                    continue;
                }

                if(ui_channel_index >= s_adq_ai_module_info.m_uiNumberOfChannels)
                {
                    continue;
                }

                break;
            }

            if(b_continue)
            {
                continue;
            }

            unsigned int ui_range_index;
            ui_range_index = 0;

            if(s_adq_ai_module_info.m_uiNumberOfRanges > 1)
            {
                b_continue = false;
            
                while(true)
                {
                    printf("The channel has the following input ranges:\n\n");
                 
                    for(unsigned int ui_index_range = 0; ui_index_range < s_adq_ai_module_info.m_uiNumberOfRanges; ui_index_range++)
                    {
                        wchar_t* wsz_physical_unit;

                        if(s_adq_ai_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_VOLT)
                        {
                            wsz_physical_unit = L"Volt";
                        }
                        else if(s_adq_ai_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_AMPERE)
                        {
                            wsz_physical_unit = L"Ampere";
                        }
                        else // if(s_adq_ai_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_OHM)
                        {
                            wsz_physical_unit = L"Ohm";
                        }

                        printf( "%2d.  Digital Min.: %u  Digital Max.: %u  Unit: %6S  Physical Min.: %7.4f  Physical Max.: %7.4f\n",
                                ui_index_range,
                                s_adq_ai_module_info.m_arrRanges[ui_index_range].m_uiDigitalMin,
                                s_adq_ai_module_info.m_arrRanges[ui_index_range].m_uiDigitalMax,
                                wsz_physical_unit,
                                s_adq_ai_module_info.m_arrRanges[ui_index_range].m_dPhysicalMin,
                                s_adq_ai_module_info.m_arrRanges[ui_index_range].m_dPhysicalMax                                     );
                    }

                    printf("\n");

                    printf("Your choice 0 - %u (RETURN to exit): ",
                           s_adq_ai_module_info.m_uiNumberOfRanges - 1  );

                    char sz_response[8];

                    gets_s(&sz_response[0], 8);

                    printf("\n\n");

                    if(sz_response[0] == 0)
                    {
                        b_continue = true;
                    
                        break;
                    }

                    if( sscanf_s(&sz_response[0], "%u", &ui_range_index) != 1)
                    {
                        continue;
                    }

                    if(ui_range_index >= s_adq_ai_module_info.m_uiNumberOfRanges)
                    {
                        continue;
                    }

                    break;
                }

                if(b_continue)
                {
                    continue;
                }
            }

            ui_adq_error = adqAISetChannelAndRange(ui_board_index, ui_module_index, ui_channel_index, ui_range_index);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u channel index: %u range index: %u - adqAISetChannelAndRange returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_channel_index, ui_range_index, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            d_factor = (s_adq_ai_module_info.m_arrRanges[ui_range_index].m_dPhysicalMax - s_adq_ai_module_info.m_arrRanges[ui_range_index].m_dPhysicalMin) / 
                                        (double)(s_adq_ai_module_info.m_arrRanges[ui_range_index].m_uiDigitalMax - s_adq_ai_module_info.m_arrRanges[ui_range_index].m_uiDigitalMin);
        
            d_constant = s_adq_ai_module_info.m_arrRanges[ui_range_index].m_dPhysicalMin - (double)s_adq_ai_module_info.m_arrRanges[ui_range_index].m_uiDigitalMin * d_factor; 

            if(s_adq_ai_module_info.m_arrRanges[ui_range_index].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_VOLT)
            {
                wsz_physical_unit = L"Volt";
            }
            else if(s_adq_ai_module_info.m_arrRanges[ui_range_index].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_AMPERE)
            {
                wsz_physical_unit = L"Ampere";
            }
            else // if(s_adq_ai_module_info.m_arrRanges[ui_range_index].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_OHM)
            {
                wsz_physical_unit = L"Ohm";
            }

            break;
        }

        int i_required_frequency;

        char buffer[64];

        b_terminate = false;

        while(true)
        {
            printf("Required frequency in Hz (RETURN to terminate): ");

            gets_s(&buffer[0], 64);

            printf("\n\n");
        
            if(buffer[0] == 0)
            {
                b_terminate = true;
            
                break;
            }
        
            if(sscanf_s(&buffer[0], "%d", &i_required_frequency) != 1 )
            {
                continue;
            }

            if(i_required_frequency < 1)
            {
                continue;
            }

            break;
        }

        if(b_terminate)
        {
            printf("Press any key to exit.\n");

            adqClose();

            _getch();

            exit(-1);
        }

        double d_nominal_frequency = (double)i_required_frequency;

        double d_actual_frequency;

        ui_adq_error = adqSetFrequency(ui_board_index, ui_module_index, d_nominal_frequency, d_actual_frequency);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqSetFrequency returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            adqClose();

            _getch();

            exit(-1);
        }

        printf("Nominal frequency %f Hz    Actual frequency: %f\n\n",  d_nominal_frequency, d_actual_frequency);

        unsigned int ui_number_of_values;

        while(true)
        {
            printf("Number of values (RETURN to terminate): ");

            gets_s(&buffer[0], 64);

            printf("\n\n");
        
            if(buffer[0] == 0)
            {
                b_terminate = true;
            
                break;
            }
        
            if( (sscanf_s(&buffer[0], "%u", &ui_number_of_values) != 1 ) )
            {
                continue;
            }

            if(ui_number_of_values < 1)
            {
                continue;
            }

            break;
        }

        if(b_terminate)
        {
            printf("Press any key to exit.\n");

            adqClose();

            _getch();

            exit(-1);
        }

        printf( "Press any key to acquire %u values from board index: %u, module index: %u at %f Hz\n\n"
                "You will have to wait %d seconds to see the results\n\n",
                ui_number_of_values, 
                ui_board_index, 
                ui_module_index,
                d_actual_frequency,
                (int)( (double)ui_number_of_values / d_actual_frequency + 0.5 )                               );

        _getch();

        unsigned int* p_ui_data = new unsigned int[ui_number_of_values];

        unsigned int ui_number_of_values_returned;

        ui_adq_error = adqReadBuffer(ui_board_index, ui_module_index, ui_number_of_values, p_ui_data, ui_number_of_values_returned);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqReadBuffer returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            adqClose();

            _getch();

            exit(-1);
        }

        printf( "adqReadBuffer returned %u values\n"
                "--------------------------------\n\n",
                ui_number_of_values_returned            );

        if(arr_read_buffer_modules[ui_index_read_buffer_module].m_uiModuleType == ADQ_MODULE_TYPE_AI)
        {
            for(unsigned int ui_index_value = 0; ui_index_value < ui_number_of_values_returned; ui_index_value++)
            {
                 printf("%5d.    Digital: %5u (0x%08X)    -    Physical %7.3f %S\n",
                        ui_index_value + 1,
                        p_ui_data[ui_index_value],
                        p_ui_data[ui_index_value],
                        (double)p_ui_data[ui_index_value] * d_factor + d_constant,
                        wsz_physical_unit                                           );
            }
        }
        else
        {
            for(unsigned int ui_index_value = 0; ui_index_value < ui_number_of_values_returned; ui_index_value++)
            {
                printf( "%5d.    %5u (0x%08X)\n", 
                        ui_index_value + 1,
                        p_ui_data[ui_index_value],
                        p_ui_data[ui_index_value]   );
            }
        }

        delete p_ui_data;

        printf("\n\n");

        printf("Press any key to continue\n\n");

        _getch();
    }

    printf("\n\n");

    delete []arr_read_buffer_modules;

    adqClose();

    printf("Press any key to terminate.\n");

    _getch();

    return 0;
}

