#include <windows.h>

#include <stdio.h>
#include <stdlib.h>

#include <tchar.h>

#include <conio.h>

#define _USE_MATH_DEFINES

#include <math.h>

#include "..\\adqDevice.h"

#ifdef WIN64

#pragma comment(lib, "..\\adqDevice_64.lib")

#else

#pragma comment(lib, "..\\adqDevice.lib")

#endif

struct SWriteBufferModule
{
    unsigned int    m_uiBoardIndex;

    unsigned int    m_uiModuleIndex;

    unsigned int    m_uiModuleType;

    unsigned int    m_uiClockType;

    double          m_dMaxfrequencyHz;
};

int _tmain(int argc, _TCHAR* argv[])
{
   // Increase the size of the console buffer
    
    HANDLE  h_console = GetStdHandle(STD_OUTPUT_HANDLE);

    COORD coord;

    coord.X = 160;
    
    coord.Y = 9999;
    
    SetConsoleScreenBufferSize(h_console, coord);

    bool b_terminate = false;

    bool b_remote = false;

    wchar_t wsz_remote_name_or_IP_address[128];

    while(true)
    {
        printf( "1. Open local\n"
                "2. Open remote\n"
                "\n"
                "Your choice: 1 or 2 (RETURN to exit): ");

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            b_terminate = true;
            
            break;
        }

        unsigned int ui_choice = 0;

        if( sscanf_s(&sz_response[0], "%u", &ui_choice) != 1)
        {
            continue;
        }

        if(ui_choice == 1)
        {
            break;
        }

        if(ui_choice == 2)
        {
            b_remote = true;

            printf("Name or IP-Address (RETURN to exit): ");

            _getws_s(&wsz_remote_name_or_IP_address[0], 128);

            printf("\n\n");

            if( wsz_remote_name_or_IP_address[0] == (wchar_t)0 )
            {
                b_terminate = true;
            }

            break;
        }
    }

    if(b_terminate)
    {
        printf("Press any key to exit.\n");

        _getch();

        exit(-1);
    }

    unsigned int ui_adq_error;
    
    if(b_remote)
    {
        wchar_t* pwc_user_name_or_address[2] = {&wsz_remote_name_or_IP_address[0],
                                                NULL                                };

        ui_adq_error = adqOpenEx(pwc_user_name_or_address);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpenEx returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }
    else
    {
        ui_adq_error = adqOpen();

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqOpen returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            _getch();

            exit(-1);
        }
    }

    unsigned int ui_number_of_boards;
    
    ui_adq_error = adqGetNumberOfBoards(ui_number_of_boards);

    if(ui_adq_error != ADQ_ERROR_NONE)
    {
        printf("adqGetNumberOfBoards returned 0x%08X\n\n", ui_adq_error);

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    if(ui_number_of_boards == 0)
    {
        printf("No ADQ boards found!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    printf("%3u ADQ Boards found\n", ui_number_of_boards);

    // Count the number of output modules

    unsigned int ui_number_of_write_buffer_modules = 0;

    for(unsigned int ui_board_index = 0; ui_board_index < ui_number_of_boards; ui_board_index++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_board_index, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned 0x%08X\n\n", ui_board_index, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_module_index = 0; ui_module_index < ui_number_of_modules; ui_module_index++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_board_index, ui_module_index, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if( (ui_module_type == ADQ_MODULE_TYPE_DO)||(ui_module_type == ADQ_MODULE_TYPE_DIO)||(ui_module_type == ADQ_MODULE_TYPE_AO) )
            {
                ++ui_number_of_write_buffer_modules;
            }
        }
    }

    if(ui_number_of_write_buffer_modules == 0)
    {
        printf("No Modules found that support adqWriteBuffer!\n\n");

        printf("Press any key to exit.\n");

        adqClose();

        _getch();

        exit(-1);
    }

    unsigned int ui_max_number_of_write_buffer_modules = ui_number_of_write_buffer_modules;

    SWriteBufferModule* arr_write_buffer_modules = new SWriteBufferModule[ui_max_number_of_write_buffer_modules];

    ui_number_of_write_buffer_modules = 0;

    for(unsigned int ui_board_index = 0; ui_board_index < ui_number_of_boards; ui_board_index++)
    {
        unsigned int ui_number_of_modules;

        ui_adq_error = adqGetNumberOfModules(ui_board_index, ui_number_of_modules);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("    Board index: %3u adqGetNumberOfModules returned 0x%08X\n\n", ui_board_index, ui_adq_error);

            printf("Press any key to continue.\n");

            _getch();

            continue;
        }

        for(unsigned int ui_module_index = 0; ui_module_index < ui_number_of_modules; ui_module_index++)
        {
            unsigned int ui_module_type;

            ui_adq_error = adqGetModuleType(ui_board_index, ui_module_index, ui_module_type);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleType returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            if ( (ui_module_type != ADQ_MODULE_TYPE_DO)&&(ui_module_type != ADQ_MODULE_TYPE_DIO)&&(ui_module_type != ADQ_MODULE_TYPE_AO) )
            {
                continue;
            }

            SADQGeneralModuleInfo s_adq_general_module_info;

            s_adq_general_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_adq_general_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            arr_write_buffer_modules[ui_number_of_write_buffer_modules].m_uiBoardIndex = ui_board_index;

            arr_write_buffer_modules[ui_number_of_write_buffer_modules].m_uiModuleIndex = ui_module_index;

            arr_write_buffer_modules[ui_number_of_write_buffer_modules].m_uiModuleType = ui_module_type;

            unsigned int ui_info_size;

            ui_adq_error = adqGetModuleInfo(ui_board_index, ui_module_index, s_adq_general_module_info, 0, NULL, ui_info_size);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index (DO Module): %3u adqGetModuleInfo returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            arr_write_buffer_modules[ui_number_of_write_buffer_modules].m_uiClockType = s_adq_general_module_info.m_uiClockType;

            if(s_adq_general_module_info.m_uiClockMinimumDivider != 0)
            {
                arr_write_buffer_modules[ui_number_of_write_buffer_modules].m_dMaxfrequencyHz = s_adq_general_module_info.m_dClockBaseFrequencyHz / (double)s_adq_general_module_info.m_uiClockMinimumDivider;
            }
            else
            {
                arr_write_buffer_modules[ui_number_of_write_buffer_modules].m_dMaxfrequencyHz = 0.0;
            }

            ++ui_number_of_write_buffer_modules;
        }
    }

    while(true)
    {
        printf("Please choose a module for adqWriteBuffer:\n\n");

        unsigned int ui_index_write_buffer_module;

        for(ui_index_write_buffer_module = 0; ui_index_write_buffer_module < ui_number_of_write_buffer_modules; ui_index_write_buffer_module++)
        {
            wchar_t* wsz_module_type = L"???????";

            switch(arr_write_buffer_modules[ui_index_write_buffer_module].m_uiModuleType)
            {
            case ADQ_MODULE_TYPE_DO:

                wsz_module_type = L"     DO";

                break;

            case ADQ_MODULE_TYPE_DIO:
                
                wsz_module_type = L"    DIO";

                break;

            case ADQ_MODULE_TYPE_AO:
                
                wsz_module_type = L"     AO";

                break;
            }

            wchar_t* wsz_clock_type = L"????????";

            switch(arr_write_buffer_modules[ui_index_write_buffer_module].m_uiClockType)
            {
            case ADQ_CLOCK_TYPE_NONE:

                wsz_clock_type = L"    None";

                break;

            case ADQ_CLOCK_TYPE_SOFTWARE:

                wsz_clock_type = L"Software";

                break;

            case ADQ_CLOCK_TYPE_HARDWARE:

                wsz_clock_type = L"Hardware";

                break;
            }

            printf("%2u.  Board index: %d  Module index: %-2d  Module type: %S  Clock type: %S  Max. frequency: %8.2f Hz\n",
                   ui_index_write_buffer_module + 1,
                   arr_write_buffer_modules[ui_index_write_buffer_module].m_uiBoardIndex,
                   arr_write_buffer_modules[ui_index_write_buffer_module].m_uiModuleIndex,
                   wsz_module_type,
                   wsz_clock_type,
                   arr_write_buffer_modules[ui_index_write_buffer_module].m_dMaxfrequencyHz                          );
        }

        printf("\n");

        printf("Your choice 1 - %d (RETURN to exit): ",
               ui_number_of_write_buffer_modules         );

        char sz_response[8];

        gets_s(&sz_response[0], 8);

        printf("\n\n");

        if(sz_response[0] == 0)
        {
            break;
        }

        if( sscanf_s(&sz_response[0], "%u", &ui_index_write_buffer_module) != 1)
        {
            continue;
        }

        if( (ui_index_write_buffer_module < 1)||(ui_index_write_buffer_module > ui_number_of_write_buffer_modules) )
        {
            continue;
        }

        --ui_index_write_buffer_module;

        unsigned int ui_board_index = arr_write_buffer_modules[ui_index_write_buffer_module].m_uiBoardIndex;

        unsigned int ui_module_index = arr_write_buffer_modules[ui_index_write_buffer_module].m_uiModuleIndex;

        int i_required_output_frequency;

        char buffer[64];

        b_terminate = false;

        while(true)
        {
            printf("Required output frequency in Hz (RETURN to terminate): ");

            gets_s(&buffer[0], 64);

            printf("\n\n");
        
            if(buffer[0] == 0)
            {
                b_terminate = true;
            
                break;
            }
        
            if(sscanf_s(&buffer[0], "%d", &i_required_output_frequency) != 1 )
            {
                continue;
            }

            if(i_required_output_frequency < 1)
            {
                continue;
            }

            break;
        }

        if(b_terminate)
        {
            printf("Press any key to exit.\n");

            adqClose();

            _getch();

            exit(-1);
        }

        double d_nominal_output_frequency = (double)i_required_output_frequency;

        double d_actual_output_frequency;

        ui_adq_error = adqSetFrequency(ui_board_index, ui_module_index, d_nominal_output_frequency, d_actual_output_frequency);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqSetFrequency returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            adqClose();

            _getch();

            exit(-1);
        }

        printf("Nominal output frequency %f Hz    Actual output frequency: %f\n\n",  d_nominal_output_frequency, d_actual_output_frequency);

        bool b_terminate = false;

        unsigned int* p_ui_data = NULL;

        unsigned int ui_number_of_values = 0;

        if( (arr_write_buffer_modules[ui_index_write_buffer_module].m_uiModuleType == ADQ_MODULE_TYPE_DIO)||
                                    (arr_write_buffer_modules[ui_index_write_buffer_module].m_uiModuleType == ADQ_MODULE_TYPE_DO) )
        {
            if(arr_write_buffer_modules[ui_index_write_buffer_module].m_uiModuleType == ADQ_MODULE_TYPE_DIO)
            {
                ui_adq_error = adqDIOSetDirection(ui_board_index, ui_module_index, ADQ_DIO_OUTPUT);

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("        Board index: %3u Module index: %3u direction: Input (0x%08X) - adqDIOSetDirection returned 0x%08X\n\n", ui_board_index, ui_module_index, ADQ_DIO_INPUT, ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    continue;
                }
            }

            unsigned int ui_module_min_value = 0;

            unsigned int ui_module_max_value = 0;

            SADQGeneralModuleInfo s_adq_general_module_info;

            s_adq_general_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_adq_general_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            unsigned int ui_info_size;

            if(arr_write_buffer_modules[ui_index_write_buffer_module].m_uiModuleType == ADQ_MODULE_TYPE_DIO)
            {
                SADQDIOModuleInfo s_adq_dio_module_info;

                s_adq_dio_module_info.m_uiSize = sizeof(SADQDIOModuleInfo);

                s_adq_dio_module_info.m_uiID = SADQDIOModuleInfo_ID;

                ui_adq_error = adqGetModuleInfo(ui_board_index, ui_module_index, s_adq_general_module_info, sizeof(SADQDIOModuleInfo), (unsigned char*)&s_adq_dio_module_info, ui_info_size);

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    continue;
                }

                ui_module_max_value = (00000001 << s_adq_dio_module_info.m_uiPortWidthBits) - 1;
            }
            else if(arr_write_buffer_modules[ui_index_write_buffer_module].m_uiModuleType == ADQ_MODULE_TYPE_DO)
            {
                SADQDOModuleInfo s_adq_do_module_info;

                s_adq_do_module_info.m_uiSize = sizeof(SADQDOModuleInfo);

                s_adq_do_module_info.m_uiID = SADQDOModuleInfo_ID;

                ui_adq_error = adqGetModuleInfo(ui_board_index, ui_module_index, s_adq_general_module_info, sizeof(SADQDOModuleInfo), (unsigned char*)&s_adq_do_module_info, ui_info_size);

                if(ui_adq_error != ADQ_ERROR_NONE)
                {
                    printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_adq_error);

                    printf("Press any key to continue.\n");

                    _getch();

                    continue;
                }

                ui_module_max_value = (00000001 << s_adq_do_module_info.m_uiPortWidthBits) - 1;
            }

            unsigned int ui_min_value;

            while(true)
            {
                printf("Min value %u - %u (RETURN to terminate): ",
                       ui_module_min_value,
                       ui_module_max_value - 1                      );

                gets_s(&buffer[0], 64);

                printf("\n\n");
        
                if(buffer[0] == 0)
                {
                    b_terminate = true;
            
                    break;
                }
        
                if( (sscanf_s(&buffer[0], "%u", &ui_min_value) != 1 ) )
                {
                    continue;
                }

                if( (ui_min_value < ui_module_min_value)||(ui_min_value > ui_module_max_value) )
                {
                    continue;
                }

                break;
            }

            if(b_terminate)
            {
                printf("Press any key to exit.\n");

                adqClose();

                _getch();

                exit(-1);
            }

            unsigned int ui_max_value;

            while(true)
            {
                printf("Max value %u - %u (RETURN to terminate): ",
                       ui_min_value + 1,
                       ui_module_max_value                          );

                gets_s(&buffer[0], 64);

                printf("\n\n");
        
                if(buffer[0] == 0)
                {
                    b_terminate = true;
            
                    break;
                }
        
                if( (sscanf_s(&buffer[0], "%u", &ui_max_value) != 1 ) )
                {
                    continue;
                }

                if( (ui_max_value < ui_min_value)||(ui_max_value > ui_module_max_value) )
                {
                    continue;
                }

                break;
            }

            if(b_terminate)
            {
                printf("Press any key to exit.\n");

                adqClose();

                _getch();

                exit(-1);
            }

            int b_count_up = 0;

            int b_count_down = 0;

            while(true)
            {
                printf("1) Count up from %u to %u\n"
                       "2) Count down from %u to %u\n"
                       "3) Count up from %u to %u then down from %u to %u\n\n"
                       "Your choice: 1, 2 or 3 (RETURN to terminate): ",
                       ui_min_value, ui_max_value,
                       ui_max_value, ui_min_value,
                       ui_min_value, ui_max_value, ui_max_value, ui_min_value);

                char ch_input = _getch();

                printf("%c\n\n", ch_input);
        
                if(ch_input == 13) // RETURN 
                {
                    b_terminate = true;
            
                    break;
                }
        
                if(ch_input == '1')
                {
                    b_count_up = 1;

                    break;
                }
                else if(ch_input == '2')
                {
                    b_count_down = 1;

                    break;
                }
                else if(ch_input == '3')
                {
                    b_count_up = 1;

                    b_count_down = 1;

                    break;
                }
            }

            if(b_terminate)
            {
                printf("Press any key to exit.\n");

                adqClose();

                _getch();

                exit(-1);
            }

            ui_number_of_values = ui_max_value - ui_min_value + 1;

            if( (b_count_up != 0)&&(b_count_down != 0) )
            {
                ui_number_of_values+= ui_max_value - ui_min_value;
            }

            p_ui_data = new unsigned int[ui_number_of_values];

            if(b_count_up != 0)
            {
                for(unsigned int ui_index = 0; ui_index < ui_max_value - ui_min_value + 1; ui_index++)
                {
                    p_ui_data[ui_index] = ui_min_value + ui_index;
                }

                if(b_count_down != 0)
                {
                    for(unsigned int ui_index = ui_max_value - ui_min_value + 1; ui_index < ui_number_of_values; ui_index++)
                    {
                        p_ui_data[ui_index] = 2 * ui_max_value - ui_min_value - ui_index;
                    }
                }
            }
            else // if(b_count_down != 0)
            {
                for(unsigned int ui_index = 0; ui_index < ui_max_value - ui_min_value + 1; ui_index++)
                {
                    p_ui_data[ui_index] = ui_max_value - ui_index;
                }
            }
        }
        else if(arr_write_buffer_modules[ui_index_write_buffer_module].m_uiModuleType == ADQ_MODULE_TYPE_AO)
        {
            SADQGeneralModuleInfo s_adq_general_module_info;

            s_adq_general_module_info.m_uiSize = sizeof(SADQGeneralModuleInfo);

            s_adq_general_module_info.m_uiID = SADQGeneralModuleInfo_ID;

            SADQAOModuleInfo s_adq_ao_module_info;

            s_adq_ao_module_info.m_uiSize = sizeof(SADQAOModuleInfo);

            s_adq_ao_module_info.m_uiID = SADQAOModuleInfo_ID;

            unsigned int ui_info_size;

            ui_adq_error = adqGetModuleInfo(ui_board_index, ui_module_index, s_adq_general_module_info, sizeof(SADQAOModuleInfo), (unsigned char*)&s_adq_ao_module_info, ui_info_size);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u adqGetModuleInfo returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

            b_terminate = false;

            unsigned int ui_range_index;
            ui_range_index = 0;

            wchar_t* wsz_physical_unit = L"????";

            if(s_adq_ao_module_info.m_uiNumberOfRanges > 1)
            {
                while(true)
                {
                    printf("The channel has the following output ranges:\n\n");
                 
                    for(unsigned int ui_index_range = 0; ui_index_range < s_adq_ao_module_info.m_uiNumberOfRanges; ui_index_range++)
                    {
                        if(s_adq_ao_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_VOLT)
                        {
                            wsz_physical_unit = L"Volt";
                        }
                        else if(s_adq_ao_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_AMPERE)
                        {
                            wsz_physical_unit = L"Ampere";
                        }
                        else // if(s_adq_ao_module_info.m_arrRanges[ui_index_range].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_OHM)
                        {
                            wsz_physical_unit = L"Ohm";
                        }

                        printf( "%2d.  Digital Min.: %u  Digital Max.: %u  Unit: %6S  Physical Min.: %7.4f  Physical Max.: %7.4f\n",
                                ui_index_range,
                                s_adq_ao_module_info.m_arrRanges[ui_index_range].m_uiDigitalMin,
                                s_adq_ao_module_info.m_arrRanges[ui_index_range].m_uiDigitalMax,
                                wsz_physical_unit,
                                s_adq_ao_module_info.m_arrRanges[ui_index_range].m_dPhysicalMin,
                                s_adq_ao_module_info.m_arrRanges[ui_index_range].m_dPhysicalMax                                     );
                    }

                    printf("\n");

                    printf("Your choice 0 - %u (RETURN to exit): ",
                           s_adq_ao_module_info.m_uiNumberOfRanges - 1  );

                    char sz_response[8];

                    gets_s(&sz_response[0], 8);

                    printf("\n\n");

                    if(sz_response[0] == 0)
                    {
                        b_terminate = true;
                    
                        break;
                    }

                    if( sscanf_s(&sz_response[0], "%u", &ui_range_index) != 1)
                    {
                        continue;
                    }

                    if(ui_range_index >= s_adq_ao_module_info.m_uiNumberOfRanges)
                    {
                        continue;
                    }

                    break;
                }
            }


            if(b_terminate)
            {
                printf("Press any key to exit.\n");

                adqClose();

                _getch();

                exit(-1);
            }

/* ????

            ui_adq_error = adqAOSetRange(ui_board_index, ui_module_index, ui_range_index);

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("        Board index: %3u Module index: %3u channel index: %u range index: %u - adqAISetChannelAndRange returned 0x%08X\n\n", ui_board_index, ui_module_index, ui_channel_index, ui_range_index, ui_adq_error);

                printf("Press any key to continue.\n");

                _getch();

                continue;
            }

*/

            double d_factor = (double)(s_adq_ao_module_info.m_arrRanges[ui_range_index].m_uiDigitalMax - s_adq_ao_module_info.m_arrRanges[ui_range_index].m_uiDigitalMin) / 
                                                        (s_adq_ao_module_info.m_arrRanges[ui_range_index].m_dPhysicalMax - s_adq_ao_module_info.m_arrRanges[ui_range_index].m_dPhysicalMin);
                                        ;
        
            double d_constant = (double)s_adq_ao_module_info.m_arrRanges[ui_range_index].m_uiDigitalMin - s_adq_ao_module_info.m_arrRanges[ui_range_index].m_dPhysicalMin * d_factor; 

            if(s_adq_ao_module_info.m_arrRanges[ui_range_index].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_VOLT)
            {
                wsz_physical_unit = L"Volt";
            }
            else if(s_adq_ao_module_info.m_arrRanges[ui_range_index].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_AMPERE)
            {
                wsz_physical_unit = L"Ampere";
            }
            else // if(s_adq_ao_module_info.m_arrRanges[ui_range_index].m_uiPhysicalUnit == ADQ_PHYSICAL_UNIT_OHM)
            {
                wsz_physical_unit = L"Ohm";
            }

            double d_physical_min = s_adq_ao_module_info.m_arrRanges[ui_range_index].m_dPhysicalMin;

            double d_physical_max = s_adq_ao_module_info.m_arrRanges[ui_range_index].m_dPhysicalMax;

            double ui_digital_min = s_adq_ao_module_info.m_arrRanges[ui_range_index].m_uiDigitalMin;

            double ui_digital_max = s_adq_ao_module_info.m_arrRanges[ui_range_index].m_uiDigitalMax;

            double d_signal_frequency_from;

            double d_signal_physical_min_from;

            double d_signal_physical_max_from;

            double d_signal_frequency_to;

            double d_signal_physical_min_to;

            double d_signal_physical_max_to;

            double d_signal_length_seconds;

            // ???? The values which are simply set below should really be entered by the user

           d_signal_frequency_from = 1.0;

           d_signal_physical_min_from = -10.0;

           d_signal_physical_max_from = +10.0;

           d_signal_frequency_to = 5.0;

           d_signal_physical_min_to = -5.0;

           d_signal_physical_max_to = +5.0;

           d_signal_length_seconds = 10.0;

            ui_number_of_values = (unsigned int)( d_signal_length_seconds * d_actual_output_frequency );

            p_ui_data = new unsigned int[ui_number_of_values];

            for(unsigned int ui_index_data = 0; ui_index_data < ui_number_of_values; ui_index_data++)
            {
                double d_time = (double)ui_index_data / d_actual_output_frequency;

                double d_signal_frequency = d_signal_frequency_from + d_time * (d_signal_frequency_to - d_signal_frequency_from) / d_signal_length_seconds; 

                double d_signal_physical_min = d_signal_physical_min_from + d_time * (d_signal_physical_min_to - d_signal_physical_min_from) / d_signal_length_seconds; 

                double d_signal_physical_max = d_signal_physical_max_from + d_time * (d_signal_physical_max_to - d_signal_physical_max_from) / d_signal_length_seconds; 

                double d_value = (d_signal_physical_max + d_signal_physical_min) / 2.0 + (d_signal_physical_max - d_signal_physical_min) / 2.0 * sin(M_PI_2 * d_time * d_signal_frequency);

                p_ui_data[ui_index_data] = (unsigned int)(d_value * d_factor + d_constant);
            }
        }

        unsigned int ui_repeat_buffer;

        while(true)
        {
            printf("Repeat buffer 1 - 1000 (RETURN to terminate): ");

            gets_s(&buffer[0], 64);

            printf("\n\n");
        
            if(buffer[0] == 0)
            {
                b_terminate = true;
            
                break;
            }
        
            if( (sscanf_s(&buffer[0], "%u", &ui_repeat_buffer) != 1 ) )
            {
                continue;
            }

            if( (ui_repeat_buffer < 1)||(ui_repeat_buffer > 1000) )
            {
                continue;
            }

            break;
        }

        if(b_terminate)
        {
            printf("Press any key to exit.\n");

            adqClose();

            _getch();

            exit(-1);
        }

        printf( "Press any key to output the signal %u times on  board index: %u, module index: %u at %f Hz\n\n"
                "You will have to wait %d seconds for the operation to finish\n\n",
                ui_repeat_buffer, 
                ui_board_index, 
                ui_module_index,
                d_actual_output_frequency,
                (int)( (double)ui_number_of_values * ui_repeat_buffer / d_actual_output_frequency + 0.5 )              );

        _getch();

        unsigned int ui_number_of_values_written;

        ui_adq_error = adqWriteBuffer(ui_board_index, ui_module_index, ui_number_of_values, p_ui_data, ui_repeat_buffer, ui_number_of_values_written);

        if(ui_adq_error != ADQ_ERROR_NONE)
        {
            printf("adqWriteBuffer returned 0x%08X\n\n", ui_adq_error);

            printf("Press any key to exit.\n");

            adqClose();

            _getch();

            exit(-1);
        }

        printf( "adqWriteBuffer wrote %u values\n"
                "--------------------------------\n\n",
                ui_number_of_values_written            );

        for( ; ; )
        {
            unsigned int ui_number_of_values_acceptable = 0;

            bool b_running;

            ui_adq_error = adqWriteContinuousStatus(ui_board_index, ui_module_index, false, ui_number_of_values_acceptable, b_running); 

            if(ui_adq_error != ADQ_ERROR_NONE)
            {
                printf("adqWriteContinuousStatus returned 0x%08X\n\n", ui_adq_error);

                printf("Press any key to exit.\n");

                adqClose();

                _getch();

                exit(-1);
            }

            if(b_running)
            {
                printf("Output is running - Room for %u data values in the internal buffer\n\n", ui_number_of_values_acceptable);
            }
            else
            {
                break;
            }

            Sleep(1000);
        }

        if(p_ui_data != NULL)
        {
            delete p_ui_data;
        }

        printf("\n\n");

        printf("Output terminated - Press any key to continue\n\n");

        _getch();
    }

    printf("\n\n");

    delete[] arr_write_buffer_modules;

    adqClose();

    printf("Press any key to terminate.\n");

    _getch();

    return 0;
}

