//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the adqWriteContinuousCallback project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

#include "stdafx.h"
#include "AdqCommon.h"

#include "..\\..\\..\\adqDevice.h"

#include <cmath>
#include <iomanip>
#include <sstream>

namespace ALLDAQ
{
    using std::wstring;

    wstring RemoveTrailingZeros(const wstring &str);

    wstring FormatPhysicalValue(double physicalValue, const wstring &unitSymbol)
    {
        wstring valueStr = L"";
        double tmpVal = abs(physicalValue);
        if (tmpVal == 0)
        {
            valueStr = L"0";
        }
        else if (tmpVal < 0.000000001)
        {
            valueStr = FormatDoubleValue(physicalValue * 1000000000000) + L"p";
        }
        else if (tmpVal < 0.000001)
        {
            valueStr = FormatDoubleValue(physicalValue * 1000000000) + L"n";
        }
        else if (tmpVal < 0.001)
        {
            valueStr = FormatDoubleValue(physicalValue * 1000000) + L"µ";
        }
        else if (tmpVal < 1)
        {
            valueStr = FormatDoubleValue(physicalValue * 1000) + L"m";
        }
        else if (tmpVal >= 1000000000000)
        {
            valueStr = FormatDoubleValue(physicalValue / 1000000000000) + L"T";
        }
        else if (tmpVal >= 1000000000)
        {
            valueStr = FormatDoubleValue(physicalValue / 1000000000) + L"G";
        }
        else if (tmpVal >= 1000000)
        {
            valueStr = FormatDoubleValue(physicalValue / 1000000) + L"M";
        }
        else if (tmpVal >= 1000)
        {
            valueStr = FormatDoubleValue(physicalValue / 1000) + L"k";
        }
        else
        {
            valueStr = FormatDoubleValue(physicalValue);
        }
        return valueStr + unitSymbol;
    }

    wstring FormatDoubleValue(double physicalValue, std::streamsize precision /*= 8*/)
    {
        std::wstringstream wss;
        wss << std::fixed << std::setprecision(precision) << physicalValue;

        return RemoveTrailingZeros(wss.str());
    }

    wstring RemoveTrailingZeros(const wstring &str)
    {
        wstring s = str;
        size_t pos = s.find_last_not_of(L"0");
        if (pos != std::wstring::npos)
        {
            s.erase(pos + 1);
        }

        pos = s.find_last_not_of(L".");
        if (pos != std::wstring::npos)
        {
            s.erase(pos + 1);
        }
        return s;
    }

    wstring GetPhysicalUnitSymbol(unsigned int physicalUnit)
    {
        wstring s;
        switch (physicalUnit)
        {
            case ADQ_PHYSICAL_UNIT_VOLT:
                s = L"V";
                break;
            case ADQ_PHYSICAL_UNIT_AMPERE:
                s = L"A";
                break;
            default:
                s = L"Unknown physical unit";
                break;
        }
        return s;
    }


    wstring GetModelName(const wstring &boardName)
    {
        size_t posStart = boardName.find(L"ADQ");
        size_t posEnd = wstring::npos;

        if (posStart != wstring::npos)
        {
            posEnd = boardName.find(L" ", posStart);
        }
        else
        {
            posStart = 0;
        }

        if (posEnd == wstring::npos)
        {
            posEnd = boardName.length();
        }

        return boardName.substr(posStart, posEnd - posStart);
    }
}