//-----------------------------------------------------------------------------
// ALLDAQ, a division of ALLNET GmbH
// https://www.alldaq.com
//
// Purpose:
//       This file is part of the adqWriteContinuousCallback project of the ALLDAQ-SDK.  
//       It shows the usage of the API functions to access
//       ALLDAQ Data Acquisition Boards
// 
//-----------------------------------------------------------------------------

#pragma once

#include "AoModule.h"

#include <string>
#include <vector>
#include <memory>

class ModuleLocator
{
public:
    ModuleLocator(std::vector<std::shared_ptr<AoModule>> &aoModules);
    virtual ~ModuleLocator();
    void InitModuleLocator();
    std::vector<std::wstring> GetErrorInfo();
private:
    unsigned int m_adqOpenError;
    std::vector<std::shared_ptr<AoModule>> &m_aoModules;
    std::vector<std::wstring> m_errorInfo;
    unsigned int OpenAdqDriver();
    void FindAoModules();
    unsigned int GetNumberOfBoards();
    std::wstring GetBoardName(unsigned int boardIndex);
    unsigned int GetModules(unsigned int boardIndex, unsigned int numberOfModules);
    unsigned int GetModuleInfo(const std::wstring &boardName, unsigned int boardIndex, unsigned int moduleIndex);
    bool IsClockControlledModule(unsigned int clockType);
    void AddErrorInfo(const wchar_t *functionName, unsigned int adqError);
    void CloseAdqDriver();
};

